<?php
/*
 * ----------------------------------------------------------------------------
 * Download.php                                                      12/30/2008
 * This script will allow a web user to download a specified file from a
 * database entry.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2008 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

$GLOBALS['VA_ACTION'] = 'Downloading a File';

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');

$classDatabases = new vadvDatabases;
$classNetworks = new vadvNetworks;
$classUserfileDat = new datUserfile;
$classStatVBBSDat = new datStatVBBS;

// Check the passed parameters. This script will look directly at the database
// to determine the filename instead of passing the filename and path directly.
// Set the passed variables.
// Topic.
if (isset($_POST['t'])) {
    $t = htmlspecialchars($_POST['t']);
} else {
    die('<br /><h4>Invalid message parameters. Please press the BACK button on your browser.</h4>');
}

// Database.
if (isset($_POST['d'])) {
    $d = (integer) $_POST['d'];
} else {
    die('<br /><h4>Invalid message parameters. Please press the BACK button on your browser.</h4>');
}

// Message.
if (isset($_POST['m'])) {
    $m = (integer) $_POST['m'];
} else {
    die('<br /><h4>Invalid message parameters. Please press the BACK button on your browser.</h4>');
}

// Check to see if this page was referred by another page, and check to see where
// that page was at.
if ((!isset($_SERVER['HTTP_REFERER'])) ||
    (strpos($_SERVER['HTTP_REFERER'], $_SERVER['HTTP_HOST']) === FALSE)) {
    die('<br /><h4>You are not allowed to access this script from an outside source.<br />Press the BACK button on your browser.</h4>');
}

if ($t == 'email') {
    if (($GLOBALS['USER']['userno'] == 0) ||
        ($GLOBALS['USER']['userno'] == $GLOBALS['CONFIG']['guestacct'])) {
        die ('<br /><h4>You do not have access to download this file. Press the BACK button on your browser.</h4>');
    }
    $db = $classDatabases->SetEmailInfo($GLOBALS['USER']['userno']);
} else {
    // Retrieve the database information.
    $db = $classDatabases->GetDbInfo($t, $d);
}

// Retrieve the message information.
$msgInfo = $classDatabases->GetMessageInfo($db, $m);

// Check and make sure that the user has access to the database or is the
// recepient of the email message.
if ($t == 'email') {
    if ($msgInfo['touserno'] != $GLOBALS['USER']['userno']) {
        die ('<br /><h4>You do not have access to download this file. Press the BACK button on your browser.</h4>');
    }
} else {
    if (($db['minreadsl'] > $GLOBALS['USER']['securitylevel']) && ($GLOBALS['USER']['securitylevel'] < 255)) {
        die ('<br /><h4>You do not have access to download this file. Press the BACK button on your browser.</h4>');
    }
}

$filename = $msgInfo['attfilename'];
if ($filename != '') {
    // If a local attachment.
    if ($msgInfo['attfilepath'] != '') {
        $filepath = str_replace('/', "\\", $msgInfo['attfilepath']);
        $pos = strrpos($filepath, "\\");
        $filepath = substr($filepath, 0, $pos);
    } else {
        $filepath = $db['filepath'];
    }
    if ($filepath != '') {
        if ($GLOBALS['CONFIG']['filehost']) {
            $file = ConvertDir($GLOBALS['CONFIG']['filehost'], $filepath) . '/' . $filename;
        } else {
            $file = $filepath . '/' . $filename;
        }
    } else {
        $file = $GLOBALS['VA_SYSPATHS']['sysop'] . '/' . $filename;
    }
} else {
    die('<br /><h4>File is missing. Press the BACK button on your browser.</h4>');
}

if (!file_exists($file)) {
    die('<br /><h4>File is missing. Press the BACK button on your browser.</h4>');
}

// Get long filename.
$realpath = realpath($file);
if ($realpath !== FALSE) {
    $file = $realpath;
    $filename = basename($realpath);
}

if ((IsLoggedIn()) &&
    ($t != 'email')) {
    // Check to see if the user has enough credits.
    $credits = $GLOBALS['USER']['credits'] - (intval(filesize($file) / 1024) * $GLOBALS['VA_MAIN']['credperdlk']);
    if (($GLOBALS['VA_MAIN']['credperdlk'] > 0) &&
        ($credits < 0)) {
        die('<br /><h4>You do not have enough credits to download this file!<br />To gain more credits, try uploading new files or posting in the message area.<br />Press the BACK button on your browser.</h4>');
    } else {
        // Subtract from the user's credits.
        if (($GLOBALS['USER']['userno'] > 0) &&
            ($credits > 0)) {
            $GLOBALS['USER']['credits'] = $GLOBALS['USER']['credits'] - $credits;
            $fileString = $classUserfileDat->Read();
            $fileString = $classUserfileDat->Put($fileString, $GLOBALS['USER']['userno'], $GLOBALS['USER']);
            $classUserfileDat->Write($fileString);
        }
    }
}

// Transfer the file.
$transfer = SendFile($file);

if (!$transfer) {
    die('<br /><h4>An error occurred during the transfer. Press the BACK button on your browser.</h4>');
} else {
    // If it transferred, then update the database information.
    if ((IsLoggedIn()) &&
        ($t != 'email')) {
        // Update user information.
        $GLOBALS['USER']['dlfiles']++;
        $GLOBALS['USER']['dlkb'] = $GLOBALS['USER']['dlkb'] + intval($msgInfo['attfilesize'] / 1024);
        $fileString = $classUserfileDat->Read();
        $fileString = $classUserfileDat->Put($fileString, $GLOBALS['USER']['userno'], $GLOBALS['USER']);
        $classUserfileDat->Write($fileString);
    }

    // Update file information.
    $msgInfo['downloadno']++;
    $classDatabases->WriteMessageInfo($db, $m, $msgInfo);

    // Update BBS statistics.
    $fileString = $classStatVBBSDat->Read();
    $stats = $classStatVBBSDat->Get($fileString);
    $stats['downloadstotal']++;
    $stats['downloadstoday']++;
    $fileString = $classStatVBBSDat->Put($fileString, $stats);
    $classStatVBBSDat->Write($fileString);

    // Log the download.
    $log = 'File Download   "' . $GLOBALS['USER']['handle'] . '" <-- ' . $file;
    WriteLog('VADV-PHP.LOG', $log);

    // Send a oneliner to the uploader.
    // Steve Winn #1 @1931000 Downloaded ADEX11VA.ZIP
    $vnet = $classNetworks->GetVnetInfo();
    $oneliner = $GLOBALS['USER']['handle'] . ' #' . $GLOBALS['USER']['userno'] . ' @' . $vnet['systemid'] . ' Downloaded ' . $filename;
    $classDatabases->WriteOneliner($msgInfo['fromuserno'], $oneliner);
}

?>