<?php
/*
 * ----------------------------------------------------------------------------
 * Email-ViewMsg.php                                                 01/06/2022
 * This script will display the selected email message to the user.
 * The variable $m is passed to the script to set the message number.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2022 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

$GLOBALS['VA_ACTION'] = 'Reading Email';
$pagetitle = 'E-Mail Area';

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');

$classDatabases = new vadvDatabases;
$classNetworks = new vadvNetworks;
$classUserfileDat = new datUserfile;

define('NOINDEX', true);

// Get network information.
$VA_NETWORKS = $classNetworks->GetNetworkInfo();

if (($GLOBALS['USER']['userno'] == 0) ||
    ($GLOBALS['USER']['userno'] == $GLOBALS['CONFIG']['guestacct']) ||
    ($GLOBALS['USER']['securitylevel'] == 0)) {
    die(VADV_LoginError());
}

$userno = $GLOBALS['USER']['userno'];

// Set the passed variables.
// Ansi.
if (isset($_GET['ansi'])) {
    $ansi = (integer) $_GET['ansi'];
} else {
    $ansi = 0;
}

// Color.
if (isset($_GET['color'])) {
    $color = (integer) $_GET['color'];
} else {
    $color = 0;
}

// Html.
if (isset($_GET['html'])) {
    $html = (integer) $_GET['html'];
} else {
    $html = 0;
}

// Routing.
if (isset($_GET['routing'])) {
    $routing = (integer) $_GET['routing'];
} else {
    $routing = 0;
}

// Emoticons.
if (isset($_GET['emoticons'])) {
    $emoticons = (integer) $_GET['emoticons'];
} else {
    $emoticons = 0;
}

// Encoding.
if (isset($_GET['encoding'])) {
    $encoding = (integer) $_GET['encoding'];
} else {
    $encoding = 0;
}

// Retrieve the database information.
$db = $classDatabases->SetEmailInfo($userno);


// -------- BEGIN PAGE PROPER --------

// Retrieve the number of messages.
$msgno = $db['msgno'];
if ($msgno == 0) {
    die('<br /><h4>Invalid message. Please press the BACK button on your browser.</h4>');
}

// Retrieve the number of user messages.
if ($GLOBALS['USER_SETTINGS']['hidedeleted'] == 1) {
    $msgstat = $classDatabases->GetEmailCount($userno, 0);
} else {
    $msgstat = $classDatabases->GetEmailCount($userno);
}
$msgcnt = $msgstat['msgno'];
if ($msgcnt == 0) {
    die('<br /><h4>Invalid message. Please press the BACK button on your browser.</h4>');
}

// Message.
if (isset($_GET['m'])) {
    $m = (integer) $_GET['m'];
} else {
    $m = $msgcnt;
}
if ($m > $msgcnt) {
    $m = $msgcnt;
}
if ($m <= 0) {
    $m = 1;
}

// Get the message information.
$msgInfo = $classDatabases->GetMessageInfo($db, $msgstat[$m - 1]);

if ($msgInfo['touserno'] != $userno) {
    die('<br /><h4>You do not have access to this message. Press the BACK button on your browser.</h4>');
}

// Get the message to display.
$message = $classDatabases->GetMessage($db, $m, 2, '', $msgInfo);
$origMessage = $message;

// Apply options.
if ($html == 0) {
    $message = rtrim(str_replace('&amp;', '&', htmlspecialchars($message)));

    if ($message == '') {
        $message = rtrim(preg_replace("/(=20)||(=09)/", '', strip_tags($origMessage)));
    }
} else {
    $message = rtrim(preg_replace("/(=20)||(=09)/", '', strip_tags($message)));
}
if ($routing == 0) {
    $message = StripRouting($message);
}
if ($color == 0) {
    $message = StripBBSCodes($message);
}
if ($ansi == 0) {
    $message = StripANSICodes($message);
}
if ($encoding == 0) {
    $message = HTMLEncode($message, $GLOBALS['USER_SETTINGS']['htmlencode']);
}
if ($emoticons == 0) {
    $message = convert_bbcodes($message);
}
$message = HTMLcp437($message);

// Check for delete message flag.
if (isset($_GET['delete'])) {
    if ($_GET['delete'] == 1) {
        // If an attached file, then delete it. Non-reversible!
        if ($msgInfo['attfilename'] != '') {
            if ($GLOBALS['CONFIG']['hostname'] != '') {
                $file = ConvertDir($GLOBALS['CONFIG']['hostname'], $db['filepath']) . '/' . $msgInfo['attfilename'];
            } else {
                $file = $db['filepath'] . '/' . $msgInfo['attfilename'];
            }

            if ($msgInfo['attfilepath'] == '') {
                if (file_exists($file)) {
                    unlink($file);
                }
            } else {
                if ($GLOBALS['CONFIG']['hostname'] != '') {
                    $userDir = ConvertUnc($GLOBALS['VA_SYSPATHS']['sysop'] . '/' . $GLOBALS['USER']['userno'] . '/' . $msgInfo['attfilename']);
                } else {
                    $userDir = $GLOBALS['VA_SYSPATHS']['sysop'] . '/' . $GLOBALS['USER']['userno'] . '/' . $msgInfo['attfilename'];
                }
                if ($msgInfo['attfilepath'] == $userDir) {
                    if ($GLOBALS['CONFIG']['hostname'] != '') {
                        $file = ConvertDir($GLOBALS['CONFIG']['hostname'], $msgInfo['attfilepath']);
                    } else {
                        $file = $msgInfo['attfilepath'];
                    }
                }
                if (file_exists($file)) {
                    unlink($file);
                }
            }
            $msgInfo['attfilesize'] = 0;
            $msgInfo['attfilename'] = '';
            $msgInfo['attfilepath'] = '';
        }
        $msgInfo['deleted'] = 1;
        $classDatabases->WriteMessageInfo($db, $msgstat[$m - 1], $msgInfo);
        RedirectUser('email-dispmsg.php');
    } else if ($_GET['delete'] == 0) {
        $msgInfo['deleted'] = 0;
        $classDatabases->WriteMessageInfo($db, $msgstat[$m - 1], $msgInfo);
    }
}

$dateTime = ConvertDateTime($msgInfo['creation'], 'F j, Y');

// Check for any files.
$filename = $msgInfo['attfilename'];
if ($filename != '') {
    // If a local attachment.
    if ($msgInfo['attfilepath'] != '') {
        $filepath = str_replace('/', "\\", $msgInfo['attfilepath']);
        $pos = strrpos($filepath, "\\");
        $filepath = substr($filepath, 0, $pos);
    } else {
        $filepath = $db['filepath'];
    }
    if ($GLOBALS['CONFIG']['filehost'] != '') {
        $file = ConvertDir($GLOBALS['CONFIG']['hostname'], $filepath) . '/' . $filename;
    } else {
        $file = $filepath . '/' . $filename;
    }
} else {
    $file = '';
}

if ($file != '') {
    if (!file_exists($file)) {
        $filesize = '0 bytes';
    } else {
        $realpath = realpath($file);
        if ($realpath !== FALSE) {
            $file = $realpath;
            $filename = basename($realpath);
        }

        $filesize = filesize($file);
        if ($filesize >= 1073741824) {
            $filesize = round($filesize / 1073741824, 1) . 'GB';
        } else if ($filesize >= 1048576) {
            $filesize = round($filesize / 1048576, 1) . 'MB';
        } else if ($filesize >= 1024) {
            $filesize = intval($filesize / 1024) . 'KB';
        } else {
            $filesize = $filesize . ' bytes';
        }
    }
} else {
    $filesize = '0 bytes';
}

if (($file != '') && ($filesize > 0)) {
    $filemd5 = hash_file($GLOBALS['CONFIG']['fileencryption'], $file);
} else {
    $filemd5 = '';
}

$from = $msgInfo['fromhandle'];
if ($from == '') {
    $from = $msgInfo['fromuserno'];
}
$char = substr($from, 0, 1);
if ($char == '<') {
    $from = substr($from, 1);
}
if ($char == '"') {
    $pos = strpos($from, '"', 1);
    $from = substr($from, 1, $pos - 1);
}
$from = htmlspecialchars(trim($from));
if ($msgInfo['fromnetid'] != 1) {
    $from = $from . '@' . $msgInfo['fromnetnode'] . '*' . $msgInfo['fromnetid'];
}

$subject = htmlspecialchars(trim(StripASCIICodes(StripANSICodes(StripBBSCodes($msgInfo['subject'])))));
if ($subject == '') {
    $subject = '(No Subject)';
}

$to = $msgInfo['tohandle'];
if ($to == '') {
    $to = $msgInfo['touserno'];
}
$char = substr($to, 0, 1);
if ($char == '<') {
    $to = substr($to, 1);
}
if ($char == '"') {
    $pos = strpos($to, '"', 1);
    $to = substr($to, 1, $pos - 1);
}
$to = htmlspecialchars(trim($to));
if ($msgInfo['tonetid'] != 1) {
    $to = $to . '@' . $msgInfo['tonetnode'] . '*' . $msgInfo['tonetid'];
}

// If a supported image, then get its info.
if ((extension_loaded('gd')) &&
    ($msgInfo['deleted'] == 0) &&
    ($filesize > 0)) {
    $image = getimagesize($file);
    if ($image !== FALSE) {
        if ((extension_loaded('exif')) &&
            (($image[2] == 2) ||
             ($image[2] == 7))) {
            $extimage = exif_read_data($file, 0, TRUE);
            if ($extimage !== FALSE) {
                $message .= "\r\n\r\n---\r\n" . '<b>Image Information</b>';
                $message .= "\r\n" . 'Dimensions:    ' . $image[0] . ' x ' . $image[1];
                if (isset($extimage['EXIF']['DateTimeOriginal'])) {
                    $imagedate = str_replace(':', '/', substr($extimage['EXIF']['DateTimeOriginal'], 0, 10));
                    $imagetime = substr($extimage['EXIF']['DateTimeOriginal'], 11);
                    $timestamp = strtotime($imagedate . ' ' . $imagetime);
                    if ($timestamp > 0) {
                        $message .= "\r\n" . 'Date Created:  ' . date("l, F j, Y g:ia", $timestamp);
                    }
                }
                if (isset($extimage['FILE']['FileDateTime'])) {
                    $message .= "\r\n" . 'Date Modified: ' . date("l, F j, Y g:ia", $extimage['FILE']['FileDateTime']);
                }
                if ((isset($extimage['IFD0']['Make'])) &&
                    (isset($extimage['IFD0']['Model']))) {
                    $message .= "\r\n" . 'Camera Model:  ' . $extimage['IFD0']['Make'] . ' ' . $extimage['IFD0']['Model'];
                }
                if (isset($extimage['COMPUTED']['Copyright'])) {
                    $message .= "\r\n" . 'Copyright:     ' . $extimage['COMPUTED']['Copyright'];
                }
                $message .= "\r\n" . 'Filesize:      ' . $filesize;
            } else {
                $message = $message . "\r\n\r\n---\r\n" . 'Image Resolution: ' . $image[0] . ' x ' . $image[1];
            }
        } else {
            $message = $message . "\r\n\r\n---\r\n" . 'Image Resolution: ' . $image[0] . ' x ' . $image[1];
        }
    }
} else {
    $image = FALSE;
}

// If a MP3 file, then get its info.
if (($msgInfo['offline'] == 0) &&
    ($msgInfo['deleted'] == 0) &&
    ($filesize > 0) &&
    ($image == FALSE) &&
    (strtolower(substr($file, strlen($file) - 4)) == '.mp3')) {
    $id3 = ReadID3Info($file);
    if ($id3['tag'] == 'TAG') {
        $message .= "\r\n\r\n---\r\n" . '<b>MP3 Information</b>';
        $message .= "\r\n" . 'Song Name: ' . ucwords(strtolower($id3['songname']));
        $message .= "\r\n" . 'Artist: ' . ucwords(strtolower($id3['artist']));
        $message .= "\r\n" . 'Album: ' . ucwords(strtolower($id3['album']));
        $message .= "\r\n" . 'Year: ' . $id3['year'];
        $message .= "\r\n" . 'Genre: ' . $id3['genre'];
    }
}

// Set up Prev/Next links
$prevnum = 0;
$nextnum = 0;
if ($m > 1) {
    $prevnum = $m - 1;
}
if ($m < $msgcnt) {
    $nextnum = $m + 1;
}

$backlink = '';
if ($m >= $msgcnt - 25) {
    $backlink = 'href="email-dispmsg.php"';
} else if ($m <= 25) {
    $backlink = 'href="email-dispmsg.php?m=25"';
} else {
    $backlink = 'href="email-dispmsg.php?m=' . $m . '"';
}

// Update the message as read if not already marked and send oneliner.
if ($msgInfo['offline'] == 0) {
    $msgInfo['offline'] = 1;
    $classDatabases->WriteMessageInfo($db, $msgstat[$m - 1], $msgInfo);

    if ($msgInfo['fromnetid'] == 1) {
        $msg = $GLOBALS['USER']['handle'] . ' #' . $GLOBALS['USER']['userno'] . ' Read Email: ' . $subject;
        $classDatabases->WriteOneliner($msgInfo['fromuserno'], $msg);
    }
}

// -----------------------------------

// Output header.
include($themepath . '/templates/header.php');

// Output page
include($themepath . '/templates/viewemail.php');

// Output footer.
include($themepath . '/templates/footer.php');

?>