<?php
/*
 * ----------------------------------------------------------------------------
 * Files-DispDb.php                                                  08/26/2018
 * This script will display the databases of the specified topic.
 * The variable $t is passed to the script to set the topic number.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2018 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

// Useful if a user hits the back button.
session_cache_limiter('nocache');

$GLOBALS['VA_ACTION'] = 'Listing Files';
$pagetitle = 'File Area';

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');

$classDatabases = new vadvDatabases;
$classUserfileDat = new datUserfile;

// Set the passed variables.
// Topic.
if (isset($_GET['t'])) {
    $t = (integer) $_GET['t'];
} else {
    $t = 10;
}

if ($t < 1) {
    $t = 1;
}

// Get the topic information.
$topic = $classDatabases->GetTopicInfo($t, 'F');

// If this is an invalid section, display an error message.
if (!$topic) {
    die('<br /><h4>Invalid topic. Please press the BACK button on your browser.</h4>');
}
// Determine if the user has access to this section.
if ($topic['dbno'] == 0) {
    die('<br /><h4>You do not have access to this section. Please press the BACK button on your browser.</h4>');
}

$topicDesc = htmlspecialchars(trim($topic['desc']));
if ($topicDesc == '') {
    $topicDesc = '(No Topic Description)';
}

// Set the fileString variable.
$file = $GLOBALS['VA_SYSPATHS']['main'] . '/DATABASE.CFG';
if (!file_exists($file)) {
    die('<br /><h4>Requested configuration file is missing. Please press the BACK button on your browser.</h4>');
}

// Read the file.
$fileString = ReadCfgFile($file);

$flag = 1;
$filepost = array();
$j = 0;
for ($i = 1; $flag <= $topic['dbno']; $i++) {
    // Retrieve the database information.
    $db = $classDatabases->GetDbInfo($t, $i, $fileString);

    if ($classDatabases->DbAccess($db)) {
        $flag++;
        $RSSaccess = $classDatabases->RssAccess($db);

        // Retrieve the last post information of the database.
        $last = $classDatabases->GetLastPostInfo($db);
        $dateTime = ConvertDateTime($last['creation']);

        $filepost[$j] = $db;
        $filepost[$j]['lastFrom'] = $last['fromhandle'];
        $filepost[$j]['lastDate'] = $dateTime['date'];
        $filepost[$j]['lastTime'] = $dateTime['time'];
        $filepost[$j]['lastTZ'] = $dateTime['tz'];

        if (strlen($db['desc']) >= 31) {
            $db['desc'] = $db['desc'] . '...';
        }

        // If there is a moderator, get the handle.
        $moderator = '';
        if ($db['moderator'] != 0) {
            $user = $classUserfileDat->Get($db['moderator']);
            $moderator = htmlspecialchars(trim($user['handle']));
        }

        // Get the number of messages in the database.
        $msgno = $db['msgno'];
        if (!$msgno) {
            $msgno = 0;
        }
        $filepost[$j]['msgno'] = $db['msgno'];

         if ($dateTime['time'] != 0) {
             $from = $last['fromhandle'];
             $pos = strpos($from, '@');
             if (is_int($pos)) {
                 $from = substr($from, 0, $pos);
             }
             $char = substr($from, 0, 1);
             if ($char == '<') {
                 $from = substr($from, 1);
             }
             if ($char == '"') {
                 $pos = strpos($from, '"', 1);
                 $from = substr($from, 1, $pos - 1);
             }
             $from = htmlspecialchars(trim($from));
             $filepost[$j]['from'] = $from;
         }


        $dbDesc = htmlspecialchars(trim($db['desc']));
        if ($db['desc'] == '') {
            $dbDesc = '(No Description)';
        }

        // New messages detected, update pointer.
        $newMessage = 0;
        if (($GLOBALS['CONFIG']['newfilescan'] >= 0) &&
            ($db['devicetype'] == 0)) {
            $msgInfo = $classDatabases->GetMessageInfo($db, $db['msgcnt']);
            if (($msgInfo['timestamp'] > 0) &&
                ($msgInfo['daysold'] <= $GLOBALS['CONFIG']['newfilescan'])) {
                $newMessage = 1;
            }
        }

        $filepost[$j]['newMessage'] = $newMessage;
        $filepost[$j]['desc'] = $dbDesc;
        $filepost[$j]['moderator'] = $moderator;
        $j++;
    }
}

// Output header
include($themepath . '/templates/header.php');

// Output page
include($themepath . '/templates/filedispdb.php');

// Output footer
include($themepath . '/templates/footer.php');

?>