<?php
/*
 * ----------------------------------------------------------------------------
 * Files-DispMsg.php                                                 04/05/2014
 * This script will display a list of messages that are available in the
 * selected database.
 * The variable $t is passed to the script to set the topic number.
 * The variable $d is passed to the script to set the database number.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2014 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

// Useful if a user hits the back button.
session_cache_limiter('nocache');

$GLOBALS['VA_ACTION'] = 'Listing Files';
$pagetitle = 'File Area';

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');

$classDatabases = new vadvDatabases;

// Set the passed variables.
// Topic.
if (isset($_GET['t'])) {
    $t = (integer) $_GET['t'];
} else {
    $t = 10;
}

// Database.
if (isset($_GET['d'])) {
    $d = (integer) $_GET['d'];
} else {
    $d = 1;
}

if ($t < 1) {
    $t = 1;
}
if ($d < 1) {
    $d = 1;
}

// Retrieve the topic information.
$topic = $classDatabases->GetTopicInfo($t, 'F');

// If this is an invalid section, display an error message.
if (!$topic) {
    die('<br /><h4>Invalid topic. Please press the BACK button on your browser.</h4>');
}

// Retrieve the database information.
$db = $classDatabases->GetDbInfo($t, $d);

// Determine if the user has access to this section.
if (!$classDatabases->DbAccess($db)) {
    die('<br /><h4>You do not have access to this section. Please press the BACK button on your browser.</h4>');
}

$topicDesc = htmlspecialchars(trim($topic['desc']));
if ($topicDesc == '') {
    $topicDesc = '(No Topic Description)';
}

$dbDesc = htmlspecialchars(trim($db['desc']));
if ($dbDesc == '') {
    $dbDesc = '(No Description)';
}

if (strlen($dbDesc) >= 31) {
    $dbDesc = $dbDesc . '...';
}

// Retrieve the number of messages.
$msgno = $db['msgno'];

// Message.
if (isset($_GET['m'])) {
    $m = (integer) $_GET['m'];
} else {
    $m = $msgno;
}
if ($m > $msgno) {
    $m = $msgno;
}
if ($m <= 0) {
    $m = 1;
}

// Set up Prev/Next links
$prevlink = false;
$prevnum = 0;
$prevcnt = 0;
$nextlink = false;
$nextnum = 0;
$nextcnt = 0;
if ($m > 25) {
    $prevnum = $m - 25;
    $prevcnt = 25;
    if ($prevnum < 25) {
        $prevcnt = $prevnum;
    }
    $prevlink = true;
}

if ($m < $msgno) {
    $nextnum = $m + 25;
    if ($nextnum > $msgno) {
        $nextnum = $msgno;
    }
    $nextcnt = 25;
    if (($msgno - $m) < 25) {
        $nextcnt = ($msgno - $m);
    }
    $nextlink = true;
}

// Set the rss url variable.
$rssurl = GetCurrentUrl(0, 1, 0) . '/forum-rss.php?t=' . $t . '&d=' . $d;
$rsstitle = $GLOBALS['VA_MAIN']['bbsname'] . ' - ' . $dbDesc;

// Set the fileString variable.
$file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $db['filename'] . '.DAT';
if (!file_exists($file)) {
    die('<br /><h4>Requested database is missing. Please press the BACK button on your browser.</h4>');
}

$nofiles = 0;
$filemessages = array();

// Read the file.
$fileString = ReadCfgFile($file);

// Run the loop that will display the messages.
for ($i = $m; $i >= ($m - 24); $i--) {
    if ($msgno == 0) {
        $nofiles = 1;
        break;
    }

    if ($i <= 0) {
        break;
    }

    // Get the message information.
    $msgInfo = $classDatabases->GetMessageInfo($db, $i, $fileString);
    $dateTime = ConvertDateTime($msgInfo['creation']);
    $filemessages[$i] = array();

    $filemessages[$i]['deleted'] = $msgInfo['deleted'];
    $filemessages[$i]['daysold'] = $msgInfo['daysold'];
    $filemessages[$i]['date'] = $dateTime['date'];
    $filemessages[$i]['time'] = $dateTime['time'];
    $filemessages[$i]['tz'] = $dateTime['tz'];

    // Check for email address.
    $from = $msgInfo['fromhandle'];
    $pos = strpos($from, '@');
    if (is_int($pos)) {
        $from = substr($from, 0, $pos);
    }
    $char = substr($from, 0, 1);
    if ($char == '<') {
        $from = substr($from, 1);
    }
    if ($char == '"') {
        $pos = strpos($from, '"', 1);
        $from = substr($from, 1, $pos - 1);
    }
    $filemessages[$i]['from'] = htmlspecialchars(trim($from));

    $filemessages[$i]['subject'] = htmlspecialchars(trim(StripASCIICodes(StripANSICodes(StripBBSCodes($msgInfo['subject'])))));
    if ($filemessages[$i]['subject'] == '') {
        $filemessages[$i]['subject'] = '(No Description)';
    }

    $filename = $msgInfo['attfilename'];
    if ($filename != '') {
        if ($GLOBALS['CONFIG']['filehost'] != '') {
            $file = ConvertDir($GLOBALS['CONFIG']['hostname'], $db['filepath']) . '/' . $filename;
        } else {
            $file = $db['filepath'] . '/' . $filename;
        }
    } else {
        $file = '';
    }

    if ($file != '') {
        if (!file_exists($file)) {
            $filesize = '0 bytes';
        } else {
            $realpath = realpath($file);
            if ($realpath !== FALSE) {
                $file = $realpath;
                $filename = basename($realpath);
            }

            $filesize = filesize($file);
            if ($filesize >= 1073741824) {
                $filesize = round($filesize / 1073741824, 1) . 'GB';
            } else if ($filesize >= 1048576) {
                $filesize = round($filesize / 1048576, 1) . 'MB';
            } else if ($filesize >= 1024) {
                $filesize = intval($filesize / 1024) . 'KB';
            } else {
                $filesize = $filesize . ' bytes';
            }
        }
    } else {
        $filesize = '0 bytes';
    }
    $filemessages[$i]['filename'] = $filename;
    $filemessages[$i]['filesize'] = $filesize;
}

// Output header
include($themepath . '/templates/header.php');

// Output page
include($themepath . '/templates/filedispmsg.php');

// Output footer
include($themepath .'/templates/footer.php');

?>