<?php
/*
 * ----------------------------------------------------------------------------
 * Files-ViewMsg.php                                                 01/06/2022
 * This script will display the selected message to the user.
 * The variable $t is passed to the script to set the topic number.
 * The variable $d is passed to the script to set the database number.
 * The variable $m is passed to the script to set the message number.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2014 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

// Useful if a user hits the back button.
session_cache_limiter('nocache');

$GLOBALS['VA_ACTION'] = 'Listing Files';
$pagetitle = 'File Area';

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');

$classDatabases = new vadvDatabases;
$classNetworks = new vadvNetworks;
$classUserfileDat = new datUserfile;

define('NOINDEX', true);

$VA_NETWORKS = $classNetworks->GetNetworkInfo();

// Set the passed variables.
// Topic.
if (isset($_GET['t'])) {
    $t = (integer) $_GET['t'];
} else {
    $t = 10;
}

// Database.
if (isset($_GET['d'])) {
    $d = (integer) $_GET['d'];
} else {
    $d = 1;
}

// Ansi.
if (isset($_GET['ansi'])) {
    $ansi = (integer) $_GET['ansi'];
} else {
    $ansi = 0;
}

// Color.
if (isset($_GET['color'])) {
    $color = (integer) $_GET['color'];
} else {
    $color = 0;
}

// Html.
if (isset($_GET['html'])) {
    $html = (integer) $_GET['html'];
} else {
    $html = 0;
}

// Routing.
if (isset($_GET['routing'])) {
    $routing = (integer) $_GET['routing'];
} else {
    $routing = 0;
}

// Emoticons.
if (isset($_GET['emoticons'])) {
    $emoticons = (integer) $_GET['emoticons'];
} else {
    $emoticons = 0;
}

// Encoding.
if (isset($_GET['encoding'])) {
    $encoding = (integer) $_GET['encoding'];
} else {
    $encoding = 0;
}

if ($t < 1) {
    $t = 1;
}
if ($d < 1) {
    $d = 1;
}

// Retrieve the topic information.
$topic = $classDatabases->GetTopicInfo($t, 'F');

// If this is an invalid section, display an error message.
if ($topic == '') {
    die('<br /><h4>Invalid topic. Please press the BACK button on your browser.</h4>');
}

// Retrieve the database information.
$db = $classDatabases->GetDbInfo($t, $d);

// Retrieve the number of messages.
$msgno = $db['msgno'];
if ($msgno == 0) {
    die('<br /><h4>Invalid file. Please press the BACK button on your browser.</h4>');
}

// Determine if the user has access to this section.
if (!$classDatabases->DbAccess($db)) {
    die('<br /><h4>You do not have access to this section. Please press the BACK button on your browser.</h4>');
}

$topicDesc = htmlspecialchars(trim($topic['desc']));
if ($topicDesc == '') {
    $topicDesc = '(No Topic Description)';
}

$dbDesc = htmlspecialchars(trim($db['desc']));
if ($dbDesc == '') {
    $dbDesc = '(No Description)';
}

if (strlen($dbDesc) >= 31) {
    $dbDesc = $dbDesc . '...';
}

// Message.
if (isset($_GET['m'])) {
    $m = (integer) $_GET['m'];
} else {
    $m = $msgno;
}
if ($m > $msgno) {
    $m = $msgno;
}
if ($m <= 0) {
    $m = 1;
}

// Set the rss url variable.
$rssurl = GetCurrentUrl(0, 1, 0) . '/forum-rss.php?t=' . $t . '&d=' . $d;
$rsstitle = $GLOBALS['VA_MAIN']['bbsname'] . ' - ' . $dbDesc;

// Get the message information.
$msgInfo = $classDatabases->GetMessageInfo($db, $m);

// Get the message to display.
$message = $classDatabases->GetMessage($db, $m, 2, '', $msgInfo);
$origMessage = $message;

// Appy options.
if ($html == 0) {
    $message = rtrim(str_replace('&amp;', '&', htmlspecialchars($message)));

    if ($message == '') {
        $message = rtrim(preg_replace("/(=20)||(=09)/", '', strip_tags($origMessage)));
    }
} else {
    $message = rtrim(preg_replace("/(=20)||(=09)/", '', strip_tags($message)));
}
if ($routing == 0) {
    $message = StripRouting($message);
}
if ($color == 0) {
    $message = StripBBSCodes($message);
}
if ($ansi == 0) {
    $message = StripANSICodes($message);
}
if ($encoding == 0) {
    $message = HTMLEncode($message, $GLOBALS['USER_SETTINGS']['htmlencode']);
}
if ($emoticons == 0) {
    $message = convert_bbcodes($message);
}
$message = HTMLcp437($message);



// Set up Prev/Next links
$prevnum = 0;
$nextnum = 0;
if ($m > 1) {
    $prevnum = $m - 1;
}
if ($m < $msgno) {
    $nextnum = $m + 1;
}


$dateTime = ConvertDateTime($msgInfo['creation']);

// Check for any files.
$filename = $msgInfo['attfilename'];
if ($filename != '') {

    if ($db['filepath'] != '') {

        if ($GLOBALS['CONFIG']['filehost'] != '') {
            $file = ConvertDir($GLOBALS['CONFIG']['hostname'], $db['filepath']) . '/' . $filename;
        } else {
            $file = $db['filepath'] . '/' . $filename;
        }
    } else {
        $file = $GLOBALS['VA_SYSPATHS']['sysop'] . '/' . $filename;
    }
} else {
    $file = '';
}

if ($file != '') {

    if (!file_exists($file)) {
        $filesize = '0 bytes';
    } else {
        $realpath = realpath($file);

        if ($realpath !== FALSE) {
            $file = $realpath;
            $filename = basename($realpath);
        }

        $filesize = filesize($file);

        if ($filesize >= 1073741824) {
            $filesize = round($filesize / 1073741824, 1) . 'GB';
        } else if ($filesize >= 1048576) {
            $filesize = round($filesize / 1048576, 1) . 'MB';
        } else if ($filesize >= 1024) {
            $filesize = intval($filesize / 1024) . 'KB';
        } else {
            $filesize = $filesize . ' bytes';
        }
    }
} else {
    $filesize = '0 bytes';
}

if (($file != '') && ($filesize > 0)) {
    $filemd5 = hash_file($GLOBALS['CONFIG']['fileencryption'], $file);
} else {
    $filemd5 = '';
}

// Check for email address.
$from = $msgInfo['fromhandle'];
$pos = strpos($from, '@');

if (is_int($pos)) {
    $from = substr($from, 0, $pos);
}
$char = substr($from, 0, 1);

if ($char == '<') {
    $from = substr($from, 1);
}
if ($char == '"') {
    $pos = strpos($from, '"', 1);
    $from = substr($from, 1, $pos - 1);
}
$from = htmlspecialchars(trim($from));

$subject = htmlspecialchars(trim(StripASCIICodes(StripANSICodes(StripBBSCodes($msgInfo['subject'])))));
if ($subject == '') {
    $subject = '(No Description)';
}

// If a supported image, then get its info.
if ((extension_loaded('gd')) && ($msgInfo['offline'] == 0) && ($msgInfo['deleted'] == 0) && ($filesize > 0)) {
    $image = getimagesize($file);

    if ($image !== FALSE) {

        if ((extension_loaded('exif')) && (($image[2] == 2) || ($image[2] == 7))) {
            $extimage = exif_read_data($file, 0, TRUE);

            if ($extimage !== FALSE) {
                $message .= "\r\n\r\n---\r\n" . '<b>Image Information</b>';
                $message .= "\r\n" . 'Dimensions:    ' . $image[0] . ' x ' . $image[1];

                if (isset($extimage['EXIF']['DateTimeOriginal'])) {
                    $imagedate = str_replace(':', '/', substr($extimage['EXIF']['DateTimeOriginal'], 0, 10));
                    $imagetime = substr($extimage['EXIF']['DateTimeOriginal'], 11);
                    $timestamp = strtotime($imagedate . ' ' . $imagetime);

                    if ($timestamp > 0) {
                        $message .= "\r\n" . 'Date Created:  ' . date("l, F j, Y g:ia", $timestamp);
                    }
                }
                if (isset($extimage['FILE']['FileDateTime'])) {
                    $message .= "\r\n" . 'Date Modified: ' . date("l, F j, Y g:ia", $extimage['FILE']['FileDateTime']);
                }
                if ((isset($extimage['IFD0']['Make'])) && (isset($extimage['IFD0']['Model']))) {
                    $message .= "\r\n" . 'Camera Model:  ' . $extimage['IFD0']['Make'] . ' ' . $extimage['IFD0']['Model'];
                }
                if (isset($extimage['COMPUTED']['Copyright'])) {
                    $message .= "\r\n" . 'Copyright:     ' . $extimage['COMPUTED']['Copyright'];
                }
                $message .= "\r\n" . 'Filesize:      ' . $filesize;
            } else {
                $message = $message . "\r\n\r\n---\r\n" . 'Image Resolution: ' . $image[0] . ' x ' . $image[1];
            }
        } else {
            $message = $message . "\r\n\r\n---\r\n" . 'Image Resolution: ' . $image[0] . ' x ' . $image[1];
        }
    }
} else {
    $image = FALSE;
}

// If a MP3 file, then get its info.
if (($msgInfo['offline'] == 0) && ($msgInfo['deleted'] == 0) && ($filesize > 0) && ($image == FALSE) && (strtolower(substr($file, strlen($file) - 4)) == '.mp3')) {
    $id3 = ReadID3Info($file);

    if ($id3['tag'] == 'TAG') {
        $message .= "\r\n\r\n---\r\n" . '<b>MP3 Information</b>';
        $message .= "\r\n" . 'Song Name: ' . ucwords(strtolower($id3['songname']));
        $message .= "\r\n" . 'Artist: ' . ucwords(strtolower($id3['artist']));
        $message .= "\r\n" . 'Album: ' . ucwords(strtolower($id3['album']));
        $message .= "\r\n" . 'Year: ' . $id3['year'];
        $message .= "\r\n" . 'Genre: ' . $id3['genre'];
    }
}

// Output header
include($themepath . '/templates/header.php');

// Output page
include($themepath . '/templates/fileviewmsg.php');

// Output footer
include($themepath . '/templates/footer.php');

?>