<?php
/*
 * ----------------------------------------------------------------------------
 * Forum-DispMsg.php                                                 04/06/2014
 * This script will display a list of messages that are available in the
 * selected database.
 * The variable $t is passed to the script to set the topic number.
 * The variable $d is passed to the script to set the database number.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2014 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

// Useful if a user hits the back button.
session_cache_limiter('nocache');

$GLOBALS['VA_ACTION'] = 'Reading Messages';
$pagetitle = 'Message Forums';

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');

$classDatabases = new vadvDatabases;

// Set the passed variables.
// Topic.
if (isset($_GET['t'])) {
    $t = (integer) $_GET['t'];
} else {
    $t = 1;
}

// Database.
if (isset($_GET['d'])) {
    $d = (integer) $_GET['d'];
} else {
    $d = 1;
}

if ($t < 1) {
    $t = 1;
}
if ($d < 1) {
    $d = 1;
}

// Retrieve the topic information.
$topictypes = 'M';
if (($GLOBALS['USER']['securitylevel'] >= 255) &&
    ($GLOBALS['CONFIG']['showcustomdbs'])) {
    $topictypes = 'MO';
}
$topic = $classDatabases->GetTopicInfo($t, $topictypes);

// If this is an invalid section, display an error message.
if (!$topic) {
    die('<br /><h4>Invalid topic. Please press the BACK button on your browser.</h4>');
}

// Retrieve the database information.
$db = $classDatabases->GetDbInfo($t, $d);

// Determine if the user has access to this section.
if (!$classDatabases->DbAccess($db)) {
    die('<br /><h4>You do not have access to this section. Please press the BACK button on your browser.</h4>');
}

// Update message pointers.
if ((isset($_POST['msgupd'])) && ($_POST['msgupd'] == 1)) {
    $classDatabases->WriteMessagePointer($db, $GLOBALS['USER']['userno'], $db['msgno']);
}

$topicDesc = htmlspecialchars(trim($topic['desc']));
if ($topicDesc == '') {
    $topicDesc = '(No Topic Description)';
}

$dbDesc = htmlspecialchars(trim($db['desc']));
if ($dbDesc == '') {
    $dbDesc = '(No Description)';
}

if (strlen($dbDesc) >= 31) {
    $dbDesc = $dbDesc . '...';
}

if (IsLoggedIn()) {
    // Get the message pointer.
    $msgPointer = $classDatabases->GetMessagePointer($db, $GLOBALS['USER']['userno']);
    if ($msgPointer == 32760) {
        $msgPointer = $db['msgno'];
    }
} else {
    $msgPointer = $db['msgno'];
}

// Retrieve the number of messages.
$msgcnt = $db['msgno'];

// Message.
if (isset($_GET['m'])) {
    $m = (integer) $_GET['m'];
} else {
    $m = $msgcnt;
}
if ($m > $msgcnt) {
    $m = $msgcnt;
}
if ($m <= 0) {
    $m = 1;
}

$prevnum = 0;
$nextnum = 0;
$prevcnt = 0;
$nextcnt = 0;

if ($m > 25) {
    $prevnum = $m - 25;
    $prevcnt = 25;
    if ($prevnum < 25) {
        $prevcnt = $prevnum;
    }
}

if ($m < $msgcnt) {
    $nextnum = $m + 25;
    if ($nextnum > $msgcnt) {
        $nextnum = $msgcnt;
    }
    $nextcnt = 25;
    if (($msgcnt - $m) < 25) {
        $nextcnt = ($msgcnt - $m);
    }
}

// Set the rss url variable.
$rssurl = GetCurrentUrl(0, 1, 0) . '/forum-rss.php?t=' . $t . '&d=' . $d;
$rsstitle = $GLOBALS['VA_MAIN']['bbsname'] . ' - ' . $dbDesc;

// Set the fileString variable.
$file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $db['filename'] . '.DAT';
if (!file_exists($file)) {
    die('<br /><h4>Requested database is missing. Please press the BACK button on your browser.</h4>');
}

$nomsgs = 0;
$forummessages = array();

// Read the file.
$fileString = ReadCfgFile($file);

// Run the loop that will display the messages.
for ($i = $m; $i >= ($m - 24); $i--) {
    if ($msgcnt == 0) {
        $nomsgs = 1;
        break;
    }

    if ($i <= 0) {
        break;
    }

    // Get the message information.
    $msgInfo = $classDatabases->GetMessageInfo($db, $i, $fileString);
    $dateTime = ConvertDateTime($msgInfo['creation'], 'F j, Y');
    $forummessages[$i] = array();

    $forummessages[$i]['deleted'] = $msgInfo['deleted'];
    $forummessages[$i]['attfilename'] = $msgInfo['attfilename'];
    $forummessages[$i]['date'] = $dateTime['date'];
    $forummessages[$i]['time'] = $dateTime['time'];
    $forummessages[$i]['tz'] = $dateTime['tz'];

    // Check for email address.
    $forummessages[$i]['from'] = $msgInfo['fromhandle'];
    $pos = strpos($forummessages[$i]['from'], '@');
    if (is_int($pos)) {
        $forummessages[$i]['from'] = substr($forummessages[$i]['from'], 0, $pos);
    }
    $char = substr($forummessages[$i]['from'], 0, 1);
    if ($char == '<') {
        $forummessages[$i]['from'] = substr($forummessages[$i]['from'], 1);
    }
    if ($char == '"') {
        $pos = strpos($forummessages[$i]['from'], '"', 1);
        $forummessages[$i]['from'] = substr($forummessages[$i]['from'], 1, $pos - 1);
    }
    $forummessages[$i]['from'] = htmlspecialchars(trim($forummessages[$i]['from']));

    // Check for email address.
    $forummessages[$i]['to'] = $msgInfo['tohandle'];
    $pos = strpos($forummessages[$i]['to'], '@');
    if (is_int($pos)) {
        $forummessages[$i]['to'] = substr($forummessages[$i]['to'], 0, $pos);
    }
    $char = substr($forummessages[$i]['to'], 0, 1);
    if ($char == '<') {
        $forummessages[$i]['to'] = substr($forummessages[$i]['to'], 1);
    }
    if ($char == '"') {
        $pos = strpos($forummessages[$i]['to'], '"', 1);
        $forummessages[$i]['to'] = substr($forummessages[$i]['to'], 1, $pos - 1);
    }
    $forummessages[$i]['to'] = htmlspecialchars(trim($forummessages[$i]['to']));

    $forummessages[$i]['subject'] = htmlspecialchars(trim(StripASCIICodes(StripANSICodes(StripBBSCodes($msgInfo['subject'])))));
    if ($forummessages[$i]['subject'] == '') {
        $forummessages[$i]['subject'] = '(No Subject)';
    }

    /*$pos = strpos($forummessages[$i]['to'], '@');
    if (is_int($pos)) {
        $forummessages[$i]['to'] = substr($forummessages[$i]['to'], 0, $pos);
    }*/
}

// Output header
include($themepath . '/templates/header.php');

// Output page
include($themepath . '/templates/dispmsgs.php');

// Output footer
include($themepath . '/templates/footer.php');

?>