<?php
/*
 * ----------------------------------------------------------------------------
 * Forum-RSS.php                                                     04/06/2014
 * This script will display a list of messages in RSS format that are available
 * in the selected database.
 * The variable $t is passed to the script to set the topic number.
 * The variable $d is passed to the script to set the database number.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2014 Rusty Sand (razor) & Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

// Useful if a user hits the back button.
session_cache_limiter('nocache');

$GLOBALS['VA_ACTION'] = 'RSS';

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');

$classDatabases = new vadvDatabases;

// Set the passed variables.
// Topic.
if (isset($_GET['t'])) {
    $t = (integer) $_GET['t'];
} else {
    $t = 1;
}

// Database.
if (isset($_GET['d'])) {
    $d = (integer) $_GET['d'];
} else {
    $d = 1;
}

if ($t < 1) {
    $t = 1;
}
if ($d < 1) {
    $d = 1;
}

// Retrieve the topic information.
$topictypes = 'M';
if (($GLOBALS['USER']['securitylevel'] >= 255) &&
    ($GLOBALS['CONFIG']['showcustomdbs'])) {
    $topictypes = 'MO';
}
$topic = $classDatabases->GetTopicInfo($t, $topictypes);

// If this is an invalid section, display an error message.
if (!$topic) {
    die('<br /><h4>Invalid topic. Please press the BACK button on your browser.</h4>');
}

// Retrieve the database information.
$db = $classDatabases->GetDbInfo($t, $d);

// Determine if the user has access to this section.
if ((!$GLOBALS['CONFIG']['rssfeed']) ||
    (!$classDatabases->RssAccess($db))) {
    die('<br /><h4>You do not have access to this section. Please press the BACK button on your browser.</h4>');
}

$topicDesc = htmlspecialchars(trim($topic['desc']));
if ($topicDesc == '') {
    $topicDesc = '(No Topic Description)';
}

$dbDesc = htmlspecialchars(trim($db['desc']));
if ($dbDesc == '') {
    $dbDesc = '(No Description)';
}

// Set the fileString variable.
$file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $db['filename'] . '.DAT';
if (!file_exists($file)) {
    die('<br /><h4>Requested database is missing. Please press the BACK button on your browser.</h4>');
}

// Read the file.
$fileString = ReadCfgFile($file);

// Get base url.
$baseUrl = GetCurrentURL(0, 1) . '/';

// Display RSS
//@date_default_timezone_set("GMT");
header("Content-Type: application/xml; charset=ISO-8859-1");
$writer=new XMLWriter();
$writer->openURI('php://output');
$writer->startDocument('1.0');
$writer->setIndent(4);

// Declare it as an rss document.
$writer->startElement('rss');
$writer->writeAttribute('version', '2.0');
$writer->writeAttribute('xmlns:atom', 'http://www.w3.org/2005/Atom');
$writer->startElement('channel');
$writer->writeElement('title', $GLOBALS['VA_MAIN']['bbsname'] . ' - ' . $dbDesc);
$writer->writeElement('description', $GLOBALS['VA_MAIN']['bbsname']);
$writer->writeElement('link', $baseUrl . 'forum-dispmsg.php?t=' . $t . '&d=' . $d);
$writer->writeElement('pubDate', date("r"));

// Setup counter variables.
$firstmsg = $db['msgno'];
$lastmsg = $db['msgno'] - $GLOBALS['CONFIG']['rssfeedlimit'] + 1;
if ($lastmsg < 1) {
    $lastmsg = 1;
}

for ($i = $firstmsg; $i >= $lastmsg; $i--) {
    // Get the message information.
    $msgInfo = $classDatabases->GetMessageInfo($db, $i, $fileString);
    $dateTime = ConvertDateTime($msgInfo['creation'], 'r');

    $from = $msgInfo['fromhandle'] . ' @' . $msgInfo['fromnetnode'] . ' *' . $msgInfo['fromnetid'];
    $to = $msgInfo['tohandle'] . ' @' . $msgInfo['tonetnode'] . ' *' . $msgInfo['tonetid'];

    $subject = htmlspecialchars(trim(StripANSICodes(StripBBSCodes($msgInfo['subject']))));
    if ($subject == '') {
        $subject = '(No Subject)';
    }

    // Create header.
    if (strtoupper($db['dbtype']) == 'F') {
        $header = 'Uploader:    ' . $from . "\r\n";
        $header .= 'Description: ' . $subject . "\r\n";
        $header .= "\r\n";
    } else {
        $header = 'From:    ' . $from . "\r\n";
        $header .= 'To:      ' . $to . "\r\n";
        $header .= 'Subject: ' . $subject . "\r\n";
        $header .= "\r\n";
    }

    // Get the message to display.
    $message = $classDatabases->GetMessage($db, $i, 2, '', $msgInfo);
    $message = rtrim(str_replace('&amp;', '&', htmlspecialchars($message)));
    //$message = rtrim(preg_replace("/(=20)||(=09)/", '', strip_tags($message)));
    $message = StripRouting($message);
    $message = StripBBSCodes($message);
    $message = StripANSICodes($message);
    $message = HTMLEncode($message, $GLOBALS['USER_SETTINGS']['htmlencode']);
    $message = HTMLcp437($message);

    $writer->startElement('item');
    $writer->writeElement('title', $subject);
    $writer->writeElement('description', nl2br($header . $message));
    $writer->writeElement('pubDate', $dateTime['date']);
    $writer->writeElement('link', $baseUrl . 'forum-viewmsg.php?t=' . $t . '&d=' . $d . '&m=' . $i);
    $writer->writeElement('guid', $baseUrl . 'forum-viewmsg.php?t=' . $t . '&d=' . $d . '&m=' . $i);
    $writer->endElement(); //item
}

$writer->endElement(); //channel
$writer->endElement(); //rss
$writer->endDocument();
$writer->flush();

?>