<?php
/*
 * ----------------------------------------------------------------------------
 * Forum-ViewMsg.php                                                 04/06/2014
 * This script will display the selected message to the user.
 * The variable $t is passed to the script to set the topic number.
 * The variable $d is passed to the script to set the database number.
 * The variable $m is passed to the script to set the message number.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2014 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

// Useful if a user hits the back button.
session_cache_limiter('nocache');

$GLOBALS['VA_ACTION'] = 'Reading Messages';
$pagetitle = 'Message Forums';

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');

$classDatabases = new vadvDatabases;
$classNetworks = new vadvNetworks;
$classUserfileDat = new datUserfile;

define('NOINDEX', true);

$VA_NETWORKS = $classNetworks->GetNetworkInfo();

// Set the passed variables.
// Topic.
if (isset($_GET['t'])) {
    $t = (integer) $_GET['t'];
} else {
    $t = 1;
}

// Database.
if (isset($_GET['d'])) {
    $d = (integer) $_GET['d'];
} else {
    $d = 1;
}

// Ansi.
if (isset($_GET['ansi'])) {
    $ansi = (integer) $_GET['ansi'];
} else {
    $ansi = 0;
}

// Color.
if (isset($_GET['color'])) {
    $color = (integer) $_GET['color'];
} else {
    $color = 0;
}

// Html.
if (isset($_GET['html'])) {
    $html = (integer) $_GET['html'];
} else {
    $html = 0;
}

// Routing.
if (isset($_GET['routing'])) {
    $routing = (integer) $_GET['routing'];
} else {
    $routing = 0;
}

// Emoticons.
if (isset($_GET['emoticons'])) {
    $emoticons = (integer) $_GET['emoticons'];
} else {
    $emoticons = 0;
}

// Encoding.
if (isset($_GET['encoding'])) {
    $encoding = (integer) $_GET['encoding'];
} else {
    $encoding = 0;
}

if ($t < 1) {
    $t = 1;
}
if ($d < 1) {
    $d = 1;
}

// Retrieve the topic information.
$topictypes = 'M';
if (($GLOBALS['USER']['securitylevel'] >= 255) &&
    ($GLOBALS['CONFIG']['showcustomdbs'])) {
    $topictypes = 'MO';
}
$topic = $classDatabases->GetTopicInfo($t, $topictypes);

// If this is an invalid section, display an error message.
if ($topic == '') {
    die('<br /><h4>Invalid topic. Please press the BACK button on your browser.</h4>');
}

// Retrieve the database information.
$db = $classDatabases->GetDbInfo($t, $d);

// Retrieve the number of messages.
$msgno = $db['msgno'];
if ($msgno == 0) {
    die('<br /><h4>Invalid message. Please press the BACK button on your browser.</h4>');
}

// Determine if the user has access to this section.
if (!$classDatabases->DbAccess($db)) {
    die('<br /><h4>You do not have access to this section. Please press the BACK button on your browser.</h4>');
}

$topicDesc = htmlspecialchars(trim($topic['desc']));
if ($topicDesc == '') {
    $topicDesc = '(No Topic Description)';
}

$dbDesc = htmlspecialchars(trim($db['desc']));
if ($dbDesc == '') {
    $dbDesc = '(No Description)';
}

if (strlen($dbDesc) >= 31) {
    $dbDesc = $dbDesc . '...';
}

// Message.
if (isset($_GET['m'])) {
    $m = (integer) $_GET['m'];
} else {
    $m = $msgno;
}
if ($m > $msgno) {
    $m = $msgno;
}
if ($m <= 0) {
    $m = 1;
}

// Set the rss url variable.
$rssurl = GetCurrentUrl(0, 1, 0) . '/forum-rss.php?t=' . $t . '&d=' . $d;
$rsstitle = $GLOBALS['VA_MAIN']['bbsname'] . ' - ' . $dbDesc;

// Get the message information.
$msgInfo = $classDatabases->GetMessageInfo($db, $m);

// Get the message to display.
$message = $classDatabases->GetMessage($db, $m, 2, '', $msgInfo);
$origMessage = $message;

// Apply options.
if ($html == 0) {
    $message = rtrim(str_replace('&amp;', '&', htmlspecialchars($message)));

    // Fix for blank messages.
    if ($message == '') {
        $message = rtrim(preg_replace("/(=20)||(=09)/", '', strip_tags($origMessage)));
    }
} else {
    $message = rtrim(preg_replace("/(=20)||(=09)/", '', strip_tags($message)));
}
if ($routing == 0) {
    $message = StripRouting($message);
}
if ($color == 0) {
    $message = StripBBSCodes($message);
}
if ($ansi == 0) {
    $message = StripANSICodes($message);
}
if ($encoding == 0) {
    $message = HTMLEncode($message, $GLOBALS['USER_SETTINGS']['htmlencode']);
}
if ($emoticons == 0) {
    $message = convert_bbcodes($message);
}
$message = HTMLcp437($message);

// Determine if the message is deleted and give proper access.
if (($msgInfo['deleted'] == 1) &&
    (($GLOBALS['USER']['securitylevel'] < 255) &&
     (($db['moderator'] != $GLOBALS['USER']['userno']) ||
      ($db['moderator'] == 0)))) {
    die('<br /><h4>This message has been deleted. Please press the BACK button on your browser.</h4>');
}

// Check for delete message flag.
if (isset($_GET['delete'])) {
    if ($_GET['delete'] == 1) {
        // If an attached file, then delete it. Non-reversible!
        if ($msgInfo['attfilename'] != '') {
            if ($GLOBALS['CONFIG']['filehost'] != '') {
                $file = ConvertDir($GLOBALS['CONFIG']['hostname'], $db['filepath']) . '/' . $msgInfo['attfilename'];
            } else {
                $file = $db['filepath'] . '/' . $msgInfo['attfilename'];
            }
            if ($msgInfo['attfilepath'] == '') {
                unlink($file);
            }
            $msgInfo['attfilesize'] = 0;
            $msgInfo['attfilename'] = '';
            $msgInfo['attfilepath'] = '';
        }
        $msgInfo['deleted'] = 1;
        $classDatabases->WriteMessageInfo($db, $m, $msgInfo);
        RedirectUser('forum-dispmsg.php?t=' . $t . '&d=' . $d);
    } else if ($_GET['delete'] == 0) {
        $msgInfo['deleted'] = 0;
        $classDatabases->WriteMessageInfo($db, $m, $msgInfo);
    }
}

$prevnum = 0;
$nextnum = 0;
if ($m > 1) {
    $prevnum = $m - 1;
}
if ($m < $msgno) {
    $nextnum = $m + 1;
}

$dateTime = ConvertDateTime($msgInfo['creation'], 'F j, Y');
// Check for any files.
if ($msgInfo['attfilename'] != '') {
    if ($db['filepath'] != '') {
        if ($GLOBALS['CONFIG']['filehost'] != '') {
            $file = ConvertDir($GLOBALS['CONFIG']['hostname'], $db['filepath']) . '/' . $msgInfo['attfilename'];
        } else {
            $file = $db['filepath'] . '/' . $msgInfo['attfilename'];
        }
    } else {
        $file = $GLOBALS['VA_SYSPATHS']['sysop'] . '/' . $msgInfo['attfilename'];
    }
} else {
    $file = '';
}

if ($file != '') {
    if (!file_exists($file)) {
        $filesize = '0 bytes';
    } else {
        $filesize = filesize($file);
        if ($filesize >= 1073741824) {
            $filesize = round($filesize / 1073741824, 1) . 'GB';
        } else if ($filesize >= 1048576) {
            $filesize = round($filesize / 1048576, 1) . 'MB';
        } else if ($filesize >= 1024) {
            $filesize = intval($filesize / 1024) . 'KB';
        } else {
            $filesize = $filesize . ' bytes';
        }
    }
}

// Check for email address.
$from = $msgInfo['fromhandle'];
$pos = strpos($from, '@');
if (is_int($pos)) {
    $from = substr($from, 0, $pos);
}
$char = substr($from, 0, 1);
if ($char == '<') {
    $from = substr($from, 1);
}
if ($char == '"') {
    $pos = strpos($from, '"', 1);
    $from = substr($from, 1, $pos - 1);
}
$from = htmlspecialchars(trim($from));

// Check for email address.
$to = $msgInfo['tohandle'];
$pos = strpos($to, '@');
if (is_int($pos)) {
    $to = substr($to, 0, $pos);
}
$char = substr($to, 0, 1);
if ($char == '<') {
    $to = substr($to, 1);
}
if ($char == '"') {
    $pos = strpos($to, '"', 1);
    $to = substr($to, 1, $pos - 1);
}
$to = htmlspecialchars(trim($to));

$subject = htmlspecialchars(trim(StripASCIICodes(StripANSICodes(StripBBSCodes($msgInfo['subject'])))));
if ($subject == '') {
    $subject = '(No Subject)';
}

$pos = strpos($to, '@');
if (is_int($pos)) {
    $to = substr($to, 0, $pos);
}

// Update message pointer if needed.
if ($classDatabases->GetMessagePointer($db, $GLOBALS['USER']['userno']) < $m) {
    $classDatabases->WriteMessagePointer($db, $GLOBALS['USER']['userno'], $m);
}

// Output header
include($themepath . '/templates/header.php');

// Output page
include($themepath . '/templates/viewmsg.php');

// Output footer
include($themepath . '/templates/footer.php');

?>