<?php
/*
 * ----------------------------------------------------------------------------
 * SentEmail-ViewMsg.php                                             04/05/2014
 * This script will display the selected sent email message to the user.
 * The variable $m is passed to the script to set the message number.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2014 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

$GLOBALS['VA_ACTION'] = 'Reading Email';
$pagetitle = 'E-Mail Area';

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');

$classDatabases = new vadvDatabases;
$classNetworks = new vadvNetworks;
$classUserfileDat = new datUserfile;

define('NOINDEX', true);

// Get network information.
$VA_NETWORKS = $classNetworks->GetNetworkInfo();

if (($GLOBALS['USER']['userno'] == 0) ||
    ($GLOBALS['USER']['userno'] == $GLOBALS['CONFIG']['guestacct']) ||
    ($GLOBALS['USER']['securitylevel'] == 0)) {
    die(VADV_LoginError());
}

$userno = $GLOBALS['USER']['userno'];

// Set the passed variables.
// Ansi.
if (isset($_GET['ansi'])) {
    $ansi = (integer) $_GET['ansi'];
} else {
    $ansi = 0;
}

// Color.
if (isset($_GET['color'])) {
    $color = (integer) $_GET['color'];
} else {
    $color = 0;
}

// Html.
if (isset($_GET['html'])) {
    $html = (integer) $_GET['html'];
} else {
    $html = 0;
}

// Routing.
if (isset($_GET['routing'])) {
    $routing = (integer) $_GET['routing'];
} else {
    $routing = 0;
}

// Emoticons.
if (isset($_GET['emoticons'])) {
    $emoticons = (integer) $_GET['emoticons'];
} else {
    $emoticons = 0;
}

// Encoding.
if (isset($_GET['encoding'])) {
    $encoding = (integer) $_GET['encoding'];
} else {
    $encoding = 0;
}

// Retrieve the database information.
$db = $classDatabases->SetSentEmailInfo($userno);

// Retrieve the number of messages.
$msgno = $db['msgno'];
if ($msgno == 0) {
    die('<br /><h4>Invalid message. Please press the BACK button on your browser.</h4>');
}

if ($GLOBALS['USER_SETTINGS']['hidesentdeleted'] == 1) {
    // Retrieve the number of user messages.
    $msgstat = $classDatabases->GetSentEmailCount($userno, 0);
} else {
    $msgstat = $classDatabases->GetSentEmailCount($userno);
}
$msgcnt = $msgstat['msgno'];
if ($msgcnt == 0) {
    die('<br /><h4>Invalid message. Please press the BACK button on your browser.</h4>');
}

// Set the passed variables.
if (isset($_GET['m'])) {
    $m = (integer) $_GET['m'];
} else {
    $m = $msgcnt;
}
if ($m > $msgcnt) {
    $m = $msgcnt;
}
if ($m <= 0) {
    $m = 1;
}

// Get the message information.
$msgInfo = $classDatabases->GetMessageInfo($db, $msgstat[$m - 1]);

if ($msgInfo['fromuserno'] != $userno) {
    die('<br /><h4>You do not have access to this message. Press the BACK button on your browser.</h4>');
}

// Get the message to display.
$message = $classDatabases->GetMessage($db, $m, 2, '', $msgInfo);
$origMessage = $message;

// Apply options.
if ($html == 0) {
    $message = rtrim(str_replace('&amp;', '&', htmlspecialchars($message)));

    if ($message == '') {
        $message = rtrim(preg_replace("/(=20)||(=09)/", '', strip_tags($origMessage)));
    }
} else {
    $message = rtrim(preg_replace("/(=20)||(=09)/", '', strip_tags($message)));
}
if ($routing == 0) {
    $message = StripRouting($message);
}
if ($color == 0) {
    $message = StripBBSCodes($message);
}
if ($ansi == 0) {
    $message = StripANSICodes($message);
}
if ($encoding == 0) {
    $message = HTMLEncode($message, $GLOBALS['USER_SETTINGS']['htmlencode']);
}
if ($emoticons == 0) {
    $message = convert_bbcodes($message);
}
$message = HTMLcp437($message);

// Check for delete message flag.
if (isset($_GET['delete'])) {
    if ($_GET['delete'] == 1) {
        // If an attached file, then delete it. Non-reversible!
        if ($msgInfo['attfilename'] != '') {
            if ($GLOBALS['CONFIG']['hostname'] != '') {
                $file = ConvertDir($GLOBALS['CONFIG']['hostname'], $db['filepath']) . '/' . $msgInfo['attfilename'];
            } else {
                $file = $db['filepath'] . '/' . $msgInfo['attfilename'];
            }

            if ($msgInfo['attfilepath'] == '') {
                if (file_exists($file)) {
                    unlink($file);
                }
            } else {
                if ($GLOBALS['CONFIG']['hostname'] != '') {
                    $userDir = ConvertUnc($GLOBALS['VA_SYSPATHS']['sysop'] . '/' . $GLOBALS['USER']['userno'] . '/' . $msgInfo['attfilename']);
                } else {
                    $userDir = $GLOBALS['VA_SYSPATHS']['sysop'] . '/' . $GLOBALS['USER']['userno'] . '/' . $msgInfo['attfilename'];
                }
                if ($msgInfo['attfilepath'] == $userDir) {
                    if ($GLOBALS['CONFIG']['hostname'] != '') {
                        $file = ConvertDir($GLOBALS['CONFIG']['hostname'], $msgInfo['attfilepath']);
                    } else {
                        $file = $msgInfo['attfilepath'];
                    }
                }
                if (file_exists($file)) {
                    unlink($file);
                }
            }
            $msgInfo['attfilesize'] = 0;
            $msgInfo['attfilename'] = '';
            $msgInfo['attfilepath'] = '';
        }
        $msgInfo['deleted'] = 1;
        $classDatabases->WriteMessageInfo($db, $msgstat[$m - 1], $msgInfo);
        // Until this database is added to vcleandb, all messages are permanently
        // deleted if you uncomment this next line.
        // $classDatabases->DeleteMessage($db, $msgstat[$m - 1]);
        RedirectUser('sentemail-dispmsg.php');
    } else if ($_GET['delete'] == 0) {
        $msgInfo['deleted'] = 0;
        $classDatabases->WriteMessageInfo($db, $msgstat[$m - 1], $msgInfo);
    }
}


$dateTime = ConvertDateTime($msgInfo['creation'], 'F j, Y');

// Check for any files.
if ($msgInfo['attfilename'] != '') {
    // If a local attachment.
    if ($msgInfo['attfilepath'] != '') {
        $filepath = str_replace('/', "\\", $msgInfo['attfilepath']);
        $pos = strrpos($filepath, "\\");
        $filepath = substr($filepath, 0, $pos);
    } else {
        $filepath = $db['filepath'];
    }
    if ($GLOBALS['CONFIG']['filehost'] != '') {
        $file = ConvertDir($GLOBALS['CONFIG']['hostname'], $filepath) . '/' . $msgInfo['attfilename'];
    } else {
        $file = $filepath . '/' . $msgInfo['attfilename'];
    }
} else {
    $file = '';
}

if ($file != '') {
    if (!file_exists($file)) {
        $filesize = '0 bytes';
    } else {
        $filesize = filesize($file);
        if ($filesize >= 1073741824) {
            $filesize = round($filesize / 1073741824, 1) . 'GB';
        } else if ($filesize >= 1048576) {
            $filesize = round($filesize / 1048576, 1) . 'MB';
        } else if ($filesize >= 1024) {
            $filesize = intval($filesize / 1024) . 'KB';
        } else {
            $filesize = $filesize . ' bytes';
        }
    }
}

$from = $msgInfo['fromhandle'];
if ($from == '') {
    $from = $msgInfo['fromuserno'];
}
$char = substr($from, 0, 1);
if ($char == '<') {
    $from = substr($from, 1);
}
if ($char == '"') {
    $pos = strpos($from, '"', 1);
    $from = substr($from, 1, $pos - 1);
}
$from = htmlspecialchars(trim($from));
if ($msgInfo['fromnetid'] != 1) {
    $from = $from . '@' . $msgInfo['fromnetnode'] . '*' . $msgInfo['fromnetid'];
}

$subject = htmlspecialchars(trim(StripASCIICodes(StripANSICodes(StripBBSCodes($msgInfo['subject'])))));
if ($subject == '') {
    $subject = '(No Subject)';
}

$to = $msgInfo['tohandle'];
if ($to == '') {
    $to = $msgInfo['touserno'];
}
$char = substr($to, 0, 1);
if ($char == '<') {
    $to = substr($to, 1);
}
if ($char == '"') {
    $pos = strpos($to, '"', 1);
    $to = substr($to, 1, $pos - 1);
}
$to = htmlspecialchars(trim($to));
if ($msgInfo['tonetid'] != 1) {
    $to = $to . '@' . $msgInfo['tonetnode'] . '*' . $msgInfo['tonetid'];
}


// Update the message as read if not already marked.
if ($msgInfo['offline'] == 0) {
    $msgInfo['offline'] = 1;
    $classDatabases->WriteMessageInfo($db, $msgstat[$m - 1], $msgInfo);
}


// Set up Prev/Next links
$prev = 0;
$next = 0;
$backlink = '';
if ($m > 1) {
    $num = $m - 1;
    $prev = $num;
}

if ($m < $msgcnt) {
    $num = $m + 1;
    $next = $num;
}

if ($m >= $msgcnt - 25) {
    $backlink = 'href="sentemail-dispmsg.php"';
} else if ($m <= 25) {
    $backlink = 'href="sentemail-dispmsg.php?m=25"';
} else {
    $backlink = 'href="sentemail-dispmsg.php?m=' . $m . '"';
}

// Output header
include($themepath . '/templates/header.php');

// Output page
include($themepath . '/templates/viewsentemail.php');

// Output footer
include($themepath . '/templates/footer.php');

?>