<?php
/*
 * ----------------------------------------------------------------------------
 * AT2k.inc.php                                                      12/30/2008
 * This script contains functions that work with AT2k scripts and programs.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2008 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');


class NewsListConfig {
    /**
    * AT2k.inc.php :: Get()
    *
    * This function will return the complete newslist configuration. It is in the
    * format of $config[topicno][dbno]['setting'].
    *
    * @return
    */
    function Get() {
        $file = $GLOBALS['VA_SYSPATHS']['text'] . '/NEWSLIST.CFG';

        if (!file_exists($file)) {
            return FALSE;
        }

        $datFile = $GLOBALS['VA_SYSPATHS']['text'] . '/NEWSLIST.';
        $config = ReadAt2kConfig($file);

        if (!$config) {
            return FALSE;
        }

        $tempCfg = explode(',', $config[1]);
        $newsConfig['ver'] = $tempCfg[0];
        $newsConfig['help'] = (boolean) $tempCfg[1];

        // Get topics.
        $cnt = 0;
        $pos = 1;
        for ($i = 2; $i <= count($config); $i++) {
            $tempCfg = explode(',', $config[$i]);
            $newsConfig[$pos]['desc'] = trim(StripBBSCodes($tempCfg[0]));
            $newsConfig[$pos]['path'] = trim($tempCfg[1]);
            $cnt++;
            $pos++;
        }

        // Total number of topics.
        $newsConfig[0] = $cnt;

        // Get databases.
        for ($i = 1; $i <= $newsConfig[0]; $i++) {
            $config = ReadAt2kConfig($datFile . $i, 0);
            $cnt = 0;
            $pos = 1;
            if ($config != FALSE) {
                for ($j = 1; $j <= count($config); $j++) {
                    $newsConfig[$i][$pos]['display'] = trim(substr($config[$j], 0, 1));
                    $newsConfig[$i][$pos]['filename'] = trim(substr($config[$j], 1, 12));
                    $newsConfig[$i][$pos]['desc'] = trim(StripBBSCodes(substr($config[$j], 14)));
                    $cnt++;
                   $pos++;
                }
            }

            // Total number of bulletins.
            $newsConfig[$i][0] = $cnt;
        }

        // Return the configuration.
        return $newsConfig;
    }
}


class PostItConfig {
    /**
    * AT2k.inc.php :: Get()
    *
    * This function will retrieve the Post-It! Network configuration. It is in the
    * format of $config['setting'].
    *
    * @return mixed
    */
    function Get() {
        $file = $GLOBALS['VA_SYSPATHS']['text'] . '/POSTITNT.CFG';

        if (!file_exists($file)) {
            return FALSE;
        }

        $config = ReadAt2kConfig($file);

        if (!$config) {
            return FALSE;
        }

        // Setup the configuration variable array.
        $postitCfg['ver'] = $config[1];
        $postitCfg['topic'] = strtoupper($config[2]);
        $postitCfg['topicno'] = ord($postitCfg['topic']) - 32;
        $postitCfg['db'] = (integer) $config[3];
        $postitCfg['networknode'] = $config[4];
        $postitCfg['vnetnode'] = $config[5];
        $postitCfg['minwritesl'] = (integer) $config[6];
        $postitCfg['filter'] = (boolean) $config[7];

        // Return the configuration.
        return $postitCfg;
    }
}


class SysNewsConfig {
    /**
    * AT2k.inc.php :: Get()
    *
    * This function will read the sysnews.v script and extract the database
    * information. It is in the format of $config['setting'].
    *
    * @return mixed
    */
    function Get() {
        $file = $GLOBALS['VA_SYSPATHS']['script'] . '/SYSNEWS.V';

        if (!file_exists($file)) {
            return FALSE;
        }

        // Read the file into an array.
        $fileArray = file($file);

        if (!$fileArray) {
            return FALSE;
        }

        // Set default values.
        $config['topicno'] = 0;
        $config['topic'] = '';
        $config['db'] = 0;

        for ($i = 0; $i < count($fileArray); $i++) {
            // Take out all spaces.
            $line = str_replace(' ', '', $fileArray[$i]);

            // Search variable for DBGROUP.
            $searchGroup = substr($line, 0, 7);

            // Search variable for DB.
            $searchDb = substr($line, 0, 2);

            if (strtoupper($searchGroup) == 'DBGROUP') {
                $config['topic'] = strtoupper(substr($line, 8, 1));
            } else if (strtoupper($searchDb) == 'DB') {
                $config['db'] = (integer) substr($line, 2, 1);
                break;
            }
        }

        if (($config['topic'] == '') ||
            ($config['db'] == 0)) {
            return FALSE;
        }

        $config['topicno'] = ord($config['topic']) - 32;

        // Return the configuration.
        return $config;
    }
}
?>