<?php
/*
 * ----------------------------------------------------------------------------
 * Va-Arcs.inc.php                                                   12/31/2008
 * This script will retrieve all of the informaton related to the configured
 * archiving programs.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2008 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');


class cfgArcs {
    /**
    * Va-Arcs.inc.php :: Get()
    *
    * Retrieve all of the archive program information and store it into a
    * multidimensional array.
    *
    * Array format: $arcs[1]['setting']
    *               $arcs[0] = Total number of configured archive programs.
    *
    * @return array
    */
    function Get() {
        $fileArray = $this->Read();
        if ($fileArray === FALSE) {
            trigger_error(VA_Error(3));
        }

        // Create the array.
        $arcs = array();
        $pos = 0;

        for ($i = 1; $pos < count($fileArray); $i++) {
            // Extract the ascii version of the hex key. Spaces are represented by underscores.
            $line = trim($fileArray[0 + $pos]);
            $slength = strspn($line, '0');
            $line = substr($line, $slength);
            $length = strlen($line);
            $key = '';
            for ($j = 0; $j < $length; $j = $j + 2) {
                $val = substr($line, $j, 2);
                $key = $key . chr(hexdec($val));
            }
            $key = str_pad($key, strlen($key) + $slength, '_', STR_PAD_LEFT);

            $arcs[$i]['key'] = trim($key);
            $arcs[$i]['hexkey'] = trim($fileArray[0 + $pos]);
            $arcs[$i]['addcmd'] = trim($fileArray[1 + $pos]);
            $arcs[$i]['movecmd'] = trim($fileArray[2 + $pos]);
            $arcs[$i]['extractcmd'] = trim($fileArray[3 + $pos]);
            $arcs[$i]['viewcmd'] = trim($fileArray[4 + $pos]);
            $arcs[$i]['testcmd'] = trim($fileArray[5 + $pos]);
            $arcs[$i]['commentcmd'] = trim($fileArray[6 + $pos]);
            $pos = $pos + 7;
        }

        // Put the number of programs in position 0.
        $arcs[0] = $i - 1;

        // Return the archiving programs.
        return $arcs;
    }


    /**
    * Va-Arcs.inc.php :: Read()
    *
    * This function will read the arcs.cfg file.
    *
    * @return array
    */
    function Read() {
        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/ARCS.CFG';

        // Read the file.
        $fileArray = file($file);

        // Return the file string.
        return $fileArray;
    }


    /**
    * Va-Arcs.inc.php :: Write()
    *
    * This function will write a new arcs.cfg file based on the file array
    * passed to it. The file array must be in the same format as Get(). It is
    * a good idea to load the arcs array before creating any entries to be sure
    * all the keys will be written.
    *
    * @param array $fileArray
    * @return mixed
    */
    function Write($fileArray) {
        if (!is_array($fileArray)) {
            trigger_error(VA_Error(1));
        }

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/ARCS.CFG';

        // Open the file.
        $fp = fopen($file, 'wb');

        // Lock the file.
        flock($fp, LOCK_EX);

        for ($i = 1; $i < count($fileArray); $i++) {
            fwrite($fp, $fileArray[$i]['hexkey'] . "\r\n", strlen($fileArray[$i]['hexkey']) + 2);
            fwrite($fp, $fileArray[$i]['addcmd'] . "\r\n", strlen($fileArray[$i]['addcmd']) + 2);
            fwrite($fp, $fileArray[$i]['movecmd'] . "\r\n", strlen($fileArray[$i]['movecmd']) + 2);
            fwrite($fp, $fileArray[$i]['extractcmd'] . "\r\n", strlen($fileArray[$i]['extractcmd']) + 2);
            fwrite($fp, $fileArray[$i]['viewcmd'] . "\r\n", strlen($fileArray[$i]['viewcmd']) + 2);
            fwrite($fp, $fileArray[$i]['testcmd'] . "\r\n", strlen($fileArray[$i]['testcmd']) + 2);
            fwrite($fp, $fileArray[$i]['commentcmd'] . "\r\n", strlen($fileArray[$i]['commentcmd']) + 2);
        }

        // Unlock the file.
        flock($fp, LOCK_UN);
        fclose($fp);

        return;
    }
}
?>