<?php
/*
 * ----------------------------------------------------------------------------
 * Va-Channels.inc.php                                               12/31/2008
 * This script will retrieve all of the informaton related to the configured
 * channels. The channels.cfg file is use ONLY for backward compatibility with
 * VBBS utilities. You should use the channelx.cfg for VADV.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2008 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');


class cfgChannels {
    /**
    * Va-Channels.inc.php :: Get()
    *
    * Retrieve all of the channel information and store it into a
    * multidimensional array.
    *
    * Array format: $channels[1]['setting']
    *               $channels[0] = Total number of nodes.
    *
    * @return array
    */
    function Get() {
        $fileString = $this->Read();
        if ($fileString === FALSE) {
            trigger_error(VA_Error(3));
        }

        // Create the array.
        $channels = array();
        $offset = 0;

        for ($i = 1; $offset < strlen($fileString); $i++) {
            $channels[$i]['init'] = GetString($fileString, 1 + $offset, 40);
            $channels[$i]['baudrate'] = GetLongInt($fileString, 41 + $offset);
            $channels[$i]['lockedbaud'] = GetInteger($fileString, 45 + $offset);
            $channels[$i]['rtscts'] = GetInteger($fileString, 47 + $offset);
            $channels[$i]['serialport'] = GetInteger($fileString, 49 + $offset);
            $channels[$i]['porttype'] = GetInteger($fileString, 51 + $offset);
            $channels[$i]['baseaddy'] = dechex(GetInteger($fileString, 53 + $offset));
            $channels[$i]['irq'] = dechex(GetInteger($fileString, 55 + $offset));
            $channels[$i]['minbaud'] = GetLongInt($fileString, 57 + $offset);
            $channels[$i]['extra'] = GetString($fileString, 61 + $offset, 4);

            $offset = 64 * $i;
        }

        // Put the number of channels in position 0.
        $channels[0] = $i - 1;

        // Return the channel information.
        return $channels;
    }


    /**
    * Va-Channels.inc.php :: Put()
    *
    * This function will change the channels.cfg to match the configuration variable
    * passed. It then returns the new file string. The configuration variable MUST
    * be in the format of $variable[1]['setting'].
    *
    * @param string $fileString
    * @param array $configVariable
    * @return string
    */
    function Put($fileString,
                 $configVariable) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }
        if (!is_array($configVariable)) {
            trigger_error(VA_Error(1));
        }

        // Get the number of elements in the config variable.
        $length = count($configVariable);
        $pointer = 0;
        $offset = 0;

        if (isset($configVariable[0])) {
            $length--;
        }

        for ($i = 1; $pointer < $length; $i++) {
            $total = intval(strlen($fileString) / 64);
            if ($i > $total) {
                $fileString = $fileString . str_pad('', 64, chr(0));
            }

            if (isset($configVariable[$i])) {
                $pointer++;
                if (isset($configVariable[$i]['init'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['init'], 1 + $offset, 40);
                }
                if (isset($configVariable[$i]['baudrate'])) {
                    $fileString = PutLongInt($fileString, $configVariable[$i]['baudrate'], 41 + $offset);
                }
                if (isset($configVariable[$i]['lockedbaud'])) {
                    $fileString = PutInteger($fileString, $configVariable[$i]['lockedbaud'], 45 + $offset);
                }
                if (isset($configVariable[$i]['rtscts'])) {
                    $fileString = PutInteger($fileString, $configVariable[$i]['rtscts'], 47 + $offset);
                }
                if (isset($configVariable[$i]['serialport'])) {
                    $fileString = PutInteger($fileString, $configVariable[$i]['serialport'], 49 + $offset);
                }
                if (isset($configVariable[$i]['porttype'])) {
                    $fileString = PutInteger($fileString, $configVariable[$i]['porttype'], 51 + $offset);
                }
                if (isset($configVariable[$i]['baseaddy'])) {
                    $fileString = PutInteger($fileString, hexdec($configVariable[$i]['baseaddy']), 53 + $offset);
                }
                if (isset($configVariable[$i]['irq'])) {
                    $fileString = PutInteger($fileString, hexdec($configVariable[$i]['irq']), 55 + $offset);
                }
                if (isset($configVariable[$i]['minbaud'])) {
                    $fileString = PutLongInt($fileString, $configVariable[$i]['minbaud'], 57 + $offset);
                }
                if (isset($configVariable[$i]['extra'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['extra'], 61 + $offset, 4);
                }
            }
            $offset = 64 * $i;
        }

        // Return the new file string.
        return $fileString;
    }


    /**
    * Va-Channels.inc.php :: Read()
    *
    * This function will read the channels.cfg file.
    *
    * @return string
    */
    function Read() {
        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/CHANNELS.CFG';

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Return the file string.
        return $fileString;
    }


    /**
    * Va-Channels.inc.php :: Write()
    *
    * This function will write a new channels.cfg file based on the file string
    * passed to it. If the file string is not in the correct format, then your
    * channels.cfg file will become corrupt and the BBS could stop functioning!
    *
    * @param string $fileString
    * @return mixed
    */
    function Write($fileString) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/CHANNELS.CFG';
        $status = WriteCfgFile($file, $fileString);

        // Return the result status.
        return $status;
    }
}
?>