<?php
/*
 * ----------------------------------------------------------------------------
 * Va-Colors.inc.php                                                 12/31/2008
 * This script will retrieve all of the informaton related to the configured
 * colors that are found in COLOR.CFG.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2008 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');


class cfgColor {
    /**
    * Va-Colors.inc.php :: Get()
    *
    * Retrieve all of the color information and store it into a multidimensional
    * array.
    *
    * Array format: $color[1]['setting']
    *
    * @return array
    */
    function Get() {
        $fileString = $this->Read();
        if ($fileString === FALSE) {
            trigger_error(VA_Error(3));
        }

        // Create the array.
        $colors = array();
        $offset = 0;
        $j = 0;

        for ($i = 1; $offset < strlen($fileString); $i++) {
            if ($j > 8) {
                if ($j == 9) {
                    $j = 65;
                } else {
                    $j++;
                }
                $k = chr($j);
            } else {
                $j++;
                $k = $j;
            }
            $colors[$k]['intensity'] = GetInteger($fileString, 1 + $offset);
            $colors[$k]['foreground'] = GetInteger($fileString, 3 + $offset);
            $colors[$k]['background'] = GetInteger($fileString, 5 + $offset);
            $offset = 6 * $i;
        }

        // Return the doors.
        return $colors;
    }


    /**
    * Va-Colors.inc.php :: Put()
    *
    * This function will change the color.cfg to match the configuration variable
    * passed. It then returns the new file string. The configuration variable MUST
    * be in the format of $variable[1]['setting'].
    *
    * @param string $fileString
    * @param array $configVariable
    * @return string
    */
    function Put($fileString,
                 $configVariable) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }
        if (!is_array($configVariable)) {
            trigger_error(VA_Error(1));
        }

        // Get the number of elements in the config variable.
        $length = count($configVariable);
        $pointer = 0;
        $offset = 0;
        $j = 0;

        for ($i = 1; $pointer < $length; $i++) {
            $total = intval(strlen($fileString) / 6);
            if ($i > $total) {
                $fileString = $fileString . str_pad('', 6, chr(0));
            }

            if ($j > 8) {
                if ($j == 9) {
                    $j = 65;
                } else {
                    $j++;
                }
                $k = chr($j);
            } else {
                $j++;
                $k = $j;
            }

            if (isset($configVariable[$k])) {
                $pointer++;
                if (isset($configVariable[$k]['intensity'])) {
                    $fileString = PutInteger($fileString, $configVariable[$k]['intensity'], 1 + $offset);
                }
                if (isset($configVariable[$k]['foreground'])) {
                    $fileString = PutInteger($fileString, $configVariable[$k]['foregournd'], 3 + $offset);
                }
                if (isset($configVariable[$k]['background'])) {
                    $fileString = PutInteger($fileString, $configVariable[$k]['background'], 5 + $offset);
                }
            }
            $offset = 6 * $i;
        }

        // Return the new file string.
        return $fileString;
    }


    /**
    * Va-Colors.inc.php :: Read()
    *
    * This function will read the color.cfg file.
    *
    * @return string
    */
    function Read() {
        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/COLOR.CFG';

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Return the file string.
        return $fileString;
    }


    /**
    * Va-Colors.inc.php :: Write()
    *
    * This function will write a new color.cfg file based on the file string
    * passed to it. If the file string is not in the correct format, then your
    * color.cfg file will become corrupt and the BBS could stop functioning!
    *
    * @param string $fileString
    * @return mixed
    */
    function Write($fileString) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/COLOR.CFG';
        $status = WriteCfgFile($file, $fileString);

        // Return the result status.
        return $status;
    }
}
?>