<?php
/*
 * ----------------------------------------------------------------------------
 * Va-Doors.inc.php                                                  12/31/2008
 * This script will retrieve all of the informaton related to the configured
 * doors that are found in DOORS.CFG.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2008 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');


class cfgDoors {
    /**
    * Va-Doors.inc.php :: Get()
    *
    * Retrieve all of the doors information and store it into a multidimensional
    * array.
    *
    * Array format: $door[1]['setting']
    *               $door[0] = Total number of doors.
    *
    * @return array
    */
    function Get() {
        $fileString = $this->Read();
        if ($fileString === FALSE) {
            trigger_error(VA_Error(3));
        }

        // Create the array.
        $doors = array();
        $offset = 0;

        for ($i = 1; $offset < strlen($fileString); $i++) {
            $doors[$i]['desc'] = GetString($fileString, 1 + $offset, 32);
            $doors[$i]['command'] = GetString($fileString, 33 + $offset, 32);
            $doors[$i]['minsl'] = GetInteger($fileString, 65 + $offset);
            $doors[$i]['accflag'] = GetString($fileString, 67 + $offset, 1);
            $doors[$i]['singleuser'] = GetInteger($fileString, 68 + $offset);
            $doors[$i]['credits'] = GetInteger($fileString, 70 + $offset);
            $doors[$i]['extra'] = GetString($fileString, 72 + $offset, 3);
            $offset = 74 * $i;
        }

        // Put the number of doors in position 0.
        $doors[0] = $i - 1;

        // Return the doors.
        return $doors;

    }


    /**
    * Va-Doors.inc.php :: Put()
    *
    * This function will change the doors.cfg to match the configuration variable
    * passed. It then returns the new file string. The configuration variable MUST
    * be in the format of $variable[1]['setting'].
    *
    * @param string $fileString
    * @param array $configVariable
    * @return string
    */
    function Put($fileString,
                 $configVariable) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }
        if (!is_array($configVariable)) {
            trigger_error(VA_Error(1));
        }

        // Get the number of elements in the config variable.
        $length = count($configVariable);
        $pointer = 0;
        $offset = 0;

        if (isset($configVariable[0])) {
            $length--;
        }

        for ($i = 1; $pointer < $length; $i++) {
            $total = intval(strlen($fileString) / 74);
            if ($i > $total) {
                $fileString = $fileString . str_pad('', 74, chr(0));
            }

            if (isset($configVariable[$i])) {
                $pointer++;
                if (isset($configVariable[$i]['desc'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['desc'], 1 + $offset, 32);
                }
                if (isset($configVariable[$i]['command'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['command'], 33 + $offset, 32);
                }
                if (isset($configVariable[$i]['minsl'])) {
                    $fileString = PutInteger($fileString, $configVariable[$i]['minsl'], 65 + $offset);
                }
                if (isset($configVariable[$i]['accflag'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['accflag'], 67 + $offset, 1);
                }
                if (isset($configVariable[$i]['singleuser'])) {
                    $fileString = PutInteger($fileString, $configVariable[$i]['singleuser'], 68 + $offset);
                }
                if (isset($configVariable[$i]['credits'])) {
                    $fileString = PutInteger($fileString, $configVariable[$i]['credits'], 70 + $offset);
                }
                if (isset($configVariable[$i]['extra'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['extra'], 72 + $offset, 3);
                }
            }
            $offset = 74 * $i;
        }

        // Return the new file string.
        return $fileString;
    }


    /**
    * Va-Doors.inc.php :: Read()
    *
    * This function will read the doors.cfg file.
    *
    * @return string
    */
    function Read() {
        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/DOORS.CFG';

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Return the file string.
        return $fileString;
    }


    /**
    * Va-Doors.inc.php :: Write()
    *
    * This function will write a new doors.cfg file based on the file string
    * passed to it. If the file string is not in the correct format, then your
    * doors.cfg file will become corrupt and the BBS could stop functioning!
    *
    * @param string $fileString
    * @return mixed
    */
    function Write($fileString) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/DOORS.CFG';
        $status = WriteCfgFile($file, $fileString);

        // Return the result status.
        return $status;
    }
}
?>