<?php
/*
 * ----------------------------------------------------------------------------
 * Va-Events.inc.php                                                 12/31/2008
 * This script will retrieve all of the informaton related to the configured
 * events that are found in EVENTS.CFG.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2008 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');


class cfgEvents {
    /**
    * Va-Events.inc.php :: Get()
    *
    * Retrieve all of the events information and store it into a multidimensional
    * array.
    *
    * Array format: $event[1]['setting']
    *               $event[0] = Total number of events.
    *
    * @return array
    */
    function Get() {
        $fileString = $this->Read();
        if ($fileString === FALSE) {
            trigger_error(VA_Error(3));
        }

        // Create the array.
        $events = array();
        $offset = 0;

        for ($i = 1; $offset < strlen($fileString); $i++) {
            $events[$i]['time'] = GetString($fileString, 1 + $offset, 5);
            $events[$i]['date'] = GetString($fileString, 6 + $offset, 6);
            $events[$i]['evttype'] = GetInteger($fileString, 12 + $offset);
            $events[$i]['command'] = GetString($fileString, 14 + $offset, 32);
            $offset = 45 * $i;
        }

        // Put the number of events in position 0.
        $events[0] = $i - 1;

        // Return the events.
        return $events;

    }


    /**
    * Va-Events.inc.php :: Put()
    *
    * This function will change the events.cfg to match the configuration variable
    * passed. It then returns the new file string. The configuration variable MUST
    * be in the format of $variable[1]['setting'].
    *
    * @param string $fileString
    * @param array $configVariable
    * @return string
    */
    function Put($fileString,
                 $configVariable) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }
        if (!is_array($configVariable)) {
            trigger_error(VA_Error(1));
        }

        // Get the number of elements in the config variable.
        $length = count($configVariable);
        $pointer = 0;
        $offset = 0;

        if (isset($configVariable[0])) {
            $length--;
        }

        for ($i = 1; $pointer < $length; $i++) {
            $total = intval(strlen($fileString) / 45);
            if ($i > $total) {
                $fileString = $fileString . str_pad('', 45, chr(0));
            }

            if (isset($configVariable[$i])) {
                $pointer++;
                if (isset($configVariable[$i]['time'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['time'], 1 + $offset, 5);
                }
                if (isset($configVariable[$i]['date'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['date'], 6 + $offset, 6);
                }
                if (isset($configVariable[$i]['evttype'])) {
                    $fileString = PutInteger($fileString, $configVariable[$i]['evttype'], 12 + $offset);
                }
                if (isset($configVariable[$i]['command'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['command'], 14 + $offset, 32);
                }
            }
            $offset = 45 * $i;
        }

        // Return the new file string.
        return $fileString;
    }


    /**
    * Va-Events.inc.php :: Read()
    *
    * This function will read the events.cfg file.
    *
    * @return string
    */
    function Read() {
        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/EVENTS.CFG';

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Return the file string.
        return $fileString;
    }


    /**
    * Va-Events.inc.php :: Write()
    *
    * This function will write a new events.cfg file based on the file string
    * passed to it. If the file string is not in the correct format, then your
    * events.cfg file will become corrupt and the BBS could stop functioning!
    *
    * @param string $fileString
    * @return mixed
    */
    function Write($fileString) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/EVENTS.CFG';
        $status = WriteCfgFile($file, $fileString);

        // Return the result status.
        return $status;
    }
}
?>