<?php
/*
 * ----------------------------------------------------------------------------
 * Va-Functions.inc.php                                              12/28/2021
 * This script contains a set of functions that will make it easier to get
 * information from the Virtual Advanced configuration and data files. It also
 * contains basic functions to aid you when creating new scripts.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2016 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');


/**
* Va-Functions.inc.php :: Array_StripSlashes()
*
* This function will remove slashes from an array.
*
* @param array $array
* @return array
*/
function Array_StripSlashes($array) {
    if (!is_array($array)) {
        return $array;
    }
    while (list($key) = each($array)) {
        if (is_array($array[$key])) {
            array_stripslashes($array[$key]);
        } else {
            $array[$key] = stripslashes($array[$key]);
        }
    }

    return $array;
}


/**
* Va-Functions.inc.php :: ConvertDateTime()
*
* This function will convert the many formats of dates and times into a
* standard format that VADV-PHP will use. You can specify your own formats
* if you wish by using the same parameters that date() uses.
*
* @param string $string
* @return array
*/
function ConvertDateTime($string,
                         $dateFormat = '',
                         $timeFormat = '') {
    if (!is_string($string)) {
        $string = (string) $string;
    }
    if (!is_string($dateFormat)) {
        $dateFormat = (string) $dateFormat;
    }
    if (!is_string($timeFormat)) {
        $timeFormat = (string) $timeFormat;
    }

    $date = array();
    $date['date'] = 'Date';
    $date['time'] = 'Time';
    $date['tz'] = TRUE;

    $string = trim($string);
    if ($string == '') {
        return $date;
    }

    $string = str_replace('(', '', $string);
    $string = str_replace(')', '', $string);
    $string = str_replace('[', '', $string);
    $string = str_replace(']', '', $string);

    if ((((strpos($string, '-12')) ||
         (strpos($string, '-1200'))) !== FALSE) ||
        (((strpos($string, '-11')) ||
         (strpos($string, '-1100'))) !== FALSE) ||
        (((strpos($string, '-10')) ||
         (strpos($string, '-1000'))) !== FALSE) ||
        (((strpos($string, '-9')) ||
         (strpos($string, '-900')) ||
         (strpos($string, '-0900'))) !== FALSE) ||
        (((strpos($string, '-930')) ||
         (strpos($string, '-0930'))) !== FALSE) ||
        (((strpos($string, '-8')) ||
         (strpos($string, '-800')) ||
         (strpos($string, '-0800'))) !== FALSE) ||
        (((strpos($string, '-7')) ||
         (strpos($string, '-700')) ||
         (strpos($string, '-0700'))) !== FALSE) ||
        (((strpos($string, '-6')) ||
         (strpos($string, '-600')) ||
         (strpos($string, '-0600'))) !== FALSE) ||
        (((strpos($string, '-5')) ||
         (strpos($string, '-500')) ||
         (strpos($string, '-0500'))) !== FALSE) ||
        (((strpos($string, '-4')) ||
         (strpos($string, '-400')) ||
         (strpos($string, '-0400'))) !== FALSE) ||
        (((strpos($string, '-330')) ||
         (strpos($string, '-0330'))) !== FALSE) ||
        (((strpos($string, '-3')) ||
         (strpos($string, '-300')) ||
         (strpos($string, '-0300'))) !== FALSE) ||
        (((strpos($string, '-2')) ||
         (strpos($string, '-200')) ||
         (strpos($string, '-0200'))) !== FALSE) ||
        (((strpos($string, '-1')) ||
         (strpos($string, '-100')) ||
         (strpos($string, '-0100'))) !== FALSE) ||
        ((strpos($string, '+0000')) !== FALSE) ||
        (((strpos($string, '+1')) ||
         (strpos($string, '+100')) ||
         (strpos($string, '+0100'))) !== FALSE) ||
        (((strpos($string, '+2')) ||
         (strpos($string, '+200')) ||
         (strpos($string, '+0200'))) !== FALSE) ||
        (((strpos($string, '+3')) ||
         (strpos($string, '+300')) ||
         (strpos($string, '+0300'))) !== FALSE) ||
        (((strpos($string, '+330')) ||
         (strpos($string, '+0330'))) !== FALSE) ||
        (((strpos($string, '+4')) ||
         (strpos($string, '+400')) ||
         (strpos($string, '+0400'))) !== FALSE) ||
        (((strpos($string, '+430')) ||
         (strpos($string, '+0430'))) !== FALSE) ||
        (((strpos($string, '+5')) ||
         (strpos($string, '+500')) ||
         (strpos($string, '+0500'))) !== FALSE) ||
        (((strpos($string, '+530')) ||
         (strpos($string, '+0530'))) !== FALSE) ||
        (((strpos($string, '+545')) ||
         (strpos($string, '+0545'))) !== FALSE) ||
        (((strpos($string, '+6')) ||
         (strpos($string, '+600')) ||
         (strpos($string, '+0600'))) !== FALSE) ||
        (((strpos($string, '+630')) ||
         (strpos($string, '+0630'))) !== FALSE) ||
        (((strpos($string, '+7')) ||
         (strpos($string, '+700')) ||
         (strpos($string, '+0700'))) !== FALSE) ||
        (((strpos($string, '+8')) ||
         (strpos($string, '+800')) ||
         (strpos($string, '+0800'))) !== FALSE) ||
        (((strpos($string, '+845')) ||
         (strpos($string, '+0845'))) !== FALSE) ||
        (((strpos($string, '+9')) ||
         (strpos($string, '+900')) ||
         (strpos($string, '+0900'))) !== FALSE) ||
        (((strpos($string, '+930')) ||
         (strpos($string, '+0930'))) !== FALSE) ||
        (((strpos($string, '+10')) ||
         (strpos($string, '+1000'))) !== FALSE) ||
        ((strpos($string, '+1030')) !== FALSE) ||
        (((strpos($string, '+11')) ||
         (strpos($string, '+1100'))) !== FALSE) ||
        ((strpos($string, '+1130')) !== FALSE) ||
        (((strpos($string, '+12')) ||
         (strpos($string, '+1200'))) !== FALSE) ||
        ((strpos($string, '+1245')) !== FALSE) ||
        (((strpos($string, '+13')) ||
         (strpos($string, '+1300'))) !== FALSE) ||
        (((strpos($string, '+14')) ||
         (strpos($string, '+1400'))) !== FALSE)) {
        $date['tz'] = TRUE;
    } else {
        $date['tz'] = FALSE;
    }

    $tzString = "/\bacdt|acst|adt|aedt|aest|akdt|akst|ast|awst|bst|cdt|cest|cet|cst|cxt|edt|eest|eet|est|gmt|haa|hac|hadt|hae|hap|har|hast|hat|hay|hna|hnc|hne|hnp|hnr|hnt|hny|ist|mdt|mesz|mez|mst|ndt|nft|nst|pdt|pst|utc|west|wet|wst\b/i";

    // If it found tz information, then strip any tz strings.
    if ($date['tz']) {
        $string = preg_replace($tzString, '', $string);
    } else if (preg_match($tzString, $string)) {
        // Otherwise, if it contains one of the strings, then set tz to true.
        $date['tz'] = TRUE;
    }

    $timeStamp = strtotime($string);

    if ($timeStamp == -1) {
        $pos = strpos($string, '19');
        $year = (integer) substr($string, $pos, 4);
        while (($year < 1900) &&
               ($pos !== FALSE)) {
            $pos = strpos($string, '19', $pos + 1);
            $year = (integer) substr($string, $pos, 4);
        }
        if (($year < 1970) &&
            ($year >= 1900)) {
            $newstring = substr_replace($string, $year + 100, $pos, 4);
            $timeStamp = strtotime($newstring);
        } else {
            $a = substr($string, 0, 10);
            $b = substr($string, 10, 9);
            $c = substr($string, 19, 5);
            $newstring = $a . $c . $b;
            $timeStamp = strtotime($newstring);
        }
        if ($timeStamp == -1) {
            return $date;
        }
    }

    $timeStamp = $timeStamp - (date("O") / 100 * 60 * 60) + ($GLOBALS['USER_SETTINGS']['timezone'] / 100 * 60 * 60);

    if (trim($dateFormat) == '') {
        $date['date'] = trim(date("M j, Y", $timeStamp));
    } else {
        $date['date'] = trim(date($dateFormat, $timeStamp));
    }
    if (trim($timeFormat) == '') {
        $date['time'] = trim(date("g:i A", $timeStamp));
    } else {
        $date['time'] = trim(date($timeFormat, $timeStamp));
    }

    return $date;
}


/**
* Va-Functions.inc.php :: ConvertDir()
*
* This function will convert a local directory into a network address.
*
* @param string $hostname
* @param string $dir
* @return string
*/
function ConvertDir($hostname,
                    $dir) {
    if (!is_string($hostname)) {
        $hostname = (string) $hostname;
    }
    if (!is_string($dir)) {
        $dir = (string) $dir;
    }

    // Change to forwardslashes.
    $dir = str_replace("\\", '/', $dir);

    // Get the drive letter.
    $drive = substr($dir, 0, 1);

    // Get the rest of the path.
    $line = substr($dir, 2);
    $path = '//' . trim($hostname) . '/' . trim($drive) . trim($line);

    // Change to backslashes
    $path = str_replace('/', "\\", $path);

    // Return the new path.
    return $path;
}


/**
* Va-Functions :: ConvertLongFilename()
*
* This function will convert a long filename into a DOS 8.3 filename.
* You can pass the directory the filename will reside in ($dir), and if it
* detects multiple files with the same name it can rename it without disrupting
* the exiting files.
*
* @param string $filename
* @param string $dir
* @return string
*/
function ConvertLongFilename($filename,
                             $dir = '') {
    if (!is_string($filename)) {
        $filename = (string) $filename;
    }
    if (!is_string($dir)) {
        $dir = (string) $dir;
    }

    $filename = trim($filename);

    // Replace any spaces with an underscore.
    $filename = str_replace(' ', '_', $filename);

    // Find the period.
    $pos = strrpos($filename, '.');

    // If there is no extension, then set it to the end of the file.
    if (($pos == 0) ||
        ($pos === FALSE)) {
        $pos = strlen($filename);
    }

    // Get the filename.
    $file = substr($filename, 0, $pos);
    if ($pos < strlen($filename)) {
        // Get the extension.
        $ext = substr($filename, $pos + 1);
        if (strlen($ext) > 3) {
            $ext = substr($ext, 0, 3);
        }
    } else {
        $ext = '';
    }

    // If the filename is 8.3 compliant create the filename variable.
    if (strlen($file) <= 8) {
        $filename = $file . '.' . $ext;
    } else {
        $file = substr($file, 0, 6);
        $filename = $file . '~1.' . $ext;
    }

    if ($dir != '') {
        $path = str_replace('/', "\\", $dir);
        $filepath = $path . '/' . $filename;

        // If that file already exists...
        if (file_exists($filepath)) {
            $file = substr($file, 0, 6);
            $filename = $file . '~1.' . $ext;
            $filepath = $path . '/' . $filename;
            for ($i = 1; (file_exists($filepath) && ($i < 99)); $i++) {
                if ($i == 10) {
                    $file = substr($file, 0, 5);
                }
                $filename = $file . '~' . $i . '.' . $ext;
                $filepath = $path . '/' . $filename;
            }
        }
    }

    return $filename;
}


/**
* Va-Functions.inc.php :: ConvertUnc()
*
* This function will convert a UNC path into a local directory. For Windows
* systems only.
*
* @param string $unc
* @return string
*/
function ConvertUnc($unc) {
    if (!is_string($unc)) {
        $unc = (string) $unc;
    }

    // Change to forwardslashes.
    $unc = str_replace("\\", '/', $unc);

    // Get the position of the drive letter.
    $pos = strpos($unc, '/', 2);

    // Get the drive letter.
    $drive = substr($unc, $pos + 1, 1);

    // Get the rest of the path.
    $dir = substr($unc, $pos + 3);
    if ($dir == '') {
        $path = $drive . ':';
    } else {
        $path = $drive . ':/' . $dir;
    }

    // Change to backslashes.
    $path = str_replace('/', "\\", $path);

    // Return the new path.
    return $path;
}


/**
* Va-Functions.inc.php :: CustomErrorHandler()
*
* This is a custom error handler in case all hell breaks loose.
*
* @param integer $errNumber
* @param string $errDesc
* @param string $errFile
* @param integer $errLine
*/
function CustomErrorHandler($errNumber,
                            $errDesc,
                            $errFile,
                            $errLine) {
    if (!is_int($errNumber)) {
        $errNumber = (integer) $errNumber;
    }
    if (!is_string($errDesc)) {
        $errDesc = (string)  $errDesc;
    }
    if (!is_string($errFile)) {
        $errFile = (string)  $errFile;
    }
    if (!is_int($errLine)) {
        $errLine = (integer) $errLine;
    }

    global $VERSION;
    global $DEBUG_MAIL;
    global $DEBUG_EMAIL;
    global $DEBUG_SMTP_SERVER;

    // Send an email to the VADV-PHP bug report address.
    if ($DEBUG_MAIL == TRUE) {
        if (gethostbyname($DEBUG_SMTP_SERVER) != $DEBUG_SMTP_SERVER) {
            $phpVersion = phpversion();
            $currentURL = GetCurrentURL();

            $pos = strrpos($errFile, chr(92)) + 1;
            $file = substr($errFile, $pos);

            // Create error message.
            $error = <<<EOD
Error number: $errNumber
Description:  $errDesc
Script name:  $file
Line number:  $errLine

URL:          $currentURL
PHP version:  $phpVersion
EOD;

            ini_set('SMTP', $DEBUG_SMTP_SERVER);
            ini_set('sendmail_from', $DEBUG_EMAIL);
            $headers = "Date: " . date("r") . "\r\n";
            $headers .= "Reply-To: " . $DEBUG_EMAIL . "\r\n";
            $headers .= "MIME-Version: 1.0\r\n";
            $headers .= "Content-Type: text/plain; charset=us-ascii\r\n";
            $headers .= "Content-Transfer-Encoding: 7bit\r\n";
            $headers .= "X-Sender: Bug Report<" . $DEBUG_EMAIL . ">\r\n";
            $headers .= "X-Mailer: PHP/" . $phpVersion . "\r\n";
            $headers .= "X-Priority: 3\r\n";
            $headers .= "Return-Path: <" . $DEBUG_EMAIL . ">\r\n";

            mail($DEBUG_EMAIL,
                 '[NON-FATAL] VADV-PHP ERROR (' . $VERSION . ')',
                 'http://' . $_SERVER['SERVER_NAME'] . "\n" . $error,
                 $headers);
            ini_restore('SMTP');
            ini_restore('sendmail_from');
        }
    }

    // Display error message to user.
    print "<h2>Unable to display page.</h2>\n";
    print "<p>An error occurred while processing your request. The site administrator has been notified.</p>\n";
    print "<p>We apologize for the inconvenience.</p>";
    VA_Die('This page will attempt to be redirected in 5 seconds.');
}


/**
* Va-Functions.inc.php :: DetectSpider()
*
* This function detects if the current user client is a known search engine
* spider. If so, it returns TRUE, else it returns FALSE.
*
* @return boolean
*/
function DetectSpider() {
    // The following search engine identifier information was obtained from:
    // http://www.jafsoft.com/searchengines/webbots.html

    $result = FALSE;

    if (isset($_SERVER['HTTP_USER_AGENT'])) {
        $referer = trim($_SERVER['HTTP_USER_AGENT']);

        if ($referer != '') {
            // Webbots...
            if (
               // Generic list...
                (stristr($referer, 'bot')                           !== FALSE) ||
                (stristr($referer, 'crawl')                         !== FALSE) ||
                (stristr($referer, 'spider')                        !== FALSE) ||
               // V32...
                (stristr($referer, 'V32Update')                     !== FALSE) ||
                (stristr($referer, 'VADV32')                        !== FALSE) ||
               // Bot list...
                (stristr($referer, 'AbachoBOT')                     !== FALSE) ||
                (stristr($referer, 'ABCdatos')                      !== FALSE) ||
                (stristr($referer, 'AESOP_com_SpiderMan')           !== FALSE) ||
                (stristr($referer, 'ah-ha.com')                     !== FALSE) ||
                (stristr($referer, 'ia_archiver')                   !== FALSE) ||
                (stristr($referer, 'Scooter')                       !== FALSE) ||
                (stristr($referer, 'Mercator')                      !== FALSE) ||
                (stristr($referer, 'Scooter2_Mercator_3')           !== FALSE) ||
                (stristr($referer, 'roach.smo.av.com')              !== FALSE) ||
                (stristr($referer, 'Tv<nn>_Merc_resh_26_1_D')       !== FALSE) ||
                (stristr($referer, 'AltaVista')                     !== FALSE) ||
                (stristr($referer, 'FAST-WebCrawler')               !== FALSE) ||
                (stristr($referer, 'Wget')                          !== FALSE) ||
                (stristr($referer, 'Acoon Robot')                   !== FALSE) ||
                (stristr($referer, 'antibot')                       !== FALSE) ||
                (stristr($referer, 'Atomz')                         !== FALSE) ||
                (stristr($referer, 'AxmoRobot')                     !== FALSE) ||
                (stristr($referer, 'Buscaplus Robi')                !== FALSE) ||
                (stristr($referer, 'CanSeek')                       !== FALSE) ||
                (stristr($referer, 'ChristCRAWLER')                 !== FALSE) ||
                (stristr($referer, 'Clushbot')                      !== FALSE) ||
                (stristr($referer, 'Crawler')                       !== FALSE) ||
                (stristr($referer, 'DaAdLe.com ROBOT')              !== FALSE) ||
                (stristr($referer, 'RaBot')                         !== FALSE) ||
                (stristr($referer, 'DeepIndex')                     !== FALSE) ||
                (stristr($referer, 'DittoSpyder')                   !== FALSE) ||
                (stristr($referer, 'Jack')                          !== FALSE) ||
                (stristr($referer, 'Speedy Spider')                 !== FALSE) ||
                (stristr($referer, 'ArchitextSpider')               !== FALSE) ||
                (stristr($referer, 'ArchitectSpider')               !== FALSE) ||
                (stristr($referer, 'EuripBot')                      !== FALSE) ||
                (stristr($referer, 'Arachnoidea')                   !== FALSE) ||
                (stristr($referer, 'EZResult')                      !== FALSE) ||
                (stristr($referer, 'Fast PartnerSite')              !== FALSE) ||
                (stristr($referer, 'FAST Data Search')              !== FALSE) ||
                (stristr($referer, 'KIT-Fireball')                  !== FALSE) ||
                (stristr($referer, 'FyberSearch')                   !== FALSE) ||
                (stristr($referer, 'GalaxyBot')                     !== FALSE) ||
                (stristr($referer, 'geckobot')                      !== FALSE) ||
                (stristr($referer, 'GenCrawler')                    !== FALSE) ||
                (stristr($referer, 'GeonaBot')                      !== FALSE) ||
                (stristr($referer, 'Googlebot')                     !== FALSE) ||
                (stristr($referer, 'moget')                         !== FALSE) ||
                (stristr($referer, 'Aranha')                        !== FALSE) ||
                (stristr($referer, 'Slurp')                         !== FALSE) ||
                (stristr($referer, 'Toutatis')                      !== FALSE) ||
                (stristr($referer, 'Hubater')                       !== FALSE) ||
                (stristr($referer, 'IlTrovatore-Setaccio')          !== FALSE) ||
                (stristr($referer, 'IncyWincy')                     !== FALSE) ||
                (stristr($referer, 'UltraSeek')                     !== FALSE) ||
                (stristr($referer, 'InfoSeek')                      !== FALSE) ||
                (stristr($referer, 'Mole2')                         !== FALSE) ||
                (stristr($referer, 'MP3Bot')                        !== FALSE) ||
                (stristr($referer, 'ip3000')                        !== FALSE) ||
                (stristr($referer, 'kuloko-bot')                    !== FALSE) ||
                (stristr($referer, 'LNSpiderguy')                   !== FALSE) ||
                (stristr($referer, 'Linknzbot')                     !== FALSE) ||
                (stristr($referer, 'lookbot')                       !== FALSE) ||
                (stristr($referer, 'MantraAgent')                   !== FALSE) ||
                (stristr($referer, 'NetResearchServer')             !== FALSE) ||
                (stristr($referer, 'Lycos')                         !== FALSE) ||
                (stristr($referer, 'JoocerBot')                     !== FALSE) ||
                (stristr($referer, 'HenryTheMiragoRobot')           !== FALSE) ||
                (stristr($referer, 'MojeekBot')                     !== FALSE) ||
                (stristr($referer, 'mozDex')                        !== FALSE) ||
                (stristr($referer, 'MSNBOT')                        !== FALSE) ||
                (stristr($referer, 'Navadoo Crawler')               !== FALSE) ||
                (stristr($referer, 'Gulliver')                      !== FALSE) ||
                (stristr($referer, 'ObjectsSearch')                 !== FALSE) ||
                (stristr($referer, 'OnetSzukaj')                    !== FALSE) ||
                (stristr($referer, 'PicoSearch')                    !== FALSE) ||
                (stristr($referer, 'PJspider')                      !== FALSE) ||
                (stristr($referer, 'DIIbot')                        !== FALSE) ||
                (stristr($referer, 'nttdirectory_robot')            !== FALSE) ||
                (stristr($referer, 'griffon')                       !== FALSE) ||
                (stristr($referer, 'maxbot')                        !== FALSE) ||
                (stristr($referer, 'gazz')                          !== FALSE) ||
                (stristr($referer, 'NationalDirectory-SuperSpider') !== FALSE) ||
                (stristr($referer, 'naver')                         !== FALSE) ||
                (stristr($referer, 'noxtrumbot')                    !== FALSE) ||
                (stristr($referer, 'Openfind')                      !== FALSE) ||
                (stristr($referer, 'Openbot')                       !== FALSE) ||
                (stristr($referer, 'psbot')                         !== FALSE) ||
                (stristr($referer, 'CrawlerBoy')                    !== FALSE) ||
                (stristr($referer, 'QweeryBot')                     !== FALSE) ||
                (stristr($referer, 'AlkalineBOT')                   !== FALSE) ||
                (stristr($referer, 'StackRambler')                  !== FALSE) ||
                (stristr($referer, 'SeznamBot')                     !== FALSE) ||
                (stristr($referer, 'Search-10')                     !== FALSE) ||
                (stristr($referer, 'Fluffy')                        !== FALSE) ||
                (stristr($referer, 'Scrubby')                       !== FALSE) ||
                (stristr($referer, 'asterias')                      !== FALSE) ||
                (stristr($referer, 'speedfind')                     !== FALSE) ||
                (stristr($referer, 'Kototoi')                       !== FALSE) ||
                (stristr($referer, 'SearchByUsa')                   !== FALSE) ||
                (stristr($referer, 'Searchspider')                  !== FALSE) ||
                (stristr($referer, 'SightQuestBot')                 !== FALSE) ||
                (stristr($referer, 'Spider_Monkey')                 !== FALSE) ||
                (stristr($referer, 'Surfnomore')                    !== FALSE) ||
                (stristr($referer, 'teoma')                         !== FALSE) ||
                (stristr($referer, 'Teradex_Mapper')                !== FALSE) ||
                (stristr($referer, 'ESISmartSpider')                !== FALSE) ||
                (stristr($referer, 'TraficDublu')                   !== FALSE) ||
                (stristr($referer, 'Tutorial Crawler')              !== FALSE) ||
                (stristr($referer, 'UK Searcher')                   !== FALSE) ||
                (stristr($referer, 'Vivante')                       !== FALSE) ||
                (stristr($referer, 'appie')                         !== FALSE) ||
                (stristr($referer, 'Nazilla')                       !== FALSE) ||
                (stristr($referer, 'WebWombat')                     !== FALSE) ||
                (stristr($referer, 'marvin')                        !== FALSE) ||
                (stristr($referer, 'MuscatFerret')                  !== FALSE) ||
                (stristr($referer, 'WhizBang')                      !== FALSE) ||
                (stristr($referer, 'ZyBorg')                        !== FALSE) ||
                (stristr($referer, 'WIRE WebRefiner')               !== FALSE) ||
                (stristr($referer, 'WSCbot')                        !== FALSE) ||
                (stristr($referer, 'Yandex')                        !== FALSE) ||
                (stristr($referer, 'Yellopet-Spider')               !== FALSE) ||
                (stristr($referer, 'Findexa Crawler')               !== FALSE) ||
                (stristr($referer, 'YBSbot')                        !== FALSE) ||
                (stristr($referer, 'Iron33')                        !== FALSE) ||
               // Research bots...
                (stristr($referer, 'citenikbot')                    !== FALSE) ||
                (stristr($referer, 'CLIPS-index')                   !== FALSE) ||
                (stristr($referer, 'cosmos')                        !== FALSE) ||
                (stristr($referer, 'D2KWebCrawler')                 !== FALSE) ||
                (stristr($referer, 'DiaGem')                        !== FALSE) ||
                (stristr($referer, 'Digimarc')                      !== FALSE) ||
                (stristr($referer, 'print!')                         !== FALSE) ||
                (stristr($referer, 'FinaleRobot')                   !== FALSE) ||
                (stristr($referer, 'Ideare - SignSite')             !== FALSE) ||
                (stristr($referer, 'GentleSpider')                  !== FALSE) ||
                (stristr($referer, 'Gulper Web Bot')                !== FALSE) ||
                (stristr($referer, 'Iarbin')                        !== FALSE) ||
                (stristr($referer, 'IRLbot')                        !== FALSE) ||
                (stristr($referer, 'KnowItAll')                     !== FALSE) ||
                (stristr($referer, 'MJ12bot')                       !== FALSE) ||
                (stristr($referer, 'OntoSpider')                    !== FALSE) ||
                (stristr($referer, 'sherlock_spider')               !== FALSE) ||
                (stristr($referer, 'S.T.A.L.K.E.R.')                !== FALSE) ||
                (stristr($referer, 'Steeler')                       !== FALSE) ||
                (stristr($referer, 'ru-robot')                      !== FALSE) ||
                (stristr($referer, 'USyd-NLP-Spider')               !== FALSE) ||
                (stristr($referer, 'WebGather')                     !== FALSE) ||
                (stristr($referer, 'xyro')                          !== FALSE) ||
                (stristr($referer, 'Zao/')                          !== FALSE) ||
                (stristr($referer, 'Zao-Crawler')                   !== FALSE) ||
               // Bots not listed at jafsoft.com...
                (stristr($referer, 'Baiduspider')                   !== FALSE) ||
                (stristr($referer, 'BecomeBot')                     !== FALSE) ||
                (stristr($referer, 'Exabot')                        !== FALSE) ||
                (stristr($referer, 'Fraunhofer ITWM')               !== FALSE) ||
                (stristr($referer, 'Gigaroo')                       !== FALSE) ||
                (stristr($referer, 'libwww-perl')                   !== FALSE) ||
                (stristr($referer, 'NextGenSearchBot')              !== FALSE) ||
                (stristr($referer, 'OutfoxBot')                     !== FALSE) ||
                (stristr($referer, 'RufusBot')                      !== FALSE) ||
                (stristr($referer, 'sogou spider')                  !== FALSE) ||
                (stristr($referer, 'Webaroo')                       !== FALSE) ||
                (stristr($referer, 'Whirlpool Web Engine')          !== FALSE) ||
                (stristr($referer, 'WhoIsBase')                     !== FALSE)) {
               // It is a spider...
               $result = TRUE;
            }
        }
    }

    return $result;
}


/**
* Va-Functions.inc.php :: FixHeartColors()
*
* This routine will take $string and replace any heart codes within the string
* with html color codes.
*
* @param string $string
* @return string
*/
function FixHeartColors($string,
                        $default = '') {
    if (!is_string($string)) {
        $string = (string) $string;
    }
    if (!is_string($default)) {
        $default = (string) $default;
    }

    // Check to see if the variable was passed, if not then return the string.
    if (trim($string) == '') {
        return $string;
    }

    // Temporary flag for span tagss.
    $spanFlag = 0;

    // Search for hearts.
    for ($position = 0; is_int($position);) {
        $position = strpos($string, chr(3), 0);

        // If found a heart...
        if (is_int($position)) {
            $code = substr($string, $position + 1, 1);
            switch (strtoupper($code)) {
                case '0':
                    $color = '#ADAAAD';
                    break;
                case '1':
                    $color = '#52FFFF';
                    break;
                case '2':
                    $color = '#FFFF52';
                    break;
                case '3':
                    $color = '#AD00AD';
                    break;
                case '4':
                    $color = '#FFFFFF';
                    break;
                case '5':
                    $color = '#52FF52';
                    break;
                case '6':
                    $color = '#FF5552';
                    break;
                case '7':
                    $color = '#5255FF';
                    break;
                case '8':
                    $color = '#AD5500';
                    break;
                case '9':
                    $color = '#00AAAD';
                    break;
                case 'A':
                    $color = '#00AA00';
                    break;
                case 'B':
                    $color = '#FF55FF';
                    break;
                case 'C':
                    $color = '#FFFFFF';
                    break;
                case 'D':
                    $color = '#FFFF52';
                    break;
                case 'E':
                    $color = '#52FFFF';
                    break;
                case 'F':
                    $color = '#ADAAAD';
                    break;
                case 'G':
                    $color = '#5255FF';
                    break;
                case 'H':
                    $color = '#FF5552';
                    break;
                case 'I':
                    $color = '#5255FF';
                    break;
                case 'J':
                    $color = '#FFFF52';
                    break;
                case 'K':
                    $color = '#52FFFF';
                    break;
                case 'L':
                    $color = '#ADAAAD';
                    break;
                case 'M':
                    $color = '#FFFFFF';
                    break;
                case 'N':
                    $color = '#ADAAAD';
                    break;
                case 'O':
                    $color = '#ADAAAD';
                    break;
                case 'P':
                    $color = '#AD0000';
                    break;
                case 'Q':
                    $color = '#0000AD';
                    break;
                case 'R':
                    $color = '#FFFFFF';
                    break;
                case 'S':
                    $color = '#FF5552';
                    break;
                case 'T':
                    $color = '#5255FF';
                    break;
                case 'U':
                    $color = '#FFFFFF';
                    break;
                case 'V':
                    $color = '#ADAAAD';
                    break;
                case 'W':
                    $color = '#ADAAAD';
                    break;
                case 'X':
                    $color = '#ADAAAD';
                    break;
                case 'Y':
                    $color = '#FFFFFF';
                    break;
                case 'Z':
                    $color = '#FFFFFF';
                    break;
                case '[':
                    $color = '#525552';
                    break;
                default:
                    $color = $default;
                    break;
            }

            if ($color != '') {
                // Insert the new information into the string.
                $a = substr($string, 0, $position);
                $b = substr($string, $position + 2);
                $string = $a . '<span style="color: ' . $color . '">' . $b;

                // Change span flag.
                $spanFlag = 1;

                // Check for next heart code for closing tag.
                $position = strpos($string, chr(3), 0);
                if (is_int($position)) {
                    $a = substr($string, 0, $position);
                    $b = substr($string, $position);
                    $string = $a . '</span>' . $b;
                }
            } else {
                $a = substr($string, 0, $position);
                $b = substr($string, $position + 1);
                $string = $a . $b;

                $spanFlag = 0;
            }
        }
    }

    if ($spanFlag == 1) {
        $string = $string . '</span>';
    }

    // Return the new string.
    return $string;
}


/**
* Va-Functions.inc.php :: GetAccFlags()
*
* This function will take the long integer value of the access flags and
* convert it into a string format. The returned letters are all uppercase.
*
* @param integer $accFlagsInt
* @return string
*/
function GetAccFlags($accFlagsInt) {
    if (!is_int($accFlagsInt)) {
        $accFlagsInt = (integer) $accFlagsInt;
    }

    $flags = '';

    if ($accFlagsInt == 0) {
        return $flags;
    }

    for ($i = 0; $i <= 25; $i++) {
        if ($accFlagsInt & (pow(2, $i))) {
            $flags = $flags . chr(65 + $i);
        }
    }

    return $flags;
}


/**
* Va-Functions.inc.php :: GetChar()
*
* This function will take the given file string and extract the character value
* by using its position.
*
* @param string $fileString
* @param integer $position
* @return integer
*/
function GetChar($fileString,
                 $position) {
    if (!is_string($fileString)) {
        $fileString = (string)  $fileString;
    }
    if (!is_int($position)) {
        $position = (integer) $position;
    }

    // Check to see if the variable was passed, if not then return nothing.
    if ($fileString == '') {
        return 0;
    }

    // Check to see if the position is larger than the string length.
    if ($position > strlen($fileString)) {
        return 0;
    }

    // Reduce the position by one.
    $position--;

    // Get the first ascii value.
    $a = ord(substr($fileString, $position, 1));

    // Return the integer.
    return (integer) $a;
}


/**
* Va-Functions.inc.php :: GetCurrentURL()
*
* This function simply returns the full URL of the current script. If $query is
* set to 0, then it will return the current URL without the query string.
*
* @param boolean $query
* @return string
*/
function GetCurrentURL($query = 1,
                       $baseurl = 0,
                       $allowssl = 1,
                       $forcessl = 0) {
    if (!is_bool($query)) {
        $query = (boolean) $query;
    }
    if (!is_bool($baseurl)) {
        $baseurl = (boolean) $baseurl;
    }
    if (!is_bool($allowssl)) {
        $allowssl = (boolean) $allowssl;
    }
    if (!is_bool($forcessl)) {
        $forcessl = (boolean) $forcessl;
    }

    if (!isset($_SERVER['HTTP_HOST'])) {
        return '';
    }

    if (($forcessl) ||
        (($allowssl) &&
         (IsHttps()))) {
        $scheme = 'https://';
    } else {
        $scheme = 'http://';
    }

    $script = $_SERVER['SCRIPT_NAME'];
    if ((strpos($script, 'php.exe') !== FALSE) ||
        (strpos($script, 'php-cgi.exe') !== FALSE)) {
        $script = $_SERVER['PHP_SELF'];
    }

    if ($baseurl) {
        $script = dirname($script);
    }
    $url = $scheme . $_SERVER['HTTP_HOST'] . $script;

    if (($query) &&
        (!$baseurl) &&
        (isset($_SERVER['QUERY_STRING'])) &&
        (trim($_SERVER['QUERY_STRING']) != '')) {
        $url = $url . '?' . $_SERVER['QUERY_STRING'];
    }

    // Return the url.
    return $url;
}


/**
* Va-Functions.inc.php :: GetInteger()
*
* This function will take the given file string and extract the integer value
* by using its position. If you set $unsigned to 1, then you can extract
* values up to 65535 that are positive only.
*
* @param string $fileString
* @param integer $position
* @param boolean $unsigned
* @return integer
*/
function GetInteger($fileString,
                    $position,
                    $unsigned = 0) {
    if (!is_string($fileString)) {
        $fileString = (string)  $fileString;
    }
    if (!is_int($position)) {
        $position = (integer) $position;
    }
    if (!is_bool($unsigned)) {
        $unsigned = (boolean) $unsigned;
    }

    // Check to see if the variable was passed, if not then return nothing.
    if ($fileString == '') {
        return 0;
    }

    // Check to see if the position is larger than the string length.
    if ($position > strlen($fileString)) {
        return 0;
    }

    // Reduce the position by one.
    $position--;

    // Get the first ascii value.
    $a = ord(substr($fileString, $position, 1));

    // Move to the next position.
    $position++;

    // Get the second ascii value.
    $b = 256 * ord(substr($fileString, $position, 1));

    // Create the integer.
    $integer = intval($a + $b);

    // Check for negative value.
    if ((!$unsigned) &&
        ($integer > 32767)) {
        $integer = $integer - 65535 - 1;
    }

    // Return the integer.
    return $integer;
}


/**
* Va-Functions.inc.php :: GetLongInt()
*
* This function will take the given file string and extract the long integer
* value by using its position. If you set $unsigned to 1, then you can extract
* values up to 4294967295 that are positive only.
*
* @param string $fileString
* @param integer $position
* @param boolean $unsigned
* @return integer
*/
function GetLongInt($fileString,
                    $position,
                    $unsigned = 0) {
    if (!is_string($fileString)) {
        $fileString = (string)  $fileString;
    }
    if (!is_int($position)) {
        $position = (integer) $position;
    }
    if (!is_bool($unsigned)) {
        $unsigned = (boolean) $unsigned;
    }

    // Check to see if the variable was passed, if not then return nothing.
    if ($fileString == '') {
        return 0;
    }

    // Check to see if the position is larger than the string length.
    if ($position > strlen($fileString)) {
        return 0;
    }

    // Reduce the position by one.
    $position--;

    // Get the first ascii value.
    $a = ord(substr($fileString, $position, 1));

    // Move to the next position.
    $position++;

    // Get the second ascii value.
    $b = 256 * ord(substr($fileString, $position, 1));

    // Move to the next position.
    $position++;

    // Get the third ascii value.
    $c = 65536 * ord(substr($fileString, $position, 1));

    // Move to the next position.
    $position++;

    // Get the fourth ascii value.
    $d = 16777216 * ord(substr($fileString, $position, 1));

    // Create the integer.
    $integer = intval($a + $b + $c + $d);

    // Check for negative value.
    if ((!$unsigned) &&
        ($integer > 2147483647)) {
        $integer = $integer - 2147483647 - 1;
    }

    // Return the long integer.
    return $integer;
}


/**
* Va-Functions.inc.php :: GetSingle()
*
* This function will take the given file string and extract the single precison
* value by using its position. If $round is set to any postitive value, then
* the answer will be rounded off to that many decimal places.
*
* This function does not work 100% correctly, especially for larger values. It
* correctly extracts the VADV values, but does not reliably calculate other
* values that may equal or exceed thousands.
*
* Information to do these calculations were taken from:
* http://www.learnbinary.com/binary.html
* http://www.aplawrence.com/Basics/floatingpoint.html
*
* @param string $fileString
* @param integer $position
* @param integer $round
* @return float
*/
function GetSingle($fileString,
                   $position,
                   $round = -1) {
    if (!is_string($fileString)) {
        $fileString = (string)  $fileString;
    }
    if (!is_int($position)) {
        $position = (integer) $position;
    }
    if (!is_int($round)) {
        $round = (integer) $round;
    }

    // Save the current precision setting...
    $precision = ini_get('precision');
    ini_set('precision', 40);

    // Extract the four bytes, convert to binary and pad them with zeros if needed.
    $a = str_pad(decbin(GetChar($fileString, $position)), 8, 0, STR_PAD_LEFT);
    $b = str_pad(decbin(GetChar($fileString, $position + 1)), 8, 0, STR_PAD_LEFT);
    $c = str_pad(decbin(GetChar($fileString, $position + 2)), 8, 0, STR_PAD_LEFT);
    $d = str_pad(decbin(GetChar($fileString, $position + 3)), 8, 0, STR_PAD_LEFT);

    // Put together the 32 bit binary number.
    $long = $d . $c . $b . $a;

    // Break into separate parts.
    $sign = substr($long, 0, 1);
    $exp = bindec(substr($long, 1, 8)) - 127;
    $mant = 1 . '.' . substr($long, 9);

    // Calculate the binary answer.
    // If positive, do the easy way.
    if ($exp >= 0) {
        $float = $mant * decbin(pow(2, $exp));
    } else {
        $float = $mant * (10 / (str_pad(1, 2 - $exp, 0, STR_PAD_RIGHT)));
    }

    // If the negative exp is too long, PHP does it in abbreviated notation,
    // so I'm converting it to a string in order to do the rest of the
    // calculations.
    $float = strtoupper($float);
    $pos = strpos($float, 'E');
    if ($pos !== FALSE) {
        $pol = substr($float, $pos, 1);
        $pow = (integer) substr($float, $pos + 1);
        if ($pow < 0) {
            $pow = substr($pow, 1);
        }
        $float = substr($float, 0, $pos);
        $float = str_replace('.', '', $float);
        if ($pol == '-') {
            $float = '0.' . str_pad($float, $pow - 1 + strlen($float), '0', STR_PAD_LEFT);
        } else {
            if ($pow > strlen($float)) {
                $float = str_pad($float, $pow + 1, '0', STR_PAD_RIGHT);
            } else {
                $float = substr($float, 0, $pow) . '.' . substr($float, $pow + 1);
            }
        }
    }

    // Split the number into an integer and decimal.
    $array = explode('.', $float);
    $integer = bindec($array[0]);
    if (!isset($array[1])) {
        $array[1] = '';
    }

    // Calculate the fractions of the decimal part (which is in binary).
    // This is the longer, harder way...
    $decimal = 0;
    $digit = 1;
    $fraction = 2;
    for ($i = 0; $digit != ''; $i++) {
        $digit = substr($array[1], $i, 1);
        if ($digit != '') {
            $decimal = $decimal + ((1 / $fraction) * $digit);
        }
        $fraction = $fraction * 2;
    }

    // This is the shorter, more compact way...
    // $decimal = bindec($array[1]) / pow(2, strlen($array[1]));

    // Put the two together to get your answer.
    $float = $integer + $decimal;

    // Round the answer if specified.
    if ($round >= 0) {
        $float = round($float, $round);
    } else {
        $float = round($float, 10);
    }

    // Add on the sign if negative.
    if ($sign == 1) {
        $float = 0 - $float;
    }

    // Set the precision back.
    ini_set('precision', $precision);

    // Return the floating point number.
    return (float) $float;
}


/**
* Va-Functions.inc.php :: GetString()
*
* This function will take the given file string and extract the string value
* by using its position and length.
*
* @param string $fileString
* @param integer $position
* @param integer $length
* @return string
*/
function GetString($fileString,
                   $position,
                   $length) {
    if (!is_string($fileString)) {
        $fileString = (string)  $fileString;
    }
    if (!is_int($position)) {
        $position = (integer) $position;
    }
    if (!is_int($length)) {
        $length = (integer) $length;
    }

    // Check to see if the variable was passed, if not then return nothing.
    if ($fileString == '') {
        return '';
    }

    // Check to see if the position is larger than the string length.
    if ($position > strlen($fileString)) {
        return '';
    }

    // Reduce the position by one.
    $position--;
    $string = trim(substr($fileString, $position, $length));

    return (string) $string;                // Return the string.
}


/**
* Va-Functions.inc.php :: HTMLcp437()
*
* Convert high ASCII chars to the proper HTML entities.
*
* @param string $string
* @return string
*/
function HTMLcp437($string) {
    // http://www.kostis.net/charsets/cp437.htm
    if (!is_string($string)){
        $string = (string) $string;
    }

    $string = str_replace(chr(128), "&#199;", $string);
    $string = str_replace(chr(129), "&#252;", $string);
    $string = str_replace(chr(130), "&#233;", $string);
    $string = str_replace(chr(131), "&#226;", $string);
    $string = str_replace(chr(132), "&#228;", $string);
    $string = str_replace(chr(133), "&#224;", $string);
    $string = str_replace(chr(134), "&#229;", $string);
    $string = str_replace(chr(135), "&#231;", $string);
    $string = str_replace(chr(136), "&#234;", $string);
    $string = str_replace(chr(137), "&#235;", $string);
    $string = str_replace(chr(138), "&#232;", $string);
    $string = str_replace(chr(139), "&#239;", $string);
    $string = str_replace(chr(140), "&#238;", $string);
    $string = str_replace(chr(141), "&#236;", $string);
    $string = str_replace(chr(142), "&#196;", $string);
    $string = str_replace(chr(143), "&#197;", $string);
    $string = str_replace(chr(144), "&#201;", $string);
    $string = str_replace(chr(145), "&#230;", $string);
    $string = str_replace(chr(146), "&#198;", $string);
    $string = str_replace(chr(147), "&#244;", $string);
    $string = str_replace(chr(148), "&#246;", $string);
    $string = str_replace(chr(149), "&#242;", $string);
    $string = str_replace(chr(150), "&#251;", $string);
    $string = str_replace(chr(151), "&#249;", $string);
    $string = str_replace(chr(152), "&#255;", $string);
    $string = str_replace(chr(153), "&#214;", $string);
    $string = str_replace(chr(154), "&#220;", $string);
    $string = str_replace(chr(155), "&#162;", $string);
    $string = str_replace(chr(156), "&#163;", $string);
    $string = str_replace(chr(157), "&#165;", $string);
    $string = str_replace(chr(158), "&#8359;", $string);
    $string = str_replace(chr(159), "&#402;", $string);
    $string = str_replace(chr(160), "&#225;", $string);
    $string = str_replace(chr(161), "&#237;", $string);
    $string = str_replace(chr(162), "&#243;", $string);
    $string = str_replace(chr(163), "&#250;", $string);
    $string = str_replace(chr(164), "&#241;", $string);
    $string = str_replace(chr(165), "&#209;", $string);
    $string = str_replace(chr(166), "&#170;", $string);
    $string = str_replace(chr(167), "&#186;", $string);
    $string = str_replace(chr(168), "&#191;", $string);
    $string = str_replace(chr(169), "&#8976;", $string);
    $string = str_replace(chr(170), "&#172;", $string);
    $string = str_replace(chr(171), "&#189;", $string);
    $string = str_replace(chr(172), "&#188;", $string);
    $string = str_replace(chr(173), "&#161;", $string);
    $string = str_replace(chr(174), "&#171;", $string);
    $string = str_replace(chr(175), "&#187;", $string);
    $string = str_replace(chr(176), "&#9617;", $string);
    $string = str_replace(chr(177), "&#9618;", $string);
    $string = str_replace(chr(178), "&#9619;", $string);
    $string = str_replace(chr(179), "&#9474;", $string);
    $string = str_replace(chr(180), "&#9508;", $string);
    $string = str_replace(chr(181), "&#9569;", $string);
    $string = str_replace(chr(182), "&#9570;", $string);
    $string = str_replace(chr(183), "&#9558;", $string);
    $string = str_replace(chr(184), "&#9557;", $string);
    $string = str_replace(chr(185), "&#9571;", $string);
    $string = str_replace(chr(186), "&#9553;", $string);
    $string = str_replace(chr(187), "&#9559;", $string);
    $string = str_replace(chr(188), "&#9565;", $string);
    $string = str_replace(chr(189), "&#9564;", $string);
    $string = str_replace(chr(190), "&#9563;", $string);
    $string = str_replace(chr(191), "&#9488;", $string);
    $string = str_replace(chr(192), "&#9492;", $string);
    $string = str_replace(chr(193), "&#9524;", $string);
    $string = str_replace(chr(194), "&#9516;", $string);
    $string = str_replace(chr(195), "&#9500;", $string);
    $string = str_replace(chr(196), "&#9472;", $string);
    $string = str_replace(chr(197), "&#9532;", $string);
    $string = str_replace(chr(198), "&#9566;", $string);
    $string = str_replace(chr(199), "&#9567;", $string);
    $string = str_replace(chr(200), "&#9562;", $string);
    $string = str_replace(chr(201), "&#9556;", $string);
    $string = str_replace(chr(202), "&#9577;", $string);
    $string = str_replace(chr(203), "&#9574;", $string);
    $string = str_replace(chr(204), "&#9568;", $string);
    $string = str_replace(chr(205), "&#9552;", $string);
    $string = str_replace(chr(206), "&#9580;", $string);
    $string = str_replace(chr(207), "&#9575;", $string);
    $string = str_replace(chr(208), "&#9576;", $string);
    $string = str_replace(chr(209), "&#9572;", $string);
    $string = str_replace(chr(210), "&#9573;", $string);
    $string = str_replace(chr(211), "&#9561;", $string);
    $string = str_replace(chr(212), "&#9560;", $string);
    $string = str_replace(chr(213), "&#9554;", $string);
    $string = str_replace(chr(214), "&#9555;", $string);
    $string = str_replace(chr(215), "&#9579;", $string);
    $string = str_replace(chr(216), "&#9578;", $string);
    $string = str_replace(chr(217), "&#9496;", $string);
    $string = str_replace(chr(218), "&#9484;", $string);
    $string = str_replace(chr(219), "&#9608;", $string);
    $string = str_replace(chr(220), "&#9604;", $string);
    $string = str_replace(chr(221), "&#9612;", $string);
    $string = str_replace(chr(222), "&#9616;", $string);
    $string = str_replace(chr(223), "&#9600;", $string);
    $string = str_replace(chr(224), "&#945;", $string);
    $string = str_replace(chr(225), "&#223;", $string);
    $string = str_replace(chr(226), "&#915;", $string);
    $string = str_replace(chr(227), "&#960;", $string);
    $string = str_replace(chr(228), "&#931;", $string);
    $string = str_replace(chr(229), "&#963;", $string);
    $string = str_replace(chr(230), "&#181;", $string);
    $string = str_replace(chr(231), "&#964;", $string);
    $string = str_replace(chr(232), "&#934;", $string);
    $string = str_replace(chr(233), "&#920;", $string);
    $string = str_replace(chr(234), "&#937;", $string);
    $string = str_replace(chr(235), "&#948;", $string);
    $string = str_replace(chr(236), "&#8734;", $string);
    $string = str_replace(chr(237), "&#966;", $string);
    $string = str_replace(chr(238), "&#949;", $string);
    $string = str_replace(chr(239), "&#8745;", $string);
    $string = str_replace(chr(240), "&#8801;", $string);
    $string = str_replace(chr(241), "&#177;", $string);
    $string = str_replace(chr(242), "&#8805;", $string);
    $string = str_replace(chr(243), "&#8804;", $string);
    $string = str_replace(chr(244), "&#8992;", $string);
    $string = str_replace(chr(245), "&#8993;", $string);
    $string = str_replace(chr(246), "&#247;", $string);
    $string = str_replace(chr(247), "&#8776;", $string);
    $string = str_replace(chr(248), "&#176;", $string);
    $string = str_replace(chr(249), "&#8729;", $string);
    $string = str_replace(chr(250), "&#183;", $string);
    $string = str_replace(chr(251), "&#8730;", $string);
    $string = str_replace(chr(252), "&#8319;", $string);
    $string = str_replace(chr(253), "&#178;", $string);
    $string = str_replace(chr(254), "&#9632;", $string);
    $string = str_replace(chr(255), "&#160;", $string);

    return $string;
}


/**
* Va-Functions.inc.php :: CalcAge()
*
* This function will calculate the age in years of the passed month, day and
* year date.
*
* @param integer $month
* @param integer $day
* @param integer $year
* @return integer
*/
function CalcAge($month,
                 $day,
                 $year) {
    if (!is_int($month)) {
        $month = (integer) $month;
    }
    if (!is_int($day)) {
        $day = (integer) $day;
    }
    if (!is_int($year)) {
        $year = (integer) $year;
    }

    $age = date("Y") - $year;

    if ((date("m") < $month) ||
        ((date("m") == $month) &&
         (date("d") < $day))) {
        $age = $age - 1;
    }

    return $age;
}


/**
* Va-Functions.inc.php :: CalcDayNumber()
*
* This function will determine the day number of the specified date.
* Algorithm from: http://alcor.concordia.ca/~gpkatch/gdate-algorithm.html
*
* @param integer $month
* @param integer $day
* @param integer $year
* @return integer
*/
function CalcDayNumber($month,
                       $day,
                       $year) {
    if (!is_int($month)) {
        $month = (integer) $month;
    }
    if (!is_int($day)) {
        $day = (integer) $day;
    }
    if (!is_int($year)) {
        $year = (integer) $year;
    }

    $month = ($month + 9) % 12;
    $year = $year - intval($month / 10);

    return (365 * $year) +
           intval($year / 4) -
           intval($year / 100) +
           intval($year / 400) +
           intval((($month * 306) + 5) / 10) +
           $day - 1;
}


/**
* Va-Functions.inc.php :: convert_bbcodes()
*
* This function will convert text to bbcode format.
* Added to VA-PHP By David Overbeek (The Borg Collective)
* coolmax@theborg.bbs.us
*
* @param string $message
* @param boolean $adult
* @return string
*/
function convert_bbcodes($message) {
    global $rootDir;

    // Video.
    $message = preg_replace(array(
                                  "/(\[dailymotion\]|\[dvid\])([\w\-\.]+)(\[\/dailymotion\]|\[\/dvid\])/",
                                  "/(\[fliqz\])([\w\-\.]+)(\[\/fliqz\])/",
                                  "/(\[googlevideo\]|\[gvid\])([\w\-\.]+)(\[\/googlevideo\]|\[\/gvid\])/",
                                  "/(\[guba\])([\w\-\.]+)(\[\/guba\])/",
                                  "/(\[hulu\])([\w\-\.]+)(\[\/hulu\])/",
                                  "/(\[liveleak\]|\[lvid\])([\w\-\.]+)(\[\/liveleak\]|\[\/lvid\])/",
                                  "/(\[megavideo\]|\[mvid\])([\w\-\.]+)(\[\/megavideo\]|\[\/mvid\])/",
                                  "/(\[mv\])([\w\-\.]+)(\[\/mv\])/",
                                  "/(\[tudou\])([\w\-\.]+)(\[\/tudou\])/",
                                  "/(\[veoh\])([\w\-\.]+)(\[\/veoh\])/",
                                  "/(\[vimeo\])([\w\-\.]+)(\[\/vimeo\])/",
                                  "/(\[wegame\]|\[wgame\])([\w\-\.]+)(\[\/wegame\]|\[\/wgame\])/",
                                  "/(\[youaretv\]|\[youtv\])([\w\-\.]+)(\[\/youaretv\]|\[\/youtv\])/",
                                  "/(\[youku\])([\w\-\.]+)(\[\/youku\])/",
                                  "/(\[youtube\]|\[tube\])([\w\-\.]+)(\[\/youtube\]|\[\/tube\])/"
                                 ),
                            array(
                                  "<center><object width=\"425\" height=\"350\"><param name=\"movie\" value=\"http://www.dailymotion.com/swf/$2&v3=1&related=1\"></param><param name=\"allowFullScreen\" value=\"true\"></param><param name=\"allowScriptAccess\" value=\"always\"></param><embed src=\"http://www.dailymotion.com/swf/$2&v3=1&related=1\" type=\"application/x-shockwave-flash\" width=\"425\" height=\"350\" allowFullScreen=\"true\" allowScriptAccess=\"always\"></embed></object></center>",
                                  "<center><object id=\"player\" width=\"425\" height=\"350\" codebase=\"http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=9,0,28,0\"><param name=\"bgcolor\" value=\"#000000\"></param><param name=\"allowFullScreen\" value=\"true\"></param><param name=\"wmode\" value=\"transparent\"></param><embed name=\"player\" src=\"http://content.fliqz.com/components/0730009dc485422bac4d51ea23e92b3e.swf\" bgcolor=\"#000000\" width=\"425\" height=\"350\" allowFullScreen=\"true\" type=\"application/x-shockwave-flash\" wmode=\"transparent\" flashvars=\"file=$2\"></embed></object></center>",
                                  "<center><embed style=\"width:425px; height:350px;\" id=\"VideoPlayback\" type=\"application/x-shockwave-flash\" src=\"http://video.google.com/googleplayer.swf?docId=$2&hl=en\"></embed></center>",
                                  "<center><embed src=\"http://www.guba.com/f/root.swf?video_url=http://free.guba.com/uploaditem/$2/flash.flv&isEmbeddedPlayer=true\" quality=\"best\" bgcolor=\"#FFFFFF\" menu=\"true\" width=\"425px\" height=\"350px\" name=\"root\" id=\"root\" align=\"middle\" scaleMode=\"noScale\" allowScriptAccess=\"always\" allowFullScreen=\"true\" type=\"application/x-shockwave-flash\"></embed></center>",
                                  "<center><object width=\"425\" height=\"350\"><param name=\"movie\" value=\"http://www.hulu.com/embed/$2\"></param><param name=\"wmode\" value=\"transparent\"></param><embed src=\"http://www.hulu.com/embed/$2\" type=\"application/x-shockwave-flash\" wmode=\"transparent\" width=\"425\" height=\"350\"></embed></object></center>",
                                  "<center><object width=\"425\" height=\"350\"><param name=\"movie\" value=\"http://www.liveleak.com/e/$2\"></param><param name=\"wmode\" value=\"transparent\"></param><embed src=\"http://www.liveleak.com/e/$2\" type=\"application/x-shockwave-flash\" wmode=\"transparent\" width=\"425\" height=\"350\"></embed></object></center>",
                                  "<center><object width=\"425\" height=\"350\"><param name=\"movie\" value=\"http://www.megavideo.com/v/$2 9686ab2f4dbe6568af91c509b1a44e30.2705081223.0\"></param><param name=\"wmode\" value=\"transparent\"></param><embed src=\"http://www.megavideo.com/v/$2 9686ab2f4dbe6568af91c509b1a44e30.2705081223.0\" type=\"application/x-shockwave-flash\" wmode=\"transparent\" width=\"425\" height=\"350\"></embed></object></center>",
                                  "<center><embed src=\"http://www.megavideo.com/v/$2\" type=\"application/x-shockwave-flash\" wmode=\"transparent\" width=\"425\" height=\"350\"></embed></center>",
                                  "<center><object width=\"425\" height=\"350\"><param name=\"movie\" value=\"http://www.tudou.com/v/$2\"></param><param name=\"allowScriptAccess\" value=\"always\"></param><param name=\"wmode\" value=\"transparent\"></param><embed src=\"http://www.tudou.com/v/$2\" type=\"application/x-shockwave-flash\" width=\"425\" height=\"350\" allowFullScreen=\"true\" wmode=\"transparent\" allowScriptAccess=\"always\"></embed></object></center>",
                                  "<center><embed src=\"http://www.veoh.com/videodetails2.swf?player=videodetailsembedded&videoAutoPlay=0&type=v&permalinkId=$2&id=9287366\" allowFullScreen=\"true\" width=\"425\" height=\"350\" type=\"application/x-shockwave-flash\" pluginspage=\"http://www.macromedia.com/go/getflashplayer\"></embed></center>",
                                  "<center><object type=\"application/x-shockwave-flash\" width=\"425\" height=\"350\" data=\"http://www.vimeo.com/moogaloop.swf?clip_id=$2&server=www.vimeo.com&fullscreen=1&show_title=0&show_byline=0&show_portrait=0&color=\"><param name=\"quality\" value=\"best\"></param><param name=\"allowfullscreen\" value=\"true\"></param><param name=\"scale\" value=\"showAll\"></param><param name=\"movie\" value=\"http://www.vimeo.com/moogaloop.swf?clip_id=&server=www.vimeo.com&fullscreen=1&show_title=1&show_byline=1&show_portrait=0&color=\"></param></object></center>",
                                  "<center><object width=\"425\" height=\"350\"><param name=\"movie\" value=\"http://www.wegame.com/static/flash/player2.swf\"></param><param name=\"wmode\" value=\"transparent\"></param><embed src=\"http://www.wegame.com/static/flash/player2.swf\" type=\"application/x-shockwave-flash\" wmode=\"transparent\" width=\"425\" height=\"350\" flashvars=\"tag=$2\"></embed></object></center>",
                                  "<center><object width=\"425\" height=\"350\"><param name=\"movie\" value=\"http://www.youare.tv/player/yatvplayer.swf?videoID=$2\"></param><param name=\"bgcolor\" value=\"#000000\"></param><param name=\"allowFullScreen\" value=\"true\"></param><embed src=\"http://www.youare.tv/player/yatvplayer.swf?videoID=$2\" type=\"application/x-shockwave-flash\" bgcolor=\"#000000\" allowFullScreen=\"true\" width=\"425\" height=\"350\"></embed></object></center>",
                                  "<center><embed src=\"http://player.youku.com/player.php/sid/$2=/v.swf\" quality=\"high\" width=\"425\" height=\"350\" align=\"middle\" allowScriptAccess=\"sameDomain\" type=\"application/x-shockwave-flash\"></embed></center>",
                                  "<center><object width=\"425\" height=\"350\"><param name=\"movie\" value=\"http://www.youtube.com/v/$2\"></param><param name=\"wmode\" value=\"transparent\"></param><embed src=\"http://www.youtube.com/v/$2\" type=\"application/x-shockwave-flash\" width=\"425\" height=\"350\"></embed></object></center>"
                                 ),
                            $message);

    // Adult video.
    if (($GLOBALS['USER']['securitylevel'] >= 255) ||
        ($GLOBALS['USER']['age'] >= 18)) {
        $message = preg_replace(array(
                                      "/(\[megarotic\]|\[megar\])([\w\-\.]+)(\[\/megarotic\]|\[\/megar\])/",
                                      "/(\[pornhub\]|\[phub\])([\w\-\.]+)(\[\/pornhub\]|\[\/phub\])/",
                                      "/(\[pornotube\]|\[ptube\])([\w\-\.]+)(\[\/pornotube\]|\[\/ptube\])/",
                                      "/(\[redtube\]|\[rtube\])([\w\-\.]+)(\[\/redtube\]|\[\/rtube\])/",
                                     ),
                                array(
                                      "<center><object width=\"425\" height=\"350\"><param name=\"movie\" value=\"http://video.megarotic.com/v/$2\"></param><param name=\"wmode\" value=\"transparent\"></param><embed src=\"http://video.megarotic.com/v/$2\" type=\"application/x-shockwave-flash\" wmode=\"transparent\" width=\"425\" height=\"350\"></embed></object></center>",
                                      "<center><embed src=\"http://www.pornhub.com/videoplayer.swf\" FlashVars=\"config=http://www.pornhub.com/flvplayer.php?viewkey=$2\" quality=\"high\" bgcolor=\"#000000\"  width=\"425\" height=\"350\" loop=\"false\" align=\"middle\" allowFullScreen=\"true\" allowScriptAccess=\"always\" type=\"application/x-shockwave-flash\" scale=\"exactfit\"></embed></center>",
                                      "<center><embed src=\"http://pornotube.com/player/v.swf?v=$2\" loop=\"false\" quality=\"high\" width=\"425\" height=\"350\" name=\"pornoPlayer\" allowfullscreen=\"true\" allowScriptAccess=\"always\" type=\"application/x-shockwave-flash\"></embed></center>",
                                      "<center><object width=\"425\" height=\"350\"><param name=\"movie\" value=\"http://embed.redtube.com/player/\"><param name=\"FlashVars\" value=\"id=10753&style=redtube\"><embed src=\"http://embed.redtube.com/player/?id=$2&style=redtube\" type=\"application/x-shockwave-flash\" width=\"425\" height=\"350\"></object></center>",
                                     ),
                                $message);
    }

    // Formatting.
    $bb_codes = array(
        '[b]' => '<span style="font-weight:bold">', '[/b]' => '</span>',          // Bold.
        '[i]' => '<span style="font-style:italic">', '[/i]' => '</span>',         // Italic.
        '[u]' => '<span style="text-decoration:underline">', '[/u]' => '</span>', // Underline.
        '[img]' => '<img src="', '[/img]' => '" />'                                 // Image.
    );

    $keys = array_keys($bb_codes);
    $values = array_values($bb_codes);

    for ($i = 0; $i < count($keys); $i++) {
        $message = str_replace($keys[$i],
                               $values[$i],
                               $message);
    }

    // Smilies (DO NOT RE-ORDER).
    $bb_codes = array(
        '&lt;):)'       => '48.gif',        // <):)     cowboy
        ':)]'           => '100.gif',       // :)]      on the phone
        '&gt;:)'        => '19.gif',        // >:)      devil
        ':|:'           => 'a12.gif',       // :|:      up yours!
        '^:)^'          => '77.gif',        // ^:)^     not worthy
        '/:)'           => '23.gif',        // /:)      raised eyebrow
        ':))'           => '21.gif',        // :))      laughing
        ':)'            => '1.gif',         // :)       happy
        ':(('           => '20.gif',        // :((      crying
        ':('            => '2.gif',         // :(       sad
        ':&quot;&gt;'   => '9.gif',         // :">      blushing
        ';;)'           => '5.gif',         // ;;)      batting eyelashes
        ';)'            => '3.gif',         // ;)       winking
        ');D('          => '6.gif',         // );D(     big hug  - Changed from >:D<
        ':D'            => '4.gif',         // :D       big grin
        ':-/'           => '7.gif',         // :-/      confused
        ':x'            => '8.gif',         // :x       love struck
        '&gt;:P'        => '47.gif',        // >:P      phbbbbt
        ':P'            => '10.gif',        // :P       tongue
        ':-*'           => '11.gif',        // :-*      kiss
        '=(('           => '12.gif',        // =((      broken heart
        ':-O'           => '13.gif',        // :-O      surprise
        '~X('           => '102.gif',       // ~X(      at wits' end (Pulling My Hair Out)
        'X('            => '14.gif',        // X(       angry
        ':&gt;'         => '15.gif',        // :>       smug
        'B-)'           => '16.gif',        // B-)      cool
        '#:-S'          => '18.gif',        // #:-S     whew!
        ':-SS'          => '42.gif',        // :-SS     nailbiting
        ':-S'           => '17.gif',        // :-S      worried
        '(:|'           => '37.gif',        // (:|      yawn
        ':|'            => '22.gif',        // :|       straight face
        'O:-)'          => '25.gif',        // O:-)     angel
        ':-B'           => '26.gif',        // :-B      nerd
        '=;'            => '27.gif',        // =;       talk to the hand
        ':-c'           => '101.gif',       // :-c      call me
        ':-h'           => '103.gif',       // :-h      wave
        ':-t'           => '104.gif',       // :-t      time out
        '8-&gt;'        => '105.gif',       // 8->      daydreaming
        'I-)'           => '28.gif',        // I-)      sleepy
        '8-|'           => '29.gif',        // 8-|      rolling eyes
        'L-)'           => '30.gif',        // L-)      loser
        ':-&lt;'        => '46.gif',        // :-<      sigh
        ':-&'           => '31.gif',        // :-&      sick
        ':-$'           => '32.gif',        // :-$      don't tell anyone
        '[-('           => '33.gif',        // [-(      not talking
        ':O)'           => '34.gif',        // :O)      clown
        '8-}'           => '35.gif',        // 8-}      silly
        '&lt;:-P'       => '36.gif',        // <:-P     party
        '=P~'           => '38.gif',        // =P~      drooling
        ':-?'           => '39.gif',        // :-?      thinking
        '#-o'           => '40.gif',        // #-o      d'oh
        '=D&gt;'        => '41.gif',        // =D>      applause
        '@-)'           => '43.gif',        // @-)      hypnotized
        ':^o'           => '44.gif',        // :^o      liar
        ':-w'           => '45.gif',        // :-w      waiting
        '=))'           => '24.gif'         // =))      rolling on the floor
    );

    $keys = array_keys($bb_codes);
    $values = array_values($bb_codes);

    for ($i = 0; $i < count($keys); $i++) {
        $message = str_replace($keys[$i],
                               '<img src="' . $rootDir . 'images/emoticons/' . $values[$i] . '" align="top" alt="' . $keys[$i] . '"/>',
                               $message);
    }

    return $message;
}


/**
* Va-Functions.inc.php :: HtmlEncode()
*
* This function will take the passed line and add html href information for any
* links found. In basic mode, it will search for mailto, ftp, http and telnet.
* In enhanced mode, it will search for unregistered urls.
*
* @param string $string
* @param boolean $enhanced
* @param integer $length
* @return string
*/
function HTMLEncode($string,
                    $enhanced = 0,
                    $length = 45) {
    // http://www.w3.org/Addressing/schemes
    if (!is_string($string)) {
        $string = (string) $string;
    }
    if (!is_bool($enhanced)) {
        $enhanced = (boolean) $enhanced;
    }
    if (!is_int($length)) {
        $length = (integer) $length;
    }


    // Add a space so a match will be found from char 1.
    $string = ' ' . $string;

    $string = preg_replace_callback("/([\s|\r\n|\r|\n|\(|\[\{\-:;\'\"\<.,?])([f|F][t|T][p|P](s?):\/\/)([\w\.\/\&\=\-\+\(\),~:;@#%]+)(\??)([\w\.\/\&\=\-\+\(\),~:;#%]+)/",
                                    function ($matches) use ($length) {
                                        //$url = 'ftp' . $matches[3] . '://' . $matches[4] . $matches[5];
                                        //$link = $matches[2] . $matches[4] . $matches[5];
                                        $url = $matches[2] . $matches[4] . $matches[5] . $matches[6];
                                        $link = $url;
                                        if (strlen($link) > $length) {
                                            $link = substr($url, 0, $length) . '...';
                                        }
                                        return $matches[1] . '<a href="' . $url . '">' . $link . '</a>';
                                    },
                                    $string);
    $string = preg_replace_callback("/([\s|\r\n|\r|\n|\(|\[\{\-:;\'\"\<.,?])([f|F][t|T][p|P]\.)([\w\.\/\&\=\?\-~:;%]+)/",
                                    function ($matches) use ($length) {
                                        $url = 'ftp://' . $matches[2] . $matches[3];
                                        $link = $matches[2] . $matches[3];
                                        if (strlen($link) > $length) {
                                            $link = substr($url, 0, $length) . '...';
                                        }
                                        return $matches[1] . '<a href="' . $url . '">' . $link . '</a>';
                                    },
                                    $string);
    $string = preg_replace_callback("/([\s|\r\n|\r|\n|\(|\[\{\-:;\'\"\<.,?])([h|H][t|T][t|T][p|P](s?):\/\/)([\w\.\/\&\=\-\+\(\),~:;@#%]+)(\??)([\w\.\/\&\=\-\+\(\),~:;#%]+)/",
                                    function ($matches) use ($length) {
                                        $url = $matches[2] . $matches[4] . $matches[5] . $matches[6];
                                        $link = $url;
                                        //$link = 'http' . $matches[3] . '://' . $matches[4] . $matches[5] . $matches[6];
                                        if (strlen($link) > $length) {
                                            $link = substr($url, 0, $length) . '...';
                                        }
                                        return $matches[1] . '<a href="' . $url . '">' . $link . '</a>';
                                    },
                                    $string);
    $string = preg_replace_callback("/([\s|\r\n|\r|\n|\(|\[\{\-:;\'\"\<.,?])([w|W][w|W][w|W]\.)([\w\.\/\&\=\-\+\(\),~:;@#%]+)(\??)([\w\.\/\&\=\-\+\(\),~:;#%]+)/",
                                    function ($matches) use ($length) {
                                        $url = 'http://' . $matches[2] . $matches[3] . $matches[4] . $matches[5];
                                        $link = $matches[2] . $matches[3] . $matches[4] . $matches[5];
                                        if (strlen($link) > $length) {
                                            $link = substr($url, 0, $length) . '...';
                                        }
                                        return $matches[1] . '<a href="' . $url . '">' . $link . '</a>';
                                    },
                                    $string);
    $string = preg_replace_callback("/([\s|\r\n|\r|\n|\(|\[\{\-:;\'\"\<.,?])([t|T][e|E][l|L][n|N][e|E][t|T]:\/\/([\w~:@]+))([\w\.\/\&\=\?\-:]+)/",
                                    function ($matches) use ($length) {
                                        $url = 'telnet://' . $matches[3] . $matches[4];
                                        $link = $matches[2] . $matches[4];
                                        if (strlen($link) > $length) {
                                            $link = substr($url, 0, $length) . '...';
                                        }
                                        return $matches[1] . '<a href="' . $url . '">' . $link . '</a>';
                                    },
                                    $string);
    $string = preg_replace_callback("/([\s|\r\n|\r|\n|\(|\[\{\-:;\'\"\<.,?])(([\w\.\-]+))(@)([\w\.\-]+)\b\.([\w\.\-]+)\b/i",
                                    function ($matches) use ($length) {
                                        $url = 'mailto:' . $matches[3] . $matches[4] . $matches[5] . '.' . $matches[6];
                                        $link = $matches[3] . $matches[4] . $matches[5] . '.' . $matches[6];
                                        if (strlen($link) > $length) {
                                            $link = substr($url, 0, $length) . '...';
                                        }
                                        return $matches[1] . '<a href="' . $url . '">' . $link . '</a>';
                                    },
                                    $string);

    if ($enhanced) {
        $string = preg_replace_callback("/([\s|\r\n|\r|\n|\(|\[\{\-:;\'\"\<.,?])([g|G][o|O][p|P][h|H][e|E][r|R]:\/\/)([\w\.\/\&\=\?\-:|%]+)/",
                                        function ($matches) use ($length) {
                                            $url = 'gopher://' . $matches[3];
                                            $link = $matches[2] . $matches[3];
                                            if (strlen($link) > $length) {
                                                $link = substr($url, 0, $length) . '...';
                                            }
                                            return $matches[1] . '<a href="' . $url . '">' . $link . '</a>';
                                        },
                                        $string);
        $string = preg_replace_callback("/([\s|\r\n|\r|\n|\(|\[\{\-:;\'\"\<.,?])([n|N][e|E][w|W][s|S]:)([\w\.\/\&\=\?\-:*]+)/",
                                        function ($matches) use ($length) {
                                            $url = 'news:' . $matches[3];
                                            $link = $matches[2] . $matches[3];
                                            if (strlen($link) > $length) {
                                                $link = substr($url, 0, $length) . '...';
                                            }
                                            return $matches[1] . '<a href="' . $url . '">' . $link . '</a>';
                                        },
                                        $string);
        $string = preg_replace_callback("/([\s|\r\n|\r|\n|\(|\[\{\-:;\'\"\<.,?])([n|N][n|N][t|T][p|P]:\/\/)([\w.\/\&\=\?\-~:#]+)/",
                                        function ($matches) use ($length) {
                                            $url = 'nntp://' . $matches[3];
                                            $link = $matches[2] . $matches[3];
                                            if (strlen($link) > $length) {
                                                $link = substr($url, 0, $length) . '...';
                                            }
                                            return $matches[1] . '<a href="' . $url . '">' . $link . '</a>';
                                        },
                                        $string);
        $string = preg_replace_callback("/([\s|\r\n|\r|\n|\(|\[\{\-:;\'\"\<.,?])([a|A][i|I][m|M]:)([\w\.\/\&\=\?\-:#]+)/",
                                        function ($matches) use ($length) {
                                            $url = 'aim:' . $matches[3];
                                            $link = $matches[2] . $matches[3];
                                            if (strlen($link) > $length) {
                                                $link = substr($url, 0, $length) . '...';
                                            }
                                            return $matches[1] . '<a href="' . $url . '">' . $link . '</a>';
                                        },
                                        $string);
    }

    // Return the encoded string, remove that inital space.
    return substr($string, 1);
}



/**
* Va-Functions.inc.php :: HtmlSpace()
*
* Add non-breaking spaces to a string for output.
*
* @param string $string
* @return string
*/
function HtmlSpace($string) {
    if (!is_string($string)) {
        $string = (string) $string;
    }

    $string = str_replace('  ', '&nbsp;&nbsp;', $string);
    return $string;
}


/**
* Va-Functions.inc.php :: IsHttps()
*
* Determines if the current page's protocol.
*
* @return boolean
*/
function IsHttps() {
    if (isset($_SERVER['HTTPS'])) {
        if ((strtolower($_SERVER['HTTPS']) == 'on') ||
            ($_SERVER['HTTPS'] == '1')) {
            return TRUE;
        }
    } elseif ((isset($_SERVER['SERVER_PORT'])) &&
              ($_SERVER['SERVER_PORT'] == '443')) {
        return TRUE;
    }

    return FALSE;
}


/**
* Va-Functions.inc.php :: IsLoggedIn()
*
* This function checks to see if the current user is logged in.
*
* @return boolean
*/
function IsLoggedIn() {
    if (($GLOBALS['USER']['userno'] > 0) &&
        ($GLOBALS['USER']['userno'] != $GLOBALS['CONFIG']['guestacct']) &&
        ($GLOBALS['USER']['securitylevel'] > 0)) {
        return TRUE;
    } else {
        return FALSE;
    }
}


/**
* Va-Functions.inc.php :: lTime()
*
* Returns local unixtimestamp.
*
* @return integer
*/
function lTime() {
    return gmmktime(date("H"), date("i"), date("s"), date("n"), date("j"), date("Y"));
}


/**
* Va-Functions.inc.php :: MakeAccFlags()
*
* This function will take the string representation of user access flags and
* convert it into an integer suitable for writing to userfile.dat
*
* @param string $accFlags
* @return integer
*/
function MakeAccFlags($accFlags) {
    if (!is_string($accFlags)) {
        $accFlags = (string) $accFlags;
    }

    // Remove any white space and convert it to upper case.
    $accFlags = strtoupper(str_replace(' ', '', $accFlags));
    $accFlagsInt = 0;

    for ($i = strlen($accFlags); $i >= 0; $i--) {
        // Get a character.
        $letter = substr($accFlags, $i, 1);

        // It must be a letter, or ignore it.
        if (((ord($letter) >= 65) &&
             (ord($letter) <= 90)) &&
            (strpos($accFlags, $letter) === $i)) {  // If the letter does not occur before this one...
            $accFlagsInt = $accFlagsInt + pow(2, ord($letter) - 65);
        }
    }

    // Return the integer.
    return $accFlagsInt;
}


/**
* Va-Functions.inc.php :: MakeChar()
*
* This function will take the value given and convert it into a one byte
* integer represented by ascii characters.
*
* @param integer $value
* @return string
*/
function MakeChar($value) {
    if (!is_int($value)) {
        $value = (integer) $value;
    }

    if ($value == 0) {
        return chr(0);
    }
    if ($value > 256) {
        trigger_error(VA_Error(2));
    }

    // Set the variable to the ascii character.
    $a = chr($value);

    // Return the character.
    return $a;
}


/**
* Va-Functions.inc.php :: MakeFlags()
*
* This function will take the string $accFlags and convert it to the 26 character
* representation which is writable to userfile.dat.
*
* @param string $accFlags
* @return string
*/
function MakeFlags($accFlags) {
    if (!is_string($accFlags)) {
        $accFlags = (string) $accFlags;
    }

    // Remove any white space and convert it to upper case.
    $accFlags = strtoupper(str_replace(' ', '', $accFlags));

    // Create the blank string.
    $newFlags = str_pad('', 26, ' ');

    // Check to see if the variable was passed, if not then return nothing.
    if ($accFlags == '') {
        return $newFlags;
    }

    for ($i = strlen($accFlags); $i >= 0; $i--) {
        // Get a character.
        $letter = substr($accFlags, $i, 1);
        if ((ord($letter) >= 65) &&
            (ord($letter) <= 90)) {
            $newFlags = substr_replace($newFlags, $letter, ord($letter) - 65, 1);
        }
    }

    // Return the new string.
    return $newFlags;
}


/**
* Va-Functions.inc.php :: MakeInteger()
*
* This function will take the value given and convert it into a two byte
* integer represented by ascii characters. The values can be from -32768 to
* 32767. If you set $unsigned to 1, then you can create an integer with a value
* up to 65535.
*
* @param integer $value
* @param boolean $unsigned
* @return string
*/
function MakeInteger($value,
                     $unsigned = 0) {
    if (!is_int($value)) {
        $value = (integer) $value;
    }
    if (!is_bool($unsigned)) {
        $unsigned = (boolean) $unsigned;
    }

    if ($value == 0) {
        return chr(0) . chr(0);
    }
    if (!$unsigned) {
        // This is the variable type maximum.
        if ($value > 32767) {
            trigger_error(VA_Error(2));
        }
        if ($value < -32768) {
            trigger_error(VA_Error(2));
        }
        if ($value < 0) {
            $value = 65535 + $value + 1;
        }
    } else {
        // This is the maximum that can be stored.
        if ($value > 65535) {
            trigger_error(VA_Error(2));
        }
        if ($value < 0) {
            trigger_error(VA_Error(2));
        }
    }

    // Divide the value.
    $b = intval($value / 256);

    // Subtract the variable from the value.
    $value = $value - ($b * 256);

    // Set the variable to the ascii character.
    $b = chr($b);

    // Set the variable to the ascii character.
    $a = chr($value);

    // Return the ascii representation.
    return $a . $b;
}


/**
* Va-Functions.inc.php :: MakeLongInt()
*
* This function will take the value given and convert it into a four byte
* long integer represented by ASCII characters. The values can be from
* -2147483648 to 2147483647. If you set $unsigned to 1, then you can create a
* long integer with a value up to 4294967295.
*
* @param integer $value
* @param boolean $unsigned
* @return string
*/
function MakeLongInt($value,
                     $unsigned = 0) {
    if (!is_int($value)) {
        $value = (integer) $value;
    }
    if (!is_bool($unsigned)) {
        $unsigned = (boolean) $unsigned;
    }

    if ($value == 0) {
        return chr(0) . chr(0) . chr(0) . chr(0);
    }
    if (!$unsigned) {
        // This is the variable type maximum.
        if ($value > 2147483647) {
            trigger_error(VA_Error(2));
        }
        if ($value < -2147483648) {
            trigger_error(VA_Error(2));
        }
        if ($value < 0) {
            $value = 2147483647 + $value + 1;
        }
    } else {
        // This is the maximum that can be stored.
        if ($value > 4294967295) {
            trigger_error(VA_Error(2));
        }
        if ($value < 0) {
            trigger_error(VA_Error(2));
        }
    }

    // Divide the value.
    // Subtract the variable from the value.
    // Set the variable to the ascii character.
    $d = intval($value / 16777216);
    $value = $value - ($d * 16777216);
    $d = chr($d);

    // Divide the value.
    // Subtract the variable from the value.
    // Set the variable to the ascii character.
    $c = intval($value / 65536);
    $value = $value - ($c * 65536);
    $c = chr($c);

    // Divide the value.
    // Subtract the variable from the value.
    // Set the variable to the ascii character.
    $b = intval($value / 256);
    $value = $value - ($b * 256);
    $b = chr($b);

    // Set the variable to the ascii character.
    $a = chr($value);

    // Return the ascii representation.
    return $a . $b . $c . $d;
}


/**
* Va-Functions.inc.php :: MakeSingle()
*
* This function will take the floating point $value and convert it into a four
* byte ASCII representation. The maximum is 3.40282e+38.
*
* @param float $value
* @return string
*/
function MakeSingle($value) {
    if (!is_float($value)) {
        $value = (float) $value;
    }

    // Determine the sign.
    if ($value < 0) {
        $sign = 1;
    } else {
        $sign = 0;
    }

    // Split the number up into parts.
    $array = explode('.', $value);
    $integer = decbin($array[0]);
    $decimal = '';

    // Create the binary representation of the decimal.
    if (isset($array[1])) {
        $dec = '.' . $array[1];
        $fraction = 0;
        for ($i = 0; (($i < 10) && ($fraction != 1)) ; $i++) {
            $fraction = $dec * 2;
            if ($fraction >= 1) {
                $decimal = $decimal . '1';
                $dec = $fraction - 1;
            } else {
                $decimal = $decimal . '0';
                $dec = $fraction;
            }
        }
    }

    if ($integer > 0) {
        $exp = strlen($integer) - 1;
        $integer = substr($integer, 1);
        $float = $integer . $decimal;
    } else {
        if ($decimal != 0) {
            $exp = strpos($decimal, '1') + 1;
            $float = substr($decimal, $exp);
            $exp = '-' . $exp;
        } else {
            $exp = -127;
            $float = 0;
        }
    }

    $exp = decbin($exp + 127);
    $exp = str_pad($exp, 8, '0', STR_PAD_LEFT);

    $float = str_pad($float, 23, '0', STR_PAD_RIGHT);
    $float = $sign . $exp . $float;

    // Create the four bytes.
    $d = bindec(substr($float, 0, 8));
    $d = chr($d);
    $c = bindec(substr($float, 8, 8));
    $c = chr($c);
    $b = bindec(substr($float, 16, 8));
    $b = chr($b);
    $a = bindec(substr($float, 24, 8));
    $a = chr($a);

    // Return the ascii representation.
    return $a . $b . $c . $d;
}


/**
* Va-Functions.inc.php :: mstr_replace()
*
* This is a multi-pass function of str_replace.
*
* @param string $search
* @param string $replace
* @param string $subject
* @return string
*/
function mstr_replace($search,
                      $replace,
                      $subject) {
    if (!is_string($search)) {
        $search = (string) $search;
    }
    if (!is_string($replace)) {
        $replace = (string) $replace;
    }
    if (!is_string($subject)) {
        $subject = (string) $subject;
    }

    while (strpos($subject, $search) !== FALSE) {
        $subject = str_replace($search, $replace, $subject);
    }

    return $subject;
}


/**
* Va-Functions.inc.php :: ParseANSI()
*
* This function will parse an ANSI coded file and return it as a HTML string.
* It uses the Windows Termainal font, so it may not display correctly on non-
* Windows machines.
*
* You can specify the font-size the final result will be encoded to use. You can
* set the maximum number of rows to display (up to 1000). If 'truncate' is
* turned on, then any blank rows at the end of the file will be removed.
*
* @param string $file
* @return string
*/
function ParseANSI($fileString,
                   $fontSize = 15,
                   $maxRows = 25,
                   $truncate = 1) {
    if (!is_string($fileString)) {
        $fileString = (string) $fileString;
    }
    if (!is_int($fontSize)) {
        $fontSize = (integer) $fontSize;
    }
    if (!is_int($maxRows)) {
        $maxRows = (integer) $maxRows;
    }
    if (!is_bool($truncate)) {
        $truncate = (boolean) $truncate;
    }

    // Check the maxrows.
    if ($maxRows < 1) {
        $maxRows = 1;
    }
    if ($maxRows > 1000) {
        $maxRows = 1000;
    }
    $maxSize = $maxRows * 80;

    // Remove any codes that will not be processed.
    $pattern = array(
                     "/\e\[((\?*?(\d){1,2})|((\d){1,2};{0,1})){0,2}[R|h|l|y]/",
                     "/\e\[(((\d){1,2};{0,1}){0,4})[p]/",
                     "/\e\[6[n]/",
                     "/\e\[((\d){1,2};{0,1}){0,3}[@|L|M|P]/",
                    );
    $fileString = preg_replace($pattern, '', $fileString);

    // Two arrays are created. One being the style array, which contains the
    // information about attributes to the block. The other array contains the
    // content to display in the block.
    // 80 x 25 = 2000 blocks

    // Style array contains attribute information. It is in this format:
    // Bold ; Dim ; Underline ; Blink ; Reverse ; Invisible ; Foreground ; Background
    // By default, all blocks are set to 0;0;0;0;0;0;37;40
    $style = array();
    for ($i = 0; $i < $maxSize; $i++) {
        $style[$i] = '0;0;0;0;0;0;37;40';
    }

    // Content array contains ASCII characters. By default, all blocks contain a space.
    $content = array();
    for ($i = 0; $i < $maxSize; $i++) {
        $content[$i] = ' ';
    }

    $escapeCode = chr(27) . '[';
    $currentStyle = '0;0;0;0;0;0;37;40';

    // X position.
    $pos = 0;

    // Position in the array.
    $arrayPos = 0;

    // Saved position.
    $savedPos = '0;0';

    for ($i = 0; strlen($fileString) > 0; $i++) {
        $match = strpos($fileString, $escapeCode);
        if ($arrayPos > $maxSize - 1) {
            $arrayPos = $maxSize - 1;
        }
        if ($match === FALSE) {
            // If no match, then place the rest of the fileString into the array
            // one character at a time, up to the $maxSize.
            for ($arrayPos; (($arrayPos < $maxSize) && (strlen($fileString) > 0)); $arrayPos++) {
                $char = substr($fileString, 0, 1);
                if ($char != "\r") {
                    if ($char == "\n") {
                        $arrayPos = $arrayPos - $pos + 78;
                        $pos = 0;
                    } else {
                        $style[$arrayPos] = $currentStyle;
                        $content[$arrayPos] = HTMLcp437(htmlspecialchars($char, ENT_COMPAT | ENT_HTML401, 'ISO-8859-1'));
                        if ($pos < 80) {
                            $pos++;
                        } else {
                            $pos = 0;
                        }
                    }
                }
                $fileString = substr($fileString, 1);
            }
            $fileString = '';
        } else if ($match > 0) {
            // If the code is not at the beginning, then enter in the chars
            // up to the first code.
            for ($j = 0; $j < $match; $j++) {
                $char = substr($fileString, 0, 1);
                if ($char != "\r") {
                    if ($char == "\n") {
                        $arrayPos = $arrayPos - $pos + 78;
                        $pos = 0;
                    } else {
                        $style[$arrayPos] = $currentStyle;
                        $content[$arrayPos] = HTMLcp437(htmlspecialchars($char, ENT_COMPAT | ENT_HTML401, 'ISO-8859-1'));
                        if ($pos < 80) {
                            $pos++;
                        } else {
                            $pos = 0;
                        }
                    }
                }
                $arrayPos++;
                $fileString = substr($fileString, 1);
            }
        } else {
            $foundMatch = 0;
            if ((preg_match("/\e\[(\d){0,3}[A]/", $fileString, $code)) &&
                ($foundMatch == 0)) {
                if (strpos($fileString, $code[0]) === 0) {
                    $foundMatch = 1;
                    $max = intval($arrayPos / 80);
                    $length = strpos($fileString, 'A');
                    $length = (integer) substr($fileString, 2, $length - 2);
                    if ($length < 1) {
                        $length = 1;
                    }
                    if ($max < $length) {
                        $arrayPos = $arrayPos - ($max * 80);
                    } else {
                        $arrayPos = $arrayPos - ($length * 80);
                    }
                    $fileString = preg_replace("/\e\[(\d){0,3}[A]/", '', $fileString, 1);
                }
            }
            if ((preg_match("/\e\[(\d){0,3}[B]/", $fileString, $code)) &&
                ($foundMatch == 0)) {
                if (strpos($fileString, $code[0]) === 0) {
                    $foundMatch = 1;
                    $max = intval(($maxSize - $arrayPos) / 80) - 1;
                    $length = strpos($fileString, 'B');
                    $length = (integer) substr($fileString, 2, $length - 2);
                    if ($length < 1) {
                        $length = 1;
                    }
                    if ($max < $length) {
                        $arrayPos = $arrayPos + ($max * 80);
                    } else {
                        $arrayPos = $arrayPos + ($length * 80);
                    }
                    $fileString = preg_replace("/\e\[(\d){0,3}[B]/", '', $fileString, 1);
                }
            }
            if ((preg_match("/\e\[(\d){0,3}[C]/", $fileString, $code)) &&
                ($foundMatch == 0)) {
                if (strpos($fileString, $code[0]) === 0) {
                    $foundMatch = 1;
                    $max = 79 - $pos;
                    $length = strpos($fileString, 'C');
                    $length = (integer) substr($fileString, 2, $length - 2);
                    if ($length < 1) {
                        $length = 1;
                    }
                    if ($max < $length) {
                        $arrayPos = $arrayPos + $max;
                        $pos = $pos + $max;
                    } else {
                        $arrayPos = $arrayPos + $length;
                        $pos = $pos + $length;
                    }
                    $fileString = preg_replace("/\e\[(\d){0,3}[C]/", '', $fileString, 1);
                }
            }
            if ((preg_match("/\e\[(\d){0,3}[D]/", $fileString, $code)) &&
                ($foundMatch == 0)) {
                if (strpos($fileString, $code[0]) === 0) {
                    $foundMatch = 1;
                    $max = $pos;
                    $length = strpos($fileString, 'D');
                    $length = (integer) substr($fileString, 2, $length - 2);
                    if ($length < 1) {
                        $length = 1;
                    }
                    if (0 > $pos - $length) {
                        $arrayPos = $arrayPos - $max;
                        $pos = $pos - $max;
                    } else {
                        $arrayPos = $arrayPos - $length;
                        $pos = $pos - $length;
                    }
                    $fileString = preg_replace("/\e\[(\d){0,3}[D]/", '', $fileString, 1);
                }
            }
            if ((preg_match("/\e\[[s]/", $fileString, $code)) &&
                ($foundMatch == 0)) {
                if (strpos($fileString, $code[0]) === 0) {
                    $savedPos = $arrayPos . ';' . $pos;
                    $fileString = preg_replace("/\e\[[s]/", '', $fileString, 1);
                }
            }
            if ((preg_match("/\e\[[u]/", $fileString, $code)) &&
                ($foundMatch == 0)) {
                if (strpos($fileString, $code[0]) === 0) {
                    $restorePos = explode(';', $savedPos);
                    $arrayPos = $restorePos[0];
                    $pos = $restorePos[1];
                    $fileString = preg_replace("/\e\[[u]/", '', $fileString, 1);
                }
            }
            if ((preg_match("/\e\[((\?*?(\d){1,2})|((\d){1,2};{0,1})){0,2}[H|f]/", $fileString, $code)) &&
                ($foundMatch == 0)) {
                if (strpos($fileString, $code[0]) === 0) {
                    if (isset($code[5])) {
                        $j = (integer) $code[4];
                        $i = (integer) $code[2];
                    } else if (isset($code[2])) {
                        $j = (integer) $code[2];
                        $i = 0;
                    } else {
                        $j = 0;
                        $i = 0;
                    }

                    $j--;
                    $i--;

                    if ($j < 0) {
                        $j = 0;
                    }
                    if ($i < 0) {
                        $i = 0;
                    }

                    $arrayPos = (80 * $j) + $i;
                    $pos = $i;
                    $fileString = preg_replace("/\e\[((\?*?(\d){1,2})|((\d){1,2};{0,1})){0,2}[H|f]/", '', $fileString, 1);
                }
            }
            if ((preg_match("/\e\[2J/", $fileString, $code)) &&
                ($foundMatch == 0)) {
                if (strpos($fileString, $code[0]) === 0) {
                    // Reset the entire page.
                    for ($i = 0; $i < $maxSize; $i++) {
                        $style[$i] = '0;0;0;0;0;0;37;40';
                    }

                    for ($i = 0; $i < $maxSize; $i++) {
                        $content[$i] = ' ';
                    }

                    $currentStyle = '0;0;0;0;0;0;37;40';
                    $pos = 0;
                    $arrayPos = 0;
                    $fileString = preg_replace("/\e\[2J/", '', $fileString, 1);
                }
            }
            if ((preg_match("/\e\[[K]/", $fileString, $code)) &&
                ($foundMatch == 0)) {
                if (strpos($fileString, $code[0]) === 0) {
                    $length = (79 - $pos) + 1;
                    for ($j = 0; $j < $length; $j++) {
                        $style[$arrayPos + $j] = '0;0;0;0;0;0;37;40';
                        $content[$arrayPos + $j] = ' ';
                    }
                    $fileString = preg_replace("/\e\[[K]/", '', $fileString, 1);
                }
            }
            if ((preg_match("/\e\[(((\d){1,2};{0,1}){0,4})[m]/", $fileString, $code)) &&
                ($foundMatch == 0)) {
                if (strpos($fileString, $code[0]) === 0) {
                    $foundMatch = 1;
                    $settings = explode(';', $code[1]);
                    $current = explode(';', $currentStyle);

                    // Process codes.
                    for ($j = 0; $j < count($settings); $j++) {
                        switch ($settings[$j]) {
                            case 0:
                                $current[0] = 0;
                                $current[1] = 0;
                                $current[2] = 0;
                                $current[3] = 0;
                                $current[4] = 0;
                                $current[5] = 0;
                                $current[6] = 37;
                                $current[7] = 40;
                                break;
                            case 1:
                                $current[0] = 1;
                                break;
                            case 2:
                                $current[1] = 1;
                                break;
                            case 4:
                                $current[2] = 1;
                                break;
                            case 5:
                                $current[3] = 1;
                                break;
                            case 7:
                                $current[4] = 1;
                                break;
                            case 8:
                                $current[5] = 1;
                                break;
                            case 30:
                                $current[6] = 30;
                                break;
                            case 31:
                                $current[6] = 31;
                                break;
                            case 32:
                                $current[6] = 32;
                                break;
                            case 33:
                                $current[6] = 33;
                                break;
                            case 34:
                                $current[6] = 34;
                                break;
                            case 35:
                                $current[6] = 35;
                                break;
                            case 36:
                                $current[6] = 36;
                                break;
                            case 37:
                                $current[6] = 37;
                                break;
                            case 40:
                                $current[7] = 40;
                                break;
                            case 41:
                                $current[7] = 41;
                                break;
                            case 42:
                                $current[7] = 42;
                                break;
                            case 43:
                                $current[7] = 43;
                                break;
                            case 44:
                                $current[7] = 44;
                                break;
                            case 45:
                                $current[7] = 45;
                                break;
                            case 46:
                                $current[7] = 46;
                                break;
                            case 47:
                                $current[7] = 47;
                                break;
                        }
                    }
                    $currentStyle = implode(';', $current);
                    $style[$arrayPos] = $currentStyle;
                    $fileString = preg_replace("/\e\[(((\d){1,2};{0,1}){0,6})[m]/", '', $fileString, 1);
                }
            }
        }
    }

    // Combine the array together and return the results.
    $pos = 0;
    $string = '<table cellspacing="0" cellpadding="0" border="0" style="border-collapse: collapse"><tr><td><pre style="font-family: monospace; font-size: ' . $fontSize . 'px; color: #ADAAAD; background-color: #000000;">';
    $lastspan = '';
    $spantemp = 0;
    for ($i = 0; $i < $maxSize; $i++) {
        // Determine the span tag.
        $span = '<span style="';
        $attributes = explode(';', $style[$i]);

        switch ($attributes[6]) {
            case 30:
                if ($attributes[0] == 0) {
                    $span = $span . 'color: #000000;';
                } else {
                    $span = $span . 'color: #555555;';
                }
                break;
            case 31:
                if ($attributes[0] == 0) {
                    $span = $span . 'color: #AA0000;';
                } else {
                    $span = $span . 'color: #FF5555;';
                }
                break;
            case 32:
                if ($attributes[0] == 0) {
                    $span = $span . 'color: #00AA00;';
                } else {
                    $span = $span . 'color: #55FF55;';
                }
                break;
            case 33:
                if ($attributes[0] == 0) {
                    $span = $span . 'color: #AAAA00;';
                } else {
                    $span = $span . 'color: #FFFF55;';
                }
                break;
            case 34:
                if ($attributes[0] == 0) {
                    $span = $span . 'color: #0000AA;';
                } else {
                    $span = $span . 'color: #5555FF;';
                }
                break;
            case 35:
                if ($attributes[0] == 0) {
                    $span = $span . 'color: #AA00AA;';
                } else {
                    $span = $span . 'color: #FF55FF;';
                }
                break;
            case 36:
                if ($attributes[0] == 0) {
                    $span = $span . 'color: #00AAAA;';
                } else {
                    $span = $span . 'color: #55FFFF;';
                }
                break;
            case 37:
                if ($attributes[0] == 0) {
                    $span = $span . 'color: #AAAAAA;';
                } else {
                    $span = $span . 'color: #FFFFFF;';
                }
                break;
        }
        switch ($attributes[7]) {
            case 40:
                // Removed due to being default value.
                //$span = $span . ' background-color: #000000;';
                break;
            case 41:
                $span = $span . ' background-color: #AA0000;';
                break;
            case 42:
                $span = $span . ' background-color: #00AA00;';
                break;
            case 43:
                $span = $span . ' background-color: #AAAA00;';
                break;
            case 44:
                $span = $span . ' background-color: #0000AA;';
                break;
            case 45:
                $span = $span . ' background-color: #AA00AA;';
                break;
            case 46:
                $span = $span . ' background-color: #00AAAA;';
                break;
            case 47:
                $span = $span . ' background-color: #AAAAAA;';
                break;
        }

        $span = $span . '">';

        // This will check to see if the span tag was the same as the last. If
        // so, it will not use it since it just slows down the display and is
        // not needed.
        if ($span != $lastspan) {
            $lastspan = $span;
            if ($spantemp == 1) {
                $span = '</span>' . $span;
            }
            //$string = $string . $span . HtmlSpace($content[$i]);
            $string = $string . $span . $content[$i];
            $pos++;
        } else {
            //$string = $string . HtmlSpace($content[$i]);
            $string = $string . $content[$i];
            $spantemp = 1;
            $pos++;
        }

        if ($pos > 79) {
            $string = $string . "\n";
            $pos = 0;
        }
    }

    // Truncate blank rows.
    if ($truncate) {
        $string = rtrim($string);
    }

    if ($spantemp == 1) {
        $string = $string . '</span>';
    }

    $string = $string . '</pre></td></tr></table>' . "\n\n";

    // Return the HTML output.
    return $string;
}


/**
* Va-Functions.inc.php :: ProgramTime()
*
* This function simply returns the current timestamp in microseconds.
*
* @return integer
*/
function ProgramTime() {
    $time = microtime();
    $time = explode(' ', $time);
    return $time[0] + $time[1];
}


/**
* Va-Functions.inc.php :: PutChar()
*
* This function will put the passed integer into the file string at the given
* position. This is used for writing characters, or one byte integers.
*
* @param string $fileString
* @param integer $int
* @param integer $position
* @return string
*/
function PutChar($fileString,
                 $int,
                 $position) {
    if (!is_string($fileString)) {
        $fileString = (string)  $fileString;
    }
    if (!is_int($int)) {
        $int = (integer) $int;
    }
    if (!is_int($position)) {
        $position = (integer) $position;
    }

    if (strlen($fileString) < $position) {
       $fileString = $fileString . str_replace('*', chr(0), str_pad('', $position, '*'));
    }

    $position--;

    $int = MakeChar($int);
    $fileString = substr_replace($fileString, $int, $position, 1);

    // Return the new string.
    return $fileString;
}


/**
* Va-Functions.inc.php :: PutInteger()
*
* This function will put the passed integer into the file string at the given
* position. This is used for writing two byte integers. The default maximum
* value can be 32767, but if you set $unsigned to 1 then the maximum is raised to
* 65535 as long as it is a positive value.
*
* @param string $fileString
* @param integer $int
* @param integer $position
* @param boolean $unsigned
* @return string
*/
function PutInteger($fileString,
                    $int,
                    $position,
                    $unsigned = 0) {
    if (!is_string($fileString)) {
        $fileString = (string)  $fileString;
    }
    if (!is_int($int)) {
        $int = (integer) $int;
    }
    if (!is_int($position)) {
        $position = (integer) $position;
    }
    if (!is_bool($unsigned)) {
        $unsigned = (boolean) $unsigned;
    }

    if (strlen($fileString) < $position) {
       $fileString = $fileString . str_replace('*', chr(0), str_pad('', $position, '*'));
    }

    $position--;

    $int = MakeInteger($int, $unsigned);
    $fileString = substr_replace($fileString, $int, $position, 2);

    // Return the new string.
    return $fileString;
}


/**
* Va-Functions.inc.php :: PutLongInt()
*
* This function will put the passed integer into the file string at the given
* position. This is used for writing four byte integers. The maximum value
* can be 2147483647, but if you set $unsigned to 1, then the maximum is raise to
* 4294967295 as long as it is a positive value.
*
* @param string $fileString
* @param integer $int
* @param integer $position
* @param boolean $unsigned
* @return string
*/
function PutLongInt($fileString,
                    $int,
                    $position,
                    $unsigned = 0) {
    if (!is_string($fileString)) {
        $fileString = (string)  $fileString;
    }
    if (!is_int($int)) {
        $int = (integer) $int;
    }
    if (!is_int($position)) {
        $position = (integer) $position;
    }
    if (!is_bool($unsigned)) {
        $unsigned = (boolean) $unsigned;
    }

    if (strlen($fileString) < $position) {
       $fileString = $fileString . str_replace('*', chr(0), str_pad('', $position, '*'));
    }

    $position--;

    $int = MakeLongInt($int, $unsigned);
    $fileString = substr_replace($fileString, $int, $position, 4);

    // Return the new string.
    return $fileString;
}


/**
* Va-Functions.inc.php :: PutSingle()
*
* This function will put the passed single precision value into the file string
* at the given position.
*
* @param string $fileString
* @param integer $float
* @param integer $position
* @return string
*/
function PutSingle($fileString,
                   $float,
                   $position) {
    if (!is_string($fileString)) {
        $fileString = (string)  $fileString;
    }
    if (!is_float($float)) {
        $float = (float)   $float;
    }
    if (!is_int($position)) {
        $position = (integer) $position;
    }

    if (strlen($fileString) < $position) {
       $fileString = $fileString . str_replace('*', chr(0), str_pad('', $position, '*'));
    }

    $position--;

    $float = MakeSingle($float);
    $fileString = substr_replace($fileString, $float, $position, 4);

    // Return the new string.
    return $fileString;
}


/**
* Va-Functions.inc.php :: PutString()
*
* This function will put the passed string into the file string at the given
* position.
*
* @param string $fileString
* @param string $string
* @param integer $position
* @param integer $length
* @return string
*/
function PutString($fileString,
                   $string,
                   $position,
                   $length) {
    if (!is_string($fileString)) {
        $fileString = (string)  $fileString;
    }
    if (!is_string($string)) {
        $string = (string)  $string;
    }
    if (!is_int($position)) {
        $position = (integer) $position;
    }
    if (!is_int($length)) {
        $length = (integer) $length;
    }

    if (strlen($fileString) < $position) {
       $fileString = $fileString . str_replace('*', chr(0), str_pad('', $position, '*'));
    }

    $position--;

    if (strlen($string) < $length) {
        $string = str_pad($string, $length, chr(32));
    } else {
        $string = substr($string, 0, $length);
    }
    $fileString = substr_replace($fileString, $string, $position, $length);

    // Return the new string.
    return $fileString;
}


/**
* Va-Functions.inc.php :: ReadAt2kConfig()
*
* This function will read an AT2k script configuration file, which usually is
* stored in the BBS's TXT directory.
*
* The configuration file has comments designated by a semicolon, and values are
* positioned based (they must appear in the file in a certain order and are not
* assigned to variables).
*
* If trim is set to FALSE, then it will not trim the line of any spaces.
*
* @param string $configFile
* @return array
*/
function ReadAt2kConfig($configFile,
                        $trim = 1) {
    if (!is_string($configFile)) {
        $configFile = (string) $configFile;
    }
    if (!is_bool($trim)) {
        $trim = (boolean) $trim;
    }

    // If the configfile is missing, send an error.
    if (!file_exists($configFile)) {
        return FALSE;
    }

    // Read the configfile into an array.
    $file = @file($configFile);
    $cnt = 1;

    for ($i = 0; $i < count($file); $i++) {
        // Read a line.
        $line = trim($file[$i]);
        $comment = trim(substr($line, 0, 1));
        if (($comment != ';') &&
            ($line != '')) {
            if ($trim == TRUE) {
                // Add the line into the array.
                $var[$cnt] = $line;
            } else {
                // Add non trimmed line into the array.
                $var[$cnt] = $file[$i];
            }
            $cnt++;
        }
    }

    return $var;
}


/**
* Va-Functions.inc.php :: ReadCfgFile()
*
* This function will read the entire file specified into one variable.
*
* @param string $file
* @return string
*/
function ReadCfgFile($file) {
    if (!is_string($file)) {
        $file = (string) $file;
    }

    // Check to see if the file exists, if not return nothing.
    if (!file_exists($file)) {
        // If file is missing, then it is blank.
        return '';
    }
    if (!is_readable($file)) {
        // If file exists but cannot be read, return FALSE.
        return FALSE;
    }

    $fileString = '';

    if (filesize($file) == 0) {
        return $fileString;
    }

    if (version_compare(phpversion(), '4.3.0', '>=')) {
        if (!$fileString = @file_get_contents($file)) {
            return FALSE;
        }
    } else if (file_exists($file)) {
        if (!$fp = @fopen($file, 'rb')) {
            return FALSE;
        } else {
            if (flock($fp, LOCK_SH | LOCK_NB)) {
                $fileString = fread($fp, filesize($file));
                flock($fp, LOCK_UN);
            }
            fclose($fp);
        }
    }

    // Return the file string.
    return $fileString;
}


/**
* Va-Functions.inc.php :: ReadConfig()
*
* This function will read a configuration file and place its variables into
* an array.
*
* The type of configuration file has comments designated by a semicolon, and
* variables are assigned like 'variable='value''.
*
* @param string $configFile
* @return array
*/
function ReadConfig($configFile) {
    if (!is_string($configFile)) {
        $configFile = (string) $configFile;
    }

    // If the configfile is missing, return the default values.
    if (!file_exists($configFile)) {
        return FALSE;
    }

    // Read the configfile into an array.
    $file = @file($configFile);
    $cnt = 1;

    for ($i = 0; $i < count($file); $i++) {
        // Read a line.
        $line = trim($file[$i]);
        $comment = trim(substr($line, 0, 1));
        if (($comment != ';') &&
            ($line != '')) {
            // Add the line into the array.
            $var[$cnt] = $line;
            $cnt++;
        }
    }

    // The configuration is now in an array, so lets set the proper variables
    // with the information.

    // Examine the array and set new values.
    for ($j = 1; $j != $cnt; $j++) {
        // Determine the position of the equal symbol.
        $pos = strpos($var[$j], '=');

        // Determine the variable description.
        $desc = trim(strtolower(substr($var[$j], 0, $pos)));

        // Determine the variable value.
        $value = trim(substr($var[$j], $pos + 1));

        // Remove any spaces.
        $value = str_replace(' ', '', $value);
        $config[$desc] = strtolower($value);
    }

    // Return the configuration array.
    return $config;
}


/**
* Va-Functions.inc.php :: ReadID3Info()
*
* This function will read the ID3 information from a MP3 file.
*
* @param [type] $file
* @return
*/
function ReadID3Info($file) {
    if (!is_string($file)) {
        $file = (string) $file;
    }

    // Set default information.
    $id3['tag'] = '';
    $id3['songname'] = '';
    $id3['artist'] = '';
    $id3['album'] = '';
    $id3['year'] = '';
    $id3['comment'] = '';
    $id3['genre'] = '';

    if (!file_exists($file)) {
        return $id3;
    }

    $length = filesize($file);
    if ($length < 128) {
        return $id3;
    }

    if (!($fp = @fopen($file, 'rb'))) {
        return $id3;
    }

    // Set pointer for last 128 bytes.
    fseek($fp, $length - 128);

    // Read the last 128 bytes.
    $fileString = fread($fp, 128);
    fclose($fp);

    $id3['tag'] = GetString($fileString, 1, 3);
    if ($id3['tag'] == 'TAG') {
        $id3['songname'] = GetString($fileString, 4, 30);
        $id3['artist'] = GetString($fileString, 34, 30);
        $id3['album'] = GetString($fileString, 64, 30);
        $id3['year'] = GetString($fileString, 94, 4);
        $id3['comment'] = GetString($fileString, 98, 30);
        $genre = GetChar($fileString, 128);
        switch ($genre) {
            case 0:     $id3['genre'] = 'Blues';                break;
            case 1:     $id3['genre'] = 'Classic Rock';         break;
            case 2:     $id3['genre'] = 'Country';              break;
            case 3:     $id3['genre'] = 'Dance';                break;
            case 4:     $id3['genre'] = 'Disco';                break;
            case 5:     $id3['genre'] = 'Funk';                 break;
            case 6:     $id3['genre'] = 'Grunge';               break;
            case 7:     $id3['genre'] = 'Hip-Hop';              break;
            case 8:     $id3['genre'] = 'Jazz';                 break;
            case 9:     $id3['genre'] = 'Metal';                break;
            case 10:    $id3['genre'] = 'New Age';              break;
            case 11:    $id3['genre'] = 'Oldies';               break;
            case 12:    $id3['genre'] = 'Other';                break;
            case 13:    $id3['genre'] = 'Pop';                  break;
            case 14:    $id3['genre'] = 'R&B';                  break;
            case 15:    $id3['genre'] = 'Rap';                  break;
            case 16:    $id3['genre'] = 'Reggae';               break;
            case 17:    $id3['genre'] = 'Rock';                 break;
            case 18:    $id3['genre'] = 'Techno';               break;
            case 19:    $id3['genre'] = 'Industrial';           break;
            case 20:    $id3['genre'] = 'Alternative';          break;
            case 21:    $id3['genre'] = 'Ska';                  break;
            case 22:    $id3['genre'] = 'Death Metal';          break;
            case 23:    $id3['genre'] = 'Pranks';               break;
            case 24:    $id3['genre'] = 'Soundtrack';           break;
            case 25:    $id3['genre'] = 'Euro-Techno';          break;
            case 26:    $id3['genre'] = 'Ambient';              break;
            case 27:    $id3['genre'] = 'Trip-hop';             break;
            case 28:    $id3['genre'] = 'Vocal';                break;
            case 29:    $id3['genre'] = 'Jazz+Funk';            break;
            case 30:    $id3['genre'] = 'Fusion';               break;
            case 31:    $id3['genre'] = 'Trance';               break;
            case 32:    $id3['genre'] = 'Classical';            break;
            case 33:    $id3['genre'] = 'Instrumental';         break;
            case 34:    $id3['genre'] = 'Acid';                 break;
            case 35:    $id3['genre'] = 'House';                break;
            case 36:    $id3['genre'] = 'Game';                 break;
            case 37:    $id3['genre'] = 'Sound Clip';           break;
            case 38:    $id3['genre'] = 'Gospel';               break;
            case 39:    $id3['genre'] = 'Noise';                break;
            case 40:    $id3['genre'] = 'AlternRock';           break;
            case 41:    $id3['genre'] = 'Bass';                 break;
            case 42:    $id3['genre'] = 'Soul';                 break;
            case 43:    $id3['genre'] = 'Punk';                 break;
            case 44:    $id3['genre'] = 'Space';                break;
            case 45:    $id3['genre'] = 'Meditative';           break;
            case 46:    $id3['genre'] = 'Instrumental Pop';     break;
            case 47:    $id3['genre'] = 'Instrumental Rock';    break;
            case 48:    $id3['genre'] = 'Ethnic';               break;
            case 49:    $id3['genre'] = 'Gothic';               break;
            case 50:    $id3['genre'] = 'Darkwave';             break;
            case 51:    $id3['genre'] = 'Techno-Industrial';    break;
            case 52:    $id3['genre'] = 'Electronic';           break;
            case 53:    $id3['genre'] = 'Pop-Folk';             break;
            case 54:    $id3['genre'] = 'Eurodance';            break;
            case 55:    $id3['genre'] = 'Dream';                break;
            case 56:    $id3['genre'] = 'Southern Rock';        break;
            case 57:    $id3['genre'] = 'Comedy';               break;
            case 58:    $id3['genre'] = 'Cult';                 break;
            case 59:    $id3['genre'] = 'Gansta';               break;
            case 60:    $id3['genre'] = 'Top 40';               break;
            case 61:    $id3['genre'] = 'Christian Rap';        break;
            case 62:    $id3['genre'] = 'Pop/Funk';             break;
            case 63:    $id3['genre'] = 'Jungle';               break;
            case 64:    $id3['genre'] = 'Native American';      break;
            case 65:    $id3['genre'] = 'Cabaret';              break;
            case 66:    $id3['genre'] = 'New Wave';             break;
            case 67:    $id3['genre'] = 'Psychadelic';          break;
            case 68:    $id3['genre'] = 'Rave';                 break;
            case 69:    $id3['genre'] = 'Showtunes';            break;
            case 70:    $id3['genre'] = 'Trailer';              break;
            case 71:    $id3['genre'] = 'Lo-Fi';                break;
            case 72:    $id3['genre'] = 'Tribal';               break;
            case 73:    $id3['genre'] = 'Acid Punk';            break;
            case 74:    $id3['genre'] = 'Acid Jazz';            break;
            case 75:    $id3['genre'] = 'Polka';                break;
            case 76:    $id3['genre'] = 'Retro';                break;
            case 77:    $id3['genre'] = 'Musical';              break;
            case 78:    $id3['genre'] = 'Rock & Roll';          break;
            case 79:    $id3['genre'] = 'Hard Rock';            break;
            case 80:    $id3['genre'] = 'Folk';                 break;
            case 81:    $id3['genre'] = 'Folk-Rock';            break;
            case 82:    $id3['genre'] = 'National Rock';        break;
            case 83:    $id3['genre'] = 'Swing';                break;
            case 84:    $id3['genre'] = 'Fast Fusion';          break;
            case 85:    $id3['genre'] = 'Bebob';                break;
            case 86:    $id3['genre'] = 'Latin';                break;
            case 87:    $id3['genre'] = 'Revival';              break;
            case 88:    $id3['genre'] = 'Celtic';               break;
            case 89:    $id3['genre'] = 'Bluegrass';            break;
            case 90:    $id3['genre'] = 'Avantgarde';           break;
            case 91:    $id3['genre'] = 'Gothic Rock';          break;
            case 92:    $id3['genre'] = 'Progressive Rock';     break;
            case 93:    $id3['genre'] = 'Psychadelic Rock';     break;
            case 94:    $id3['genre'] = 'Symphonic Rock';       break;
            case 95:    $id3['gnere'] = 'Slow Rock';            break;
            case 96:    $id3['genre'] = 'Big Band';             break;
            case 97:    $id3['genre'] = 'Chorus';               break;
            case 98:    $id3['genre'] = 'Easy Listening';       break;
            case 99:    $id3['genre'] = 'Acoustic';             break;
            case 100:   $id3['genre'] = 'Humour';               break;
            case 101:   $id3['genre'] = 'Speech';               break;
            case 102:   $id3['genre'] = 'Chanson';              break;
            case 103:   $id3['genre'] = 'Opera';                break;
            case 104:   $id3['genre'] = 'Chamber Music';        break;
            case 105:   $id3['genre'] = 'Sonata';               break;
            case 106:   $id3['genre'] = 'Symphony';             break;
            case 107:   $id3['genre'] = 'Booty Bass';           break;
            case 108:   $id3['genre'] = 'Primus';               break;
            case 109:   $id3['genre'] = 'Porn Groove';          break;
            case 110:   $id3['genre'] = 'Satire';               break;
            case 111:   $id3['genre'] = 'Slow Jam';             break;
            case 112:   $id3['genre'] = 'Club';                 break;
            case 113:   $id3['genre'] = 'Tango';                break;
            case 114:   $id3['genre'] = 'Samba';                break;
            case 115:   $id3['genre'] = 'Folklore';             break;
            case 116:   $id3['genre'] = 'Ballad';               break;
            case 117:   $id3['genre'] = 'Power Ballad';         break;
            case 118:   $id3['genre'] = 'Rhythmic Sould';       break;
            case 119:   $id3['genre'] = 'Freestyle';            break;
            case 120:   $id3['genre'] = 'Duet';                 break;
            case 121:   $id3['genre'] = 'Punk Rock';            break;
            case 122:   $id3['genre'] = 'Drum Solo';            break;
            case 123:   $id3['genre'] = 'A Capella';            break;
            case 124:   $id3['genre'] = 'Euro-House';           break;
            case 125:   $id3['genre'] = 'Dance Hall';           break;
            default:    $id3['genre'] = 'Unknown';              break;
        }
    }

    // Return the tag information.
    return $id3;
}


/**
* Va-Funtions.inc.php :: RedirectUser()
*
* This function will redirect the user to the specified URL. It uses a primary
* base of JavaScript with a backup meta tag for those browsers that do not
* support JavaScript. Note that when this is called, the current page will
* exit and the redirection will take place. You may set delay the redirection
* a specific number of seconds by using the optional $delay variable.
*
* @param string $url
* @param integer $delay
* @param boolean $forceSSL
*/
function RedirectUser($url = '',
                      $delay = 0,
                      $forceSSL = 0) {
    if (!is_string($url)) {
        $url = (string)  $url;
    }
    if (!is_int($delay)) {
        $delay = (integer) $delay;
    }
    if (!is_bool($forceSSL)) {
        $forceSSL = (boolean) $forceSSL;
    }

    global $includeDir;

    if (ob_get_length()) {
        // Flush the current buffer.
        ob_clean();
    }

    // Stop session writing.
    session_write_close();

    // If URL is blank, set it to current URL.
    $url = trim($url);
    if ($url == '') {
        $url = 'http://' . $_SERVER['SERVER_NAME'] . $_SERVER['REQUEST_URI'];
    }

    // Force SSL.
    if ($forceSSL) {
        $url = str_replace('http://', 'https://', $url);
    }

    if ((!headers_sent()) &&
        (func_num_args() <= 2) &&
        ((strpos($url, 'http://') !== FALSE) ||
         (strpos($url, 'https://') !== FALSE)) &&
        (strpos(strtoupper($_SERVER['SERVER_SOFTWARE']), 'IIS') === FALSE)) {
        if ($delay > 0) {
            sleep($delay);
        }
        header('Location: ' . $url);
    } else {
?>

<html>
<head>
    <meta http-equiv="Refresh" content="<?php print $delay; ?>; url=<?php print $url; ?>" />
</head>
<body>
    <noscript>
        <b>Your browser does not have JavaScript enabled.</b><br />
        Enable JavaScript for a better experience while using this site.<br />
        Click <a href="<?php print $url; ?>">here</a> to continue.
    </noscript>
</body>
</html>

<?php
    }

    die();
}


/**
* Va-Functions.inc.php :: SendFile()
*
* This function will send the specified file to the user using the fpassthru()
* function.
*
* @param string $file
* @return mixed
*/
function SendFile($file) {
    if (!is_string($file)) {
        $file = (string) $file;
    }

    if (!file_exists($file)) {
        return FALSE;
    }

    if (ob_get_length()) {
        // Turn off buffering.
        ob_end_clean();
    }

    // Stop session writing.
    session_write_close();

    // Set the status to default to false.
    $status = FALSE;
    $filename = basename($file);
    $filename = preg_replace('/\./', '%2e', $filename, substr_count($filename, '.') - 1);
    $filesize = filesize($file);
    //$filesizemb = intval($filesize / 1048576);

    // Determine MIME type.
    $pos = strrpos($filename, '.');
    $ext = strtolower(substr($filename, $pos));
    switch ($ext) {
        case '.ai':   $mimetype='application/postscript';   $encode='7bit';     break;
        case '.avi':  $mimetype='video/x-msvideo';          $encode='base64';   break;
        case '.doc':  $mimetype='application/msword';       $encode='base64';   break;
        case '.eps':  $mimetype='application/postscript';   $encode='7bit';     break;
        case '.exe':  $mimetype='application/octet-stream'; $encode='base64';   break;
        case '.gif':  $mimetype='image/gif';                $encode='base64';   break;
        case '.htm':  $mimetype='text/html';                $encode='7bit';     break;
        case '.html': $mimetype='text/html';                $encode='7bit';     break;
        case '.jpe':  $mimetype='image/jpeg';               $encode='base64';   break;
        case '.jpeg': $mimetype='image/jpeg';               $encode='base64';   break;
        case '.jpg':  $mimetype='image/jpeg';               $encode='base64';   break;
        case '.mpe':  $mimetype='video/mpeg';               $encode='base64';   break;
        case '.mpeg': $mimetype='video/mpeg';               $encode='base64';   break;
        case '.mov':  $mimetype='video/quicktime';          $encode='base64';   break;
        case '.mpg':  $mimetype='video/mpeg';               $encode='base64';   break;
        case '.pdf':  $mimetype='application/pdf';          $encode='base64';   break;
        case '.ps':   $mimetype='application/postscript';   $encode='7bit';     break;
        case '.qt':   $mimetype='video/quicktime';          $encode='base64';   break;
        case '.rtf':  $mimetype='application/rtf';          $encode='7bit';     break;
        case '.tar':  $mimetype='application/octet-stream'; $encode='base64';   break;
        case '.tif':  $mimetype='image/tiff';               $encode='base64';   break;
        case '.tiff': $mimetype='image/tiff';               $encode='base64';   break;
        case '.txt':  $mimetype='text/plain';               $encode='7bit';     break;
        case '.wav':  $mimetype='audio/x-wav';              $encode='base64';   break;
        case '.zip':  $mimetype='application/zip';          $encode='base64';   break;
        default:      $mimetype='application/octet-stream'; $encode='base64';   break;
    }

    // Set the headers.
    header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
    header("Cache-Control: no-store, no-cache, must-revalidate");
    header("Cache-Control: private");
    header('Pragma: public');
    header('Expires: 0');
    header('Accept-Ranges: bytes');
    header('Content-Description: File Transfer');
    header('Content-type: ' . $mimetype);
    header('Content-Disposition: attachment; filename="' . $filename . '"');
    header('Content-Transfer-Encoding: ' . $encode);
    header('Content-Length: ' . $filesize);

    // Send the file using fread.
    if ($fp = @fopen($file, 'rb')) {
        while(!feof($fp)) {
            set_time_limit(0);
            print fread($fp, 1024 * 8);
            flush();
            ob_flush();
        }
        fclose($fp);
        $status = (connection_status() == 0);
    }

    // Send the file using fpassthru.
//    $fp = @fopen($file, 'rb');
 //   $status = fpassthru($fp);

    // Return the status.
    return $status;
}


/**
* Va-Functions.inc.php :: SendOneLiner()
*
* This function will send a one-liner to the specified node, or to all active
* nodes. If the userNo is 0, then it will send the message to any user. If
* the node is set to all, it will send it to all active nodes.
*
* @param string $message
* @param integer $userNo
* @param string $node
* @return mixed
*/
function SendOneLiner($message,
                      $userNo = 0,
                      $node = 'All') {
    if (!is_string($message)) {
        $message = (string)  $message;
    }
    if (!is_int($userNo)) {
        $userNo = (integer) $userNo;
    }
    if (!is_string($node)) {
        $node = (string)  $node;
    }

    $classControlDat = new datControl;

    // Truncate the message if too long.
    if (strlen($message) > 75) {
        $message = substr($message, 0, 75);
    }
    // Add coloring.
    $message = chr(3) . '6*** ' . chr(3) . '1' . $message . "\r\n";

    $status = FALSE;
    if (($userNo == 0) &&
        ($node != 'All')) {
        $file = $GLOBALS['VA_SYSPATHS']['control'] . '/IN' . $node . '.QUE';
        // Open the file.
        if ($fp = @fopen($file, 'a')) {
            // Lock the file.
            flock($fp, LOCK_EX);

            // Write the message.
            $status = fwrite($fp, $message);

            // Unlock the file.
            flock($fp, LOCK_UN);
            fclose($fp);
        }
    } else {
        if ($userNo > 0) {
            for ($i = 0; $i < $GLOBALS['VA_MAIN']['maxchannelno']; $i++) {
                $channel = $classControlDat->Get($i);
                if ($channel['userno'] == $userNo) {
                    $file = $GLOBALS['VA_SYSPATHS']['control'] . '/IN' . $i . '.QUE';
                    // Open the file.
                    if ($fp = @fopen($file, 'a')) {
                        // Lock the file.
                        flock($fp, LOCK_EX);

                        // Write the message.
                        $status = fwrite($fp, $message);

                        // Unlock the file.
                        flock($fp, LOCK_UN);
                        fclose($fp);
                    }
                }
            }
        } else {
            for ($i = 0; $i < $GLOBALS['VA_MAIN']['maxchannelno']; $i++) {
                $channel = $classControlDat->Get($i);
                if ($channel['userno'] > 0) {
                    $file = $GLOBALS['VA_SYSPATHS']['control'] . '/IN' . $i . '.QUE';
                    // Open the file.
                    if ($fp = @fopen($file, 'a')) {
                        // Lock the file.
                        flock($fp, LOCK_EX);

                        // Write the message.
                        $status = fwrite($fp, $message);

                        // Unlock the file.
                        flock($fp, LOCK_UN);
                        fclose($fp);
                    }
                }
            }
        }
    }

    // Return the status.
    return $status;
}


/**
* Va-Functions.inc.php :: StripANSICodes()
*
* This function will strip all standard ANSI codes from the passed string.
*
* @param string $string
* @return string
*/
function StripANSICodes($string) {
    if (!is_string($string)) {
        $string = (string) $string;
    }

    // Check to see if the variable was passed, if not then return the string.
    if (trim($string) == '') {
        return $string;
    }

    $pattern = array(
                    "/\e\[(\d){0,3}[A|B|C|D]/",
                    "/\e\[((\?*?(\d){1,2})|((\d){1,2};{0,1})){0,2}[f|R|h|H|l|y]/",
                    "/\e\[2J/",
                    "/\e\[(((\d){1,2};{0,1}){0,4})[m|p]/",
                    "/\e\[6[n]/",
                    "/\e\[((\d){1,2};{0,1}){0,3}[@|L|M|P]/",
                    "/\e\[[K|s|u]/"
                    );

    // Return the new string.
    $result = preg_replace($pattern, '', $string);
    $result = str_replace(chr(27), '', $result);

    return $result;
}


/**
* Va-Functions.inc.php :: StripASCIICodes()
*
* This function will strip all non-standard ASCII codes from the passed string.
*
* @param string $string
* @return string
*/
function StripASCIICodes($string) {
    if (!is_string($string)) {
        $string = (string) $string;
    }

    // Check to see if the variable was passed, if not then return the string.
    if (trim($string) == '') {
        return $string;
    }

    // Return the new string.
    $result = preg_replace('/[^(\x20-\x7F)]*/', '', $string);

    return $result;
}


/**
* Va-Functions.inc.php :: StripBBSCodes()
*
* This routine will take $string and remove any heart codes within the string.
* It also removes diamond codes.
*
* @param string $string
* @return string
*/
function StripBBSCodes($string) {
    if (!is_string($string)) {
        $string = (string) $string;
    }

    // Check to see if the variable was passed, if not then return the string.
    if (trim($string) == '') {
        return $string;
    }

    // Search for SBBS codes.
    $string = preg_replace("/\x01\S/", '', $string);

    // Search for hearts.
    $string = preg_replace("/\x03\S/", '', $string);

    // Search the file for diamonds.
    $string = preg_replace("/\x04\S/", '', $string);

    // Search for pipe codes.
    $string = preg_replace("/\|(\d){2,2}/", '', $string);

    // Search for PcBoard codes.
    $string = preg_replace("/@X\S\S/", '', $string);

    // Search for Wildcat codes.
    $string = preg_replace("/@\S\S@/", '', $string);

    // Search for Celerity codes.
    $string = preg_replace("/\|[k|b|g|c|r|m|y|w|d|B|G|C|R|M|Y|W|S]/", '', $string);

    // Return the new string.
    return $string;
}


/**
* Va-Functions.inc.php :: StripRouting()
*
* This function will take $string and remove the lines that begin with a
* diamond code.
*
* @param string $string
* @return string
*/
function StripRouting($string) {
    if (!is_string($string)) {
        $string = (string) $string;
    }

    // Check to see if the variable was passed, if not then return the string.
    if (trim($string) == '') {
        return $string;
    }

    // Search for diamonds.
    for ($position = 0; is_int($position);) {
        $position = strpos($string, chr(4), 0);
        if (is_int($position)) {
            $endPosition = strpos($string, "\n", $position);
            if ($endPosition === FALSE) {
                $endPosition = strlen($string);
            }
            $string = substr_replace($string, '', $position, ($endPosition - $position));
        }
    }

    // Return the new string.
    return rtrim($string);
}


/**
* Va-Functions.inc.php :: UnHtmlSpecialChars()
*
* Reverses the htmlspecialchars() function.
*
* @param string $string
* @return string
*/
function UnHtmlSpecialChars($string) {
    if (!is_string($string)) {
        $string = (string) $string;
    }

    $string = str_replace('&amp;', '&', $string);
    $string = str_replace('&#039;', '\'', $string);
    $string = str_replace('&quot;', '"', $string);
    $string = str_replace('&lt;', '<', $string);
    $string = str_replace('&gt;', '>', $string);

    return $string;
}


/**
* Va-Functions.inc.php :: UpdateWhosOnline()
*
* This function will maintain the who's online listing for VADV-PHP. It is
* automatically called by vadv-php.inc.php. The variable action records
* what action the user is taking.
*
* You can set the timeout, which will change the user's time entry. If you set
* it more than 900, then the user will stay on the who's list longer, or if you
* set it below 900, then the user will stay on the list a shorter period of time.
*
* If id is set to true, then it will assign a new id to the user.
*
* @return array
*/
function UpdateWhosOnline($action = '',
                          $timeout = 900,
                          $resetId = 0) {
    if (!is_string($action)) {
        $action = (string) $action;
    }
    if (!is_int($timeout)) {
        $timeout = (integer) $timeout;
    }
    if (!is_bool($resetId)) {
        $resetId = (boolean) $resetId;
    }

    $classStatVBBSDat = new datStatVBBS;

    // Offset size.
    $fileOffset = 193;

    // Database filename.
    $file = $GLOBALS['VA_SYSPATHS']['data'] . '/WEBCALLER.LST';

    // If the filesize doesn't match up to the file offset, delete it and start over.
    if ((file_exists($file)) &&
        ((filesize($file) % $fileOffset) != 0)) {
        @unlink ($file);
    }

    // Read the file.
    $fileString = ReadCfgFile($file);

    // Read through the file looking for the current user's IP, and also expired users. Users expire after 15 mins (900s).
    $user = array();
    $online = array();
    $onlineusercnt = 0;
    $offset = 0;
    for ($i = 1; $offset < strlen($fileString); $i++) {
        $user['session'] = trim(GetString($fileString, 1 + $offset, 32));
        $user['ip'] = trim(GetString($fileString, 33 + $offset, 15));
        $user['time'] = (integer) GetString($fileString, 48 + $offset, 12);
        $user['userno'] = (integer) GetLongInt($fileString, 60 + $offset);
        $user['handle'] = trim(GetString($fileString, 64 + $offset, 40));
        $user['action'] = trim(GetString($fileString, 104 + $offset, 24));
        $user['id'] = trim(GetString($fileString, 128 + $offset, 64));
        $user['ssl'] = (integer) GetInteger($fileString, 192 + $offset);

        // Compare values to see who to remove.
        if (($user['session'] == trim(session_id())) ||
            (time() - $user['time'] > 900) ||
            (($GLOBALS['USER']['userno'] > 0) &&
             (($user['userno'] == $GLOBALS['USER']['userno']) ||
              ($user['handle'] == $GLOBALS['USER']['handle'])))) {
            if (($user['id'] != '') &&
                (!$resetId)) {
                $id = $user['id'];
            }
            $fileString = substr_replace($fileString, '', $offset, $fileOffset);
            $i--;
        } else {
            // Write to the new fileString...
            $offset = $fileOffset * $i;
            $online[$i]['session'] = trim($user['session']);
            $online[$i]['ip'] = trim($user['ip']);
            $online[$i]['time'] = trim($user['time']);
            $online[$i]['userno'] = trim($user['userno']);
            $online[$i]['handle'] = trim($user['handle']);
            $online[$i]['action'] = trim($user['action']);
            $online[$i]['id'] = trim($user['id']);
            $online[$i]['ssl'] = trim($user['ssl']);

            if ($online[$i]['userno'] > 0) {
                $onlineusercnt++;
            }
        }
    }

    // Add the current user's entry to the end if not a web spider.
    if ((!DetectSpider()) &&
        (strtolower($action) != 'rss')) {
        // Determine time difference
        $timeout = $timeout - 900;

        $sess = str_pad(session_id(), 32);
        $ip = str_pad($_SERVER['REMOTE_ADDR'], 15);
        $time = str_pad(time() + $timeout, 12);
        $userno = MakeLongInt($GLOBALS['USER']['userno']);
        $handle = str_pad($GLOBALS['USER']['handle'], 40);
        $action = str_pad(trim($action), 24);
        if (!isset($id)) {
            $id = str_pad(hash($GLOBALS['CONFIG']['encryption'], rand(-32767, 32767)), 64);
        } else {
            $id = str_pad($id, 64);
        }
        if (IsHttps()) {
            $ssl = 1;
        } else {
            $ssl = 0;
        }
        $sslString = MakeInteger($ssl);

        $fileString = $fileString . $sess . $ip . $time . $userno . $handle . $action . $id . $sslString;

        $online[$i]['session'] = trim($sess);
        $online[$i]['ip'] = trim($ip);
        $online[$i]['time'] = trim($time);
        $online[$i]['userno'] = trim($GLOBALS['USER']['userno']);
        $online[$i]['handle'] = trim($GLOBALS['USER']['handle']);
        $online[$i]['action'] = trim($action);
        $online[$i]['id'] = trim($id);
        $online[$i]['ssl'] = trim($ssl);
    } else {
        $i--;
    }

    // Write the new file.
    WriteCfgFile($file, $fileString);

    // Organize the array alphabetically by handle.
    if (count($online) > 0) {
        foreach ($online as $row) {
            $sortarr[] = $row['handle'];
        }
        array_multisort($sortarr, $online);
        array_unshift($online, $i);
    } else {
        $online[0] = 0;
    }

    $fileString = $classStatVBBSDat->Read();
    if ($fileString !== FALSE) {
        $stats = $classStatVBBSDat->Get($fileString);
        // Update max users online.
        if ($onlineusercnt > $stats['webmaxusers']) {
            $stats['webmaxusers'] = $onlineusercnt;
            $fileString = $classStatVBBSDat->Put($fileString, $stats);
            $classStatVBBSDat->Write($fileString);
        }
    }

    // Return the online users.
    return $online;
}


/**
* Va-Functions.inc.php :: UserDate()
*
* This function is just like the PHP date() function except it will factor in
* the user's timezone.
*
* @param [type] $format
* @return
*/
function UserDate($format) {
    $timeStamp = time() - ((integer) date("O") / 100 * 60 * 60) + ($GLOBALS['USER_SETTINGS']['timezone'] / 100 * 60 * 60);
    return date($format, $timeStamp);
}


/**
* Va-Functions.inc.php :: VA_Die()
*
* This function replaces the php die() command. It will display an error message
* for five seconds, and then if the directory contains an index.htm, index.html
* or index.php file, it will redirect to it.
*
* @param string $message
* @param integer $delay
*/
function VA_Die($message,
                $delay = 5) {
    if (!is_string($message)) {
        $message = (string)  $message;
    }
    if (!is_int($delay)) {
        $delay = (integer) $delay;
    }

    $index = '';
    if (file_exists('index.php')) {
        $index = 'index.php';
    } else if (file_exists('index.htm')) {
        $index = 'index.htm';
    } else if (file_exists('index.html')) {
        $index = 'index.html';
    }

    if ($index != '') {
        print $message;
        ob_flush();
        RedirectUser($index, $delay);
    } else {
        die($message);
    }
}


/**
* Va-Functions.inc.php :: VA_Error()
*
* This function will return the specified error message.
*
* @param integer $errNum
* @return string
*/
function VA_Error($errNum) {
    if (!is_int($errNum)) {
        $errNum = (integer) $errNum;
    }

    switch ($errNum) {
        case 0:
            $errMsg = 'File not found.';
            break;
        case 1:
            $errMsg = 'Variable is not proper type.';
            break;
        case 2:
            $errMsg = 'Value is out of range.';
            break;
        case 3:
            $errMsg = 'Invalid Configuration.';
            break;
        default:
            $errMsg = 'Unknown Error.';
            break;
    }

    // Return the error message.
    return (string) $errMsg;
}


/**
* Va-Functions.inc.php :: WMLSpecialChars()
*
* This function will take the string and convert any characters within it to
* be WML compatiable.
*
* @param string $text
* @return string
*/
function WMLSpecialChars($string) {
    if (!is_string($string)) {
        $string = (string) $string;
    }

    $string = str_replace('"', '&#34;', $string);
    $string = str_replace('&', '&#38;', $string);
    $string = str_replace("'", '&#39;', $string);
    $string = str_replace('<', '&#60;', $string);
    $string = str_replace('>', '&#62;', $string);
    $string = str_replace('  ', '&#160;&#160;', $string);

    $string = str_replace('$', '$$', $string);

    // Return the new string.
    return $string;
}


/**
* Va-Functions.inc.php :: WriteCfgFile()
*
* This function will create a new configuration file based on the file string
* passed.
*
* @param string $file
* @param string $fileString
* @return mixed
*/
function WriteCfgFile($file, $fileString) {
    if ($fileString === FALSE) {
        return FALSE;
    }

    if (!is_string($file)) {
        $file = (string) $file;
    }
    if (!is_string($fileString)) {
        $fileString = (string) $fileString;
    }

    $status = FALSE;

    if (version_compare(phpversion(), '5.0.0', '>=')) {
        if (!$status = @file_put_contents($file, $fileString)) {
            return FALSE;
        }
    } else {
        // Open the file.
        if (!$fp = @fopen($file, 'wb')) {
            return FALSE;
        } else {
            // Lock the file.
            if (flock($fp, LOCK_EX | LOCK_NB)) {
                // Write the new file.
                $status = fwrite($fp, $fileString, strlen($fileString));

                // Unlock the file.
                flock($fp, LOCK_UN);
            }

            // Close the file.
            fclose($fp);
        }
    }

    // Return the write status.
    return $status;
}


/**
* Va-Functions.inc.php :: WriteLog()
*
* This function will log a line into the specified file which is stored in
* the BBS's data directory. The logfile is a filename. MaxSize is an optional
* parameter which can be used to reset the log file if it is larger than the
* value (in bytes).
*
* @param string $logFilename
* @param string $line
* @param integer $maxSize
* @return mixed
*/
function WriteLog($logFilename,
                  $line,
                  $maxSize = 0) {
    if (!is_string($logFilename)) {
        $logFilename = (string)  $logFilename;
    }
    if (!is_string($line)) {
        $line = (string)  $line;
    }
    if (!is_int($maxSize)) {
        $maxSize = (integer) $maxSize;
    }

    $logFile = $GLOBALS['VA_SYSPATHS']['data'] . '/' . $logFilename;

    // If log is larger than $maxSize then reset it.
    if (($maxSize != 0) &&
        (file_exists($logFile)) &&
        (filesize($logFile) >= $maxSize)) {
        $oldFile = current(explode('.', basename($logFilename)));
        // End the old file in .old
        $oldFile = $GLOBALS['VA_SYSPATHS']['data'] . '/' . $oldFile . '.OLD';

        // If a previous old file exists, delete it.
        if (file_exists($oldFile)) {
            unlink($oldFile);
        }

        // Copy the current log.
        copy($logFile, $oldFile);

        // Delete the current log.
        unlink($logFile);
    }

    if (!file_exists($logFile)) {
        if ($fp = @fopen($logFile, 'w')) {
            flock($fp, LOCK_EX);
            fwrite($fp, "\r\n");
            fwrite($fp, "VADV-PHP Event Log\r\n");
            fwrite($fp, "-------------------------------------------------------------------------------\r\n");
            fwrite($fp, "Date           Time     IP Address         Description\r\n");
            fwrite($fp, "-------------------------------------------------------------------------------\r\n");
            flock($fp, LOCK_UN);
            fclose($fp);
        }
    }

    if (!($fp = @fopen($logFile, 'a'))) {
        return false;
    }

    // Lock the file.
    flock($fp, LOCK_EX);

    $ip = '[' . trim($_SERVER['REMOTE_ADDR']) . ']';
    $ip = str_pad($ip, 17, ' ');

    if (IsHttps()) {
        $line = $line . ' (SSL)';
    }

    $date = date('D') . ' ' . date('Y') . '-' . date('m') . '-' . date('d');
    $time = date('H') . ':' . date('i') . ':' . date('s');
    $line = $date . ' ' . $time . ' ' . $ip . '  ' . $line . "\r\n";

    // Write the line to the file.
    $status = fwrite($fp, $line);

    // Unlock the file.
    flock($fp, LOCK_UN);

    // Close the file.
    fclose($fp);

    // Return the result.
    return $status;
}

?>