<?php
/*
 * ----------------------------------------------------------------------------
 * Va-LoginAuth.inc.php                                              09/02/2011
 * This script will check the user's session data and determine if he is logged
 * in already. If not, then the user is given guest values.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2011 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');

$classStatVBBSDat = new datStatVBBS;
$classTodaysCallers = new TodaysCallers;
$classUserfileDat = new datUserfile;
$classVADVPHPDat = new datVADVPHP;

$_SESSION['auth'] = FALSE;
$_SESSION['error'] = FALSE;

// Default Guest Account Settings
// If the guest account is set, then make that the guest settings. It cannot be user 1!
if ((($GLOBALS['CONFIG']['guestacct'] != '') ||
     ($GLOBALS['CONFIG']['guestacct'] > 0)) &&
    ($GLOBALS['CONFIG']['guestacct'] != 1)) {
    $GLOBALS['USER'] = $classUserfileDat->Get($GLOBALS['CONFIG']['guestacct']);
} else {
    $GLOBALS['USER'] = $classUserfileDat->Get(0);
}

// If the session exists, then get the user's information.
if (isset($_SESSION['userno'])) {
    // Decode user number.
    $userno = (integer) base64_decode($_SESSION['userno']);

    // Decode user name.
    $username = (string) base64_decode($_SESSION['username']);

    // Decode password.
    $password = (string) base64_decode($_SESSION['password']);

    if ($userno > 0) {
        // Check for banned accounts.
        if ($GLOBALS['CONFIG']['banned'] != '') {
            $banned = explode(',', $GLOBALS['CONFIG']['banned']);
            if (count($banned) > 0) {
                for ($i = 0; $i < count($banned); $i++) {
                    if ($userno == trim($banned[$i])) {
                        $_SESSION['error'] = TRUE;
                        break;
                    }
                }
            }
        }

        // If no error so far...
        if (!$_SESSION['error']) {
            // Get user information.
            $userchk = $classUserfileDat->Get($userno);

            // Get user setttings.
            $userdat = $classVADVPHPDat->Get($userno);
            if ($userdat['password'] == '') {
                $userpw = $userchk['password'];
            } else {
                $userpw = $userdat['password'];
            }

            // Compare user passwords - they must be IDENTICAL.
            // Security level must be greater than zero.
            if (($password === $userpw) &&
                ($userchk['securitylevel'] > 0)) {
                $GLOBALS['USER'] = $userchk;
                $_SESSION['auth'] = TRUE;
            } else {
                $_SESSION['error'] = TRUE;
            }
        }
    } else {
        $_SESSION['error'] = TRUE;
    }
}

// Log the attempted login.
if ((isset($_SESSION['login'])) &&
    ($_SESSION['login'])) {
    // Successful...
    if (!$_SESSION['error']) {
        $fileString = $classStatVBBSDat->Read();
        $stats = $classStatVBBSDat->Get($fileString);
        if ($GLOBALS['USER']['securitylevel'] < 255) {
            if (!$stats['webcallstotal']) {
                $stats['webcallstotal'] = 0;
            }
            // Increase the web calls.
            $stats['webcallstotal']++;
            // Increase today's calls.
            $stats['callstoday']++;
            $fileString = $classStatVBBSDat->Put($fileString, $stats);
            $classStatVBBSDat->Write($fileString);
            if ((isset($_SESSION['mobile'])) &&
                ($_SESSION['mobile'] == TRUE)) {
                // Update the today's callers file for mobile users.
                $classTodaysCallers->Write('m');
            } else {
                // Update the today's callers file for normal users.
                $classTodaysCallers->Write();
            }
        }

        // Set last on to current date.
        $GLOBALS['USER']['laston'] = date("ymd");

        // Increase the user's total calls.
        $GLOBALS['USER']['totalcalls']++;

        // Set to current date/ time in UNIX time.
        $GLOBALS['USER']['lastcalltime'] = lTime();

        // Read the userfile.
        $fileString = $classUserfileDat->Read();
        $fileString = $classUserfileDat->Put($fileString, $GLOBALS['USER']['userno'], $GLOBALS['USER']);
        // Write the userfile.
        $classUserfileDat->Write($fileString);

        if ((isset($_SESSION['mobile'])) &&
            ($_SESSION['mobile'] == TRUE)) {
            $log = 'Mobile Login    "' . $username . '" - SUCCESSFUL! (' . $stats['webcallstotal'] . ')';
        } else {
            $log = 'Attempted Login "' . $username . '" - SUCCESSFUL! (' . $stats['webcallstotal'] . ')';
        }

        // Log the login attempt.
        WriteLog('VADV-PHP.LOG', $log, 102400);
        $_SESSION['login'] = FALSE;
    } else {
        // Failed...
        if ((isset($_SESSION['mobile'])) &&
            ($_SESSION['mobile'] == TRUE)) {
            $log = 'Mobile Login    "' . $username . '" - FAILED!';
        } else {
            $log = 'Attempted Login "' . $username . '" - FAILED!';
        }

        // Log the login attempt.
        WriteLog('VADV-PHP.LOG', $log, 102400);
        $_SESSION['login'] = FALSE;
    }
}

if (($GLOBALS['USER']['userno'] > 0) &&
    ($GLOBALS['USER']['laston'] != date("ymd"))) {
    $GLOBALS['USER']['laston'] = date("ymd");
    $GLOBALS['USER']['timecounter'] = 0;
    $GLOBALS['USER']['lastcalltime'] = lTime();
    $fileString = $classUserfileDat->Read();
    $fileString = $classUserfileDat->Put($fileString, $GLOBALS['USER']['userno'], $GLOBALS['USER']);
    $classUserfileDat->Write($fileString);
}
?>