<?php
/*
 * ----------------------------------------------------------------------------
 * Va-Misc.inc.php                                                   12/30/2008
 * This script contains miscellaneous functions.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2008 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');


class NetworkCallers {
    /**
    * Va-Misc.inc.php :: Get()
    *
    * This function will read the netcalls.lst file and break it down and store its
    * information into an array.
    *
    * @return array
    */
    function Get() {
        // Set the default variable.
        $netcalls[0] = 0;

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['data'] . '/NETCALLS.LST';

        if (!file_exists($file)) {
            return $netcalls;
        }

        // Read the file into an array.
        $file = file($file);

        for ($i = 0; $i < count($file); $i++) {
            $netcalls[$i + 1]['callno'] = trim(substr($file[$i], 2, 7));
            $netcalls[$i + 1]['nodeno'] = trim(substr($file[$i], 16, 8));
            $netcalls[$i + 1]['network'] = trim(substr($file[$i], 28, 30));
            $netcalls[$i + 1]['channel'] = trim(substr($file[$i], 60, 5));
            $netcalls[$i + 1]['time'] = trim(substr($file[$i], 68, 5));
            $netcalls[$i + 1]['baudrate'] = trim(StripBBSCodes(substr($file[$i], 77, 8)));
            if ($netcalls[$i + 1]['baudrate'] == '0') {
                $netcalls[$i + 1]['baudrate'] = 'Local';
            }
            if ($netcalls[$i + 1]['baudrate'] == '11520') {
                $netcalls[$i + 1]['baudrate'] = '115200';
            }
        }

        // Put the number of callers in position 0.
        $netcalls[0] = $i;

        // Return the network callers.
        return $netcalls;
    }
}


class TodaysCallers {
    /**
    * Va-Misc.inc.php :: Get()
    *
    * This function will read the callers.lst file and break it down and store its
    * information into an array.
    *
    * @return array
    */
    function Get() {
        // Set the default variable.
        $callers[0] = 0;

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['data'] . '/CALLERS.LST';

        if (!file_exists($file)) {
            return $callers;
        }

        // Read the file into an array.
        $file = file($file);

        for ($i = 0; $i < count($file); $i++) {
            $callers[$i + 1]['callno'] = trim(substr($file[$i], 2, 7));
            $callers[$i + 1]['userno'] = trim(substr($file[$i], 16, 8));
            $callers[$i + 1]['handle'] = trim(substr($file[$i], 28, 30));
            $callers[$i + 1]['channel'] = trim(substr($file[$i], 60, 5));
            $callers[$i + 1]['time'] = trim(substr($file[$i], 68, 5));
            $callers[$i + 1]['baudrate'] = trim(StripBBSCodes(substr($file[$i], 77)));
            if ($callers[$i + 1]['baudrate'] == '0') {
                $callers[$i + 1]['baudrate'] = 'Local';
            }
            if ($callers[$i + 1]['baudrate'] == '11520') {
                $callers[$i + 1]['baudrate'] = '115200';
            }
        }

        // Put the number of callers in position 0.
        $callers[0] = $i;

        // Return today's callers.
        return $callers;
    }


    /**
    * Va-Misc.inc.php :: Write()
    *
    * This function will write a new entry to the callers.lst file using the current
    * user's information. This function is used only for adding web users entries,
    * and NOT to add a BBS user's entry (which is done by the BBS anyway).
    *
    * The id can be set to a one character identifier to seperate different login
    * methods.
    *
    * @return mixed
    */
    function Write($id = '') {
        if (!is_string($id)) {
            $id = (string) $id;
        }

        $id = trim($id);
        if (strlen($id) > 1) {
            $id = substr($id, 0, 1);
        }

        $classStatVBBSDat = new datStatVBBS;

        $stats = $classStatVBBSDat->Get();
        $callerNumber = str_pad($stats['webcallstotal'], 7, ' ', STR_PAD_LEFT);
        $userNo = str_pad($GLOBALS['USER']['userno'], 9);
        $handle = str_pad($GLOBALS['USER']['handle'], 30);
        $channel = str_pad('Web', 6);
        $time = date('H') . ':' . date('i') . ' ';
        $baud = str_pad('VADV-PHP' . $id, 9);

        // Create the line.
        $line = chr(3) . '2'  . $callerNumber
              . chr(3) . '5: '
              . chr(3) . '7#' . $userNo
              . chr(3) . '5 ' . $handle
              . chr(3) . '1'  . $channel
              . chr(3) . '6'  . $time
              . chr(3) . '5 ' . $baud
              . chr(3) . '0'
              . "\r\n";

        $file = $GLOBALS['VA_SYSPATHS']['data'] . '/CALLERS.LST';

        // Open the file.
        $fp = fopen($file, 'a');

        // Lock the file.
        flock($fp, 2);

        // Write the new line.
        $status = fwrite($fp, $line);

        // Unlock the file.
        flock($fp, 3);

        // Close the file.
        fclose($fp);

        // Return the write status.
        return $status;
    }
}
?>