<?php
/*
 * ----------------------------------------------------------------------------
 * Va-MultiFBX.inc.php                                               12/31/2008
 * This script will retrieve all of the informaton related to the configured
 * multi-feedback settings that are found in MULTIFBX.CFG.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2008 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');


class cfgMultiFBX {
    /**
    * Va-MultiFBX.inc.php :: Get()
    *
    * Retrieve all of the multi-feedback information and store it into a
    * multidimensional array.
    *
    * Array format: $multifbx[1]['setting']
    *               $multifbx[0] = Total number of multi-feeback settings.
    *
    * @return array
    */
    function Get() {
        $fileString = $this->Read();
        if ($fileString === FALSE) {
            trigger_error(VA_Error(3));
        }

        // Create the array.
        $multifbx = array();
        $offset = 0;

        for ($i = 1; $offset < strlen($fileString); $i++) {
            $multifbx[$i]['desc'] = GetString($fileString, 1 + $offset, 40);
            $multifbx[$i]['handle'] = GetString($fileString, 41 + $offset, 40);
            $multifbx[$i]['userno'] = GetLongInt($fileString, 81 + $offset);
            $multifbx[$i]['netaddress'] = GetString($fileString, 85 + $offset, 80);
            $multifbx[$i]['netno'] = GetInteger($fileString, 165 + $offset);
            $offset = 166 * $i;
        }

        // Put the number of configured users in position 0.
        $multifbx[0] = $i - 1;

        // Return the multi-feedback settings.
        return $multifbx;
    }


    /**
    * Va-MultiFBX.inc.php :: Put()
    *
    * This function will change the multifbx.cfg to match the configuration variable
    * passed. It then returns the new file string. The configuration variable MUST
    * be in the format of $variable[1]['setting'].
    *
    * @param string $fileString
    * @param array $configVariable
    * @return string
    */
    function Put($fileString,
                 $configVariable) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }
        if (!is_array($configVariable)) {
            trigger_error(VA_Error(1));
        }

        // Get the number of elements in the config variable.
        $length = count($configVariable);
        $pointer = 0;
        $offset = 0;

        if (isset($configVariable[0])) {
            $length--;
        }

        for ($i = 1; $pointer < $length; $i++) {
            $total = intval(strlen($fileString) / 166);
            if ($i > $total) {
                $fileString = $fileString . str_pad('', 166, chr(0));
            }

            if (isset($configVariable[$i])) {
                $pointer++;
                if (isset($configVariable[$i]['desc'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['desc'], 1 + $offset, 40);
                }
                if (isset($configVariable[$i]['handle'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['handle'], 41 + $offset, 40);
                }
                if (isset($configVariable[$i]['userno'])) {
                    $fileString = PutLongInt($fileString, $configVariable[$i]['userno'], 81 + $offset);
                }
                if (isset($configVariable[$i]['netaddress'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['netaddress'], 85 + $offset, 80);
                }
                if (isset($configVariable[$i]['netno'])) {
                    $fileString = PutInteger($fileString, $configVariable[$i]['netno'], 165 + $offset);
                }
            }
            $offset = 166 * $i;
        }

        // Return the new file string.
        return $fileString;
    }


    /**
    * Va-MultiFBX.inc.php :: Read()
    *
    * This function will read the multifbx.cfg file.
    *
    * @return string
    */
    function Read() {
        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/MULTIFBX.CFG';

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Return the file string.
        return $fileString;
    }


    /**
    * Va-MultiFBX.inc.php :: Write()
    *
    * This function will write a new multifbx.cfg file based on the file string
    * passed to it. If the file string is not in the correct format, then your
    * multifbx.cfg file will become corrupt and the BBS could stop functioning!
    *
    * @param string $fileString
    * @return mixed
    */
    function Write($fileString) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/MULTIFBX.CFG';
        $status = WriteCfgFile($file, $fileString);

        // Return the result status.
        return $status;
    }
}
?>