<?php
/*
 * ----------------------------------------------------------------------------
 * Va-Network.inc.php                                                01/06/2022
 * This script will retrieve various information about configured networks.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2022 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');


class vadvNetworks {
    /**
    * Va-Networks.inc.php :: GetNetworkInfo()
    *
    * This function will return information about all networks that have been
    * configured. Networks that have not been configured are ignored.
    *
    * Variable returned is in the format of: $var[number][settings][network config]
    *
    * @return array
    */
    function GetNetworkInfo() {
        $datFile = $GLOBALS['VA_SYSPATHS']['data'] . '/NETWORKS.LST';
        $netFile = $GLOBALS['VA_SYSPATHS']['data'] . '/NET.';

        // Set default Vnet values.
        //$netInfo[0] = 1;
        $netInfo[1]['netno'] = 1;
        $netInfo[1]['nettype'] = 1;
        $netInfo[1]['desc'] = 'VirtualNet';
        $netInfo[1]['accflag'] = '';
        $netInfo[1]['settings']['systemid'] = 0;

        if (!file_exists($datFile)) {
            return $netInfo;
        }

        // Set the counter to one (vnet).
        $cnt = 1;

        // Read the file into an array.
        $fileArray = file($datFile);

        for ($i = 0; $i < count($fileArray); $i++) {
            // Read a line.
            $line = trim($fileArray[$i]);

            // Determine if the line is commented out.
            $comment = trim(substr($line, 0, 1));

            if (($line != '') &&
                ($comment != ';')) {
                // Determine the position of the comma.
                $pos = strpos($line, ',');
                if ($pos !== FALSE) {
                    // Explode the line into parts.
                    $netVar = explode(',', $line);

                    // Determine the network number.
                    $num = (integer) $netVar[0];

                    // Create the net filename.
                    $file = $netFile . $num;

                    // If the file exists, the network is configured.
                    if ((file_exists($file)) &&
                        ($num > 0)) {
                        $netInfo[$num]['netno'] = (integer) $netVar[0];
                        $netInfo[$num]['nettype'] = (integer) $netVar[1];
                        $netInfo[$num]['desc'] = trim($netVar[2]);
                        if (!isset($netVar[3])) {
                            $netInfo[$num]['accflag'] = '';
                        } else {
                            if (trim($netVar[3]) == '-') {
                                $netInfo[$num]['accflag'] = '';
                            } else {
                                $netInfo[$num]['accflag'] = trim($netVar[3]);
                            }
                        }
                        // Additional FIDOnet information.
                        if (isset($netVar[4])) {
                            $netInfo[$num]['zone'] = (integer) $netVar[4];
                        }
                        if (isset($netVar[5])) {
                            $netInfo[$num]['nodelist'] = trim($netVar[5]);
                        }
                        if (isset($netVar[6])) {
                            $netInfo[$num]['nodediff'] = trim($netVar[6]);
                        }
                        if (isset($netVar[7])) {
                            $netInfo[$num]['realname'] = (integer) $netVar[7];
                        }

                        $netSettings = ReadConfig($file);
                        $netInfo[$num]['settings'] = $netSettings;

                        if ($netVar[0] != 1) {
                            // Increase the counter.
                            $cnt++;
                        }
                    }
                }
            }
        }

        // Set the total number of configured networks.
        $netInfo[0] = $cnt;

        // Return the network info.
        return $netInfo;
    }


    /**
    * Va-Networks.inc.php :: GetVnetInfo()
    *
    * This function will return information about the VirtualNet network, which
    * is used more often than any other type of network.
    * Variable returned is in the format of: $var[network config]
    *
    * @return string
    */
    function GetVnetInfo() {
        $file = $GLOBALS['VA_SYSPATHS']['data'] . '/NET.1';

        // If the file is missing, return the default values.
        if (!file_exists($file)) {
            $vnet['systemid'] = 0;
            return $vnet;
        }

        // Read the file and return the info.
        return ReadConfig($file);
    }


    /**
    * va-networks.inc.php :: ReadNetworksLst()
    *
    * This function reads the information in the networks.lst file and places it
    * into an array.
    *
    * The array is in the format $var[number][settings]
    *
    * @return array
    */
    function ReadNetworksLst() {
        $datFile = $GLOBALS['VA_SYSPATHS']['data'] . '/NETWORKS.LST';

        if (!file_exists($datFile)) {
            return $netInfo;
        }

        // Set the counter to zero.
        $cnt = 0;

        // Read the file into an array.
        $fileArray = file($datFile);

        for ($i = 0; $i < count($fileArray); $i++) {
            // Read a line.
            $line = trim($fileArray[$i]);

            // Determine if the line is commented out.
            $comment = trim(substr($line, 0, 1));

            if (($line != '') &&
                ($comment != ';')) {
                // Determine the position of the comma.
                $pos = strpos($line, ',');
                if ($pos !== FALSE) {
                    // Explode the line into parts.
                    $netVar = explode(',', $line);

                    // Determine the network number.
                    $num = (integer) $netVar[0];

                    $netInfo[$num]['netno'] = (integer) $netVar[0];
                    $netInfo[$num]['nettype'] = (integer) $netVar[1];
                    $netInfo[$num]['desc'] = trim($netVar[2]);
                    if (!isset($netVar[3])) {
                        $netInfo[$num]['accflag'] = '';
                    } else {
                        if (trim($netVar[3]) == '-') {
                            $netInfo[$num]['accflag'] = '';
                        } else {
                            $netInfo[$num]['accflag'] = trim($netVar[3]);
                        }
                    }
                    // Additional FIDOnet information.
                    if (isset($netVar[4])) {
                        $netInfo[$num]['zone'] = (integer) $netVar[4];
                    }
                    if (isset($netVar[5])) {
                        $netInfo[$num]['nodelist'] = trim($netVar[5]);
                    }
                    if (isset($netVar[6])) {
                        $netInfo[$num]['nodediff'] = trim($netVar[6]);
                    }
                    if (isset($netVar[7])) {
                        $netInfo[$num]['realname'] = (integer) $netVar[7];
                    }

                    // Increase the counter.
                    $cnt++;
                 }
            }
        }

        // Set the total number of configured networks.
        $netInfo[0] = $cnt;

        // Return the network info.
        return $netInfo;
    }
}
?>