<?php
/*
 * ----------------------------------------------------------------------------
 * Va-QuickVal.inc.php                                               12/31/2008
 * This script will retrieve all of the informaton related to the configured
 * quick validation settings that are found in QUICKVAL.CFG.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2008 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');


class cfgQuickVal {
    /**
    * Va-QuickVal.inc.php :: Get()
    *
    * Retrieve all of the quick validation information and store it into a
    * multidimensional array.
    *
    * Array format: $quickval[1]['setting']
    *               $quickval[0] = Total number of configured quickvals.
    *
    * @return array
    */
    function Get() {
        $fileString = $this->Get();
        if ($fileString === FALSE) {
            trigger_error(VA_Error(3));
        }

        // Create the array.
        $quickval = array();
        $offset = 0;

        for ($i = 1; $offset < strlen($fileString); $i++) {
            $quickval[$i]['seclevel'] = GetInteger($fileString, 1 + $offset);
            $quickval[$i]['timelimit'] = GetInteger($fileString, 3 + $offset);
            $quickval[$i]['flags'] = GetLongInt($fileString, 5 + $offset);
            $offset = 73 * $i;
        }

        // Put the number of configured quickvals in position 0.
        $quickval[0] = $i - 1;

        // Return the quickval information.
        return $quickval;
    }


    /**
    * Va-QuickVal.inc.php :: Put()
    *
    * This function will change the quickval.cfg to match the configuration variable
    * passed. It then returns the new file string. The configuration variable MUST
    * be in the format of $variable[1]['setting'].
    *
    * @param string $fileString
    * @param array $configVariable
    * @return string
    */
    function Put($fileString,
                 $configVariable) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }
        if (!is_array($configVariable)) {
            trigger_error(VA_Error(1));
        }

        // Get the number of elements in the config variable.
        $length = count($configVariable);
        $pointer = 0;
        $offset = 0;

        if (isset($configVariable[0])) {
            $length--;
        }

        for ($i = 1; $pointer < $length; $i++) {
            $total = intval(strlen($fileString) / 73);
            if ($i > $total) {
                $fileString = $fileString . str_pad('', 73, chr(0));
            }

            if (isset($configVariable[$i])) {
                $pointer++;
                if (isset($configVariable[$i]['seclevel'])) {
                    $fileString = PutInteger($fileString, $configVariable[$i]['seclevel'], 1 + $offset);
                }
                if (isset($configVariable[$i]['timelimit'])) {
                    $fileString = PutInteger($fileString, $configVariable[$i]['timelimit'], 3 + $offset);
                }
                if (isset($configVariable[$i]['flags'])) {
                    $fileString = PutLongInt($fileString, $configVariable[$i]['flags'], 5 + $offset);
                }
            }
            $offset = 73 * $i;
        }

        // Return the new file string.
        return $fileString;
    }


    /**
    * Va-QuickVal.inc.php :: Read()
    *
    * This function will read the quickval.cfg file.
    *
    * @return string
    */
    function Read() {
        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/QUICKVAL.CFG';

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Return the file string.
        return $fileString;
    }


    /**
    * Va-QuickVal.inc.php :: Write()
    *
    * This function will write a new quickval.cfg file based on the file string
    * passed to it. If the file string is not in the correct format, then your
    * quickval.cfg file will become corrupt and the BBS could stop functioning!
    *
    * @param string $fileString
    * @return mixed
    */
    function Write($fileString) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/QUICKVAL.CFG';
        $status = WriteCfgFile($file, $fileString);

        // Return the result status.
        return $status;
    }
}
?>