<?php
/*
 * ----------------------------------------------------------------------------
 * Va-Random.inc.php                                                 12/30/2008
 * This script will retrieve all of the random quotes from the BBS's
 * RANDOM.CFG file and store them into a simple array.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2008 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');


class cfgRandom {
    /**
    * Va-Random.inc.php :: Get()
    *
    * This function will put all of the VADV random quotes into a simple array.
    *
    * @return array
    */
    function Get() {
        $fileString = $this->Read();
        if ($fileString === FALSE) {
            trigger_error(VA_Error(3));
        }

        // Create the array.
        $random = array();
        $offset = 0;

        for ($i = 1; $offset < strlen($fileString); $i++) {
            $random[$i]['line1'] = GetString($fileString, 1 + $offset, 70);
            $random[$i]['line2'] = GetString($fileString, 71 + $offset, 70);
            $random[$i]['line3'] = GetString($fileString, 141 + $offset, 70);
            $random[$i] = trim($random[$i]['line1']) . ' ' . trim($random[$i]['line2']) . ' ' . trim($random[$i]['line3']);
            $offset = 210 * $i;
        }

        // Put the number of quotes in position 0.
        $random[0] = $i - 1;

        // Return the random quotes.
        return $random;
    }


    /**
    * Va-Random.inc.php :: Put()
    *
    * This function will change the random.cfg to match the configuration variable
    * passed. It then returns the new file string. The configuration variable MUST
    * be in the format of $variable[1].
    *
    * @param string $fileString
    * @param array $configVariable
    * @return string
    */
    function Put($fileString,
                 $configVariable) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }
        if (!is_array($configVariable)) {
            trigger_error(VA_Error(1));
        }

        // Get the number of elements in the config variable.
        $length = count($configVariable);
        $pointer = 0;
        $offset = 0;

        if (isset($configVariable[0])) {
            $length--;
        }

        for ($i = 1; $pointer < $length; $i++) {
            $total = intval(strlen($fileString) / 210);
            if ($i > $total) {
                $fileString = $fileString . str_pad('', 210, chr(0));
            }

            if (isset($configVariable[$i])) {
                $pointer++;
                $fileString = PutString($fileString, $configVariable[$i], 1 + $offset, 210);
            }
            $offset = 210 * $i;
        }

        // Return the new file string.
        return $fileString;
    }


    /**
    * Va-Random.inc.php :: Read()
    *
    * This function will read the random.cfg file.
    *
    * @return string
    */
    function Read() {
        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/RANDOM.CFG';

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Return the file string.
        return $fileString;
    }


    /**
    * Va-Download.inc.php :: Write()
    *
    * This function will write a new random.cfg file based on the file string
    * passed to it. If the file string is not in the correct format, then your
    * random.cfg file will become corrupt and the BBS could stop functioning!
    *
    * @param string $fileString
    * @return mixed
    */
    function Write($fileString) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/RANDOM.CFG';
        $status = WriteCfgFile($file, $fileString);

        // Return the result status.
        return $status;
    }
}
?>