<?php
/*
 * ----------------------------------------------------------------------------
 * Va-Syspaths.inc.php                                               03/25/2011
 * This script will read the syspaths.cfg file and place its information into
 * an array for global use.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2009 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');


class cfgSysPaths {
    function __CONSTRUCT() {
        if ((!isset($GLOBALS['VA_SYSPATHS'])) ||
            (count($GLOBALS['VA_SYSPATHS']) == 0)) {
            $GLOBALS['VA_SYSPATHS'] = $this->Get();
        }
    }


    /**
    * Va-Syspaths.inc.php :: Get()
    *
    * This function will retrieve the vadv system paths from the syspaths.cfg file
    * and return the paths in an array.
    *
    * @return array
    */
    function Get() {
        // Read the file.
        $fileString = $this->Read();

        $syspaths = array();
        $syspaths['main'] = GetString($fileString, 1, 40);
        if ($GLOBALS['CONFIG']['hostname'] != '') {
            $syspaths['main'] = ConvertDir($GLOBALS['CONFIG']['hostname'], $syspaths['main']);
        }
        $syspaths['script'] = GetString($fileString, 41, 40);
        if ($GLOBALS['CONFIG']['hostname'] != '') {
            $syspaths['script'] = ConvertDir($GLOBALS['CONFIG']['hostname'], $syspaths['script']);
        }
        $syspaths['data'] = GetString($fileString, 81, 40);
        if ($GLOBALS['CONFIG']['hostname'] != '') {
            $syspaths['data'] = ConvertDir($GLOBALS['CONFIG']['hostname'], $syspaths['data']);
        }
        $syspaths['network'] = GetString($fileString, 121, 40);
        if ($GLOBALS['CONFIG']['hostname'] != '') {
            $syspaths['network'] = ConvertDir($GLOBALS['CONFIG']['hostname'], $syspaths['network']);
        }
        $syspaths['text'] = GetString($fileString, 161, 40);
        if ($GLOBALS['CONFIG']['hostname'] != '') {
            $syspaths['text'] = ConvertDir($GLOBALS['CONFIG']['hostname'], $syspaths['text']);
        }
        $syspaths['control'] = GetString($fileString, 201, 40);
        if ($GLOBALS['CONFIG']['hostname'] != '') {
            $syspaths['control'] = ConvertDir($GLOBALS['CONFIG']['hostname'], $syspaths['control']);
        }
        $syspaths['temp'] = GetString($fileString, 241, 40);
        if ($GLOBALS['CONFIG']['hostname'] != '') {
            $syspaths['temp'] = ConvertDir($GLOBALS['CONFIG']['hostname'], $syspaths['temp']);
        }
        $syspaths['db'] = GetString($fileString, 281, 40);
        if ($GLOBALS['CONFIG']['hostname'] != '') {
            $syspaths['db'] = ConvertDir($GLOBALS['CONFIG']['hostname'], $syspaths['db']);
        }
        $syspaths['sysop'] = GetString($fileString, 321, 40);
        if ($GLOBALS['CONFIG']['hostname'] != '') {
            $syspaths['sysop'] = ConvertDir($GLOBALS['CONFIG']['hostname'], $syspaths['sysop']);
        }
        $syspaths['fdx'] = GetString($fileString, 361, 40);
        if ($GLOBALS['CONFIG']['hostname'] != '') {
            $syspaths['fdx'] = ConvertDir($GLOBALS['CONFIG']['hostname'], $syspaths['fdx']);
        }

        // Return the BBS system paths.
        return $syspaths;
    }


    /**
    * Va-SysPaths.inc.php :: Put()
    *
    * This function will write updated information to the control.dat file, which
    * is used to determine the current users online.
    *
    * @param string $fileString
    * @param array $configVariable
    * @return string
    */
    function Put($fileString,
                 $configVariable) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }
        if (!is_array($configVariable)) {
            trigger_error(VA_Error(1));
        }

        if (isset($configVariable['main'])) {
            $fileString = PutString($fileString, $configVariable['main'], 1, 40);
        }
        if (isset($configVariable['script'])) {
            $fileString = PutString($fileString, $configVariable['script'], 41, 40);
        }
        if (isset($configVariable['data'])) {
            $fileString = PutString($fileString, $configVariable['data'], 81, 40);
        }
        if (isset($configVariable['network'])) {
            $fileString = PutString($fileString, $configVariable['network'], 121, 40);
        }
        if (isset($configVariable['text'])) {
            $fileString = PutString($fileString, $configVariable['text'], 161, 40);
        }
        if (isset($configVariable['control'])) {
            $fileString = PutString($fileString, $configVariable['control'], 201, 40);
        }
        if (isset($configVariable['temp'])) {
            $fileString = PutString($fileString, $configVariable['temp'], 241, 40);
        }
        if (isset($configVariable['db'])) {
            $fileString = PutString($fileString, $configVariable['db'], 281, 40);
        }
        if (isset($configVariable['sysop'])) {
            $fileString = PutString($fileString, $configVariable['sysop'], 321, 40);
        }
        if (isset($configVariable['fdx'])) {
            $fileString = PutString($fileString, $configVariable['fdx'], 361, 40);
        }

        return $fileString;
    }


    /**
    * Va-SysPaths.inc.php :: ReadSysPaths()
    *
    * This function will read the syspaths.cfg file.
    *
    * @return string
    */
    function Read() {
        // Check to see if a hostname was configured for a network path.
        if ($GLOBALS['CONFIG']['hostname'] != '') {
            $file = ConvertDir($GLOBALS['CONFIG']['hostname'], $GLOBALS['CONFIG']['bbsdir']) . '/SYSPATHS.CFG';
        } else {
            $file = $GLOBALS['CONFIG']['bbsdir'] . '/SYSPATHS.CFG';
        }

        if (!file_exists($file)) {
            trigger_error(VA_Error(0));
        }

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Return the file string.
        return $fileString;
    }


    /**
    * Va-SysPaths.inc.php :: Write()
    *
    * This function will write a new syspaths.cfg file based on the file string
    * passed to it. If the file string is not in the correct format, then your
    * syspaths.cfg file will become corrupt and the BBS could stop functioning!
    *
    * @param string $fileString
    * @return mixed
    */
    function Write($fileString) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/SYSPATHS.CFG';
        $status = WriteCfgFile($file, $fileString);

        // Return the result status.
        return $status;
    }
}
?>