<?php
/*
 * ----------------------------------------------------------------------------
 * Va-TextSect.inc.php                                               12/31/2008
 * This script will retrieve all of the informaton related to the configured
 * bulletins that are found in TEXTSECT.CFG.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2008 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');


class cfgTextSect {
    /**
    * Va-TextSect.inc.php :: Get()
    *
    * Retrieve all of the bulletin section information and store it into a
    * multidimensional array.
    *
    * Array format: $textsect[1]['setting']
    *               $textsect[1]['bulletin'][1]['setting']
    *               $textsect[0] = Total number of sections.
    *
    * @return array
    */
    function Get() {
        $fileString = $this->Read();
        if ($fileString === FALSE) {
            trigger_error(VA_Error(3));
        }

        // Create the array.
        $textSect = array();
        $offset = 0;

        for ($i = 1; $offset < strlen($fileString); $i++) {
            $textSect[$i]['desc'] = GetString($fileString, 1 + $offset, 32);
            $textSect[$i]['path'] = GetString($fileString, 33 + $offset, 32);
            $textSect[$i]['minsl'] = GetInteger($fileString, 65 + $offset);
            $textSect[$i]['accflag'] = GetString($fileString, 67 + $offset, 1);
            $offset = 67 * $i;

            // Get the bulletins for this section.
            $datFile = $GLOBALS['VA_SYSPATHS']['data'] . '/TEXT' . $i . '.DAT';
            $datFileString = ReadCfgFile($datFile);
            $datOffset = 0;

            for ($j = 1; $datOffset < strlen($datFileString); $j++) {
                $textSect[$i]['bulletin'][$j]['desc'] = GetString($datFileString, 1 + $datOffset, 32);
                $textSect[$i]['bulletin'][$j]['filename'] = GetString($datFileString, 33 + $datOffset, 12);
                $datOffset = 44 * $j;
            }

            // The number of bulletins.
            $textSect[$i]['bulletin'][0] = $j - 1;
        }

        // Put the number of sections in position 0.
        $textSect[0] = $i - 1;

        // Return the text sections.
        return $textSect;
    }


    /**
    * Va-TextSect.inc.php :: Put()
    *
    * This function will change the textsect.cfg to match the configuration variable
    * passed. It then returns the new file string. The configuration variable MUST
    * be in the format of $variable[1]['setting']. Currently this function does not
    * allow you to change the actual bulletin descriptions or filenames.
    *
    * @param string $fileString
    * @param array $configVariable
    * @return string
    */
    function Put($fileString,
                 $configVariable) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }
        if (!is_array($configVariable)) {
            trigger_error(VA_Error(1));
        }

        // Get the number of elements in the config variable.
        $length = count($configVariable) - 1;
        $pointer = 0;
        $offset = 0;

        if (isset($configVariable[0])) {
            $length--;
        }

        for ($i = 1; $pointer < $length; $i++) {
            $total = intval(strlen($fileString) / 67);
            if ($i > $total) {
                $fileString = $fileString . str_pad('', 67, chr(0));
            }

            if (isset($configVariable[$i])) {
                $pointer++;
                if (isset($configVariable[$i]['desc'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['desc'], 1 + $offset, 32);
                }
                if (isset($configVariable[$i]['path'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['path'], 33 + $offset, 32);
                }
                if (isset($configVariable[$i]['minsl'])) {
                    $fileString = PutInteger($fileString, $configVariable[$i]['minsl'], 65 + $offset);
                }
                if (isset($configVariable[$i]['accflag'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['accflag'], 67 + $offset, 1);
                }
            }
            $offset = 67 * $i;
        }

        // Return the new file string.
        return $fileString;
    }


    /**
    * Va-TextSect.inc.php :: Read()
    *
    * This function will read the textsect.cfg file.
    *
    * @return string
    */
    function Read() {
        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/TEXTSECT.CFG';

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Return the file string.
        return $fileString;
    }


    /**
    * Va-TextSect.inc.php :: Write()
    *
    * This function will write a new textsect.cfg file based on the file string
    * passed to it. If the file string is not in the correct format, then your
    * textsect.cfg file will become corrupt and the BBS could stop functioning!
    *
    * @param string $fileString
    * @return mixed
    */
    function Write($fileString) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/TEXTSECT.CFG';
        $status = WriteCfgFile($file, $fileString);

        // Return the result status.
        return $status;
    }
}
?>