<?php
/*
 * ----------------------------------------------------------------------------
 * Va-Upload.inc.php                                                 12/31/2008
 * This script will retrieve all of the informaton related to the configured
 * upload protocols that are found in UPLOAD.CFG.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2008 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');


class cfgUpload {
    /**
    * Va-Upload.inc.php :: Get()
    *
    * Retrieve all of the upload protocol information and store it into a
    * multidimensional array.
    *
    * Array format: $upload[1]['setting']
    *               $upload[0] = Total number of upload protocols.
    *
    * @return array
    */
    function Get() {
        $fileString = $this->Read();
        if ($fileString === FALSE) {
            trigger_error(VA_Error(3));
        }

        // Create the array.
        $upload = array();
        $offset = 0;

        for ($i = 1; $offset < strlen($fileString); $i++) {
            $upload[$i]['hotkey'] = GetString($fileString, 1 + $offset, 1);
            $upload[$i]['desc'] = GetString($fileString, 2 + $offset, 23);
            $upload[$i]['batch'] = GetChar($fileString, 25 + $offset);
            $upload[$i]['command'] = GetString($fileString, 26 + $offset, 48);
            $offset = 73 * $i;
        }

        // Put the number of upload protocols in position 0.
        $upload[0] = $i - 1;

        // Return the upload protocols.
        return $upload;
    }


    /**
    * Va-Upload.inc.php :: Put()
    *
    * This function will change the upload.cfg to match the configuration variable
    * passed. It then returns the new file string. The configuration variable MUST
    * be in the format of $variable[1]['setting'].
    *
    * @param string $fileString
    * @param array $configVariable
    * @return string
    */
    function Put($fileString,
                 $configVariable) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }
        if (!is_array($configVariable)) {
            trigger_error(VA_Error(1));
        }

        // Get the number of elements in the config variable.
        $length = count($configVariable);
        $pointer = 0;
        $offset = 0;

        if (isset($configVariable[0])) {
            $length--;
        }

        for ($i = 1; $pointer < $length; $i++) {
            $total = intval(strlen($fileString) / 73);
            if ($i > $total) {
                $fileString = $fileString . str_pad('', 73, chr(0));
            }

            if (isset($configVariable[$i])) {
                $pointer++;
                if (isset($configVariable[$i]['hotkey'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['hotkey'], 1 + $offset, 1);
                }
                if (isset($configVariable[$i]['desc'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['desc'], 2 + $offset, 23);
                }
                if (isset($configVariable[$i]['batch'])) {
                    $fileString = PutChar($fileString, $configVariable[$i]['batch'], 25 + $offset);
                }
                if (isset($configVariable[$i]['command'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['command'], 26 + $offset, 48);
                }
            }
            $offset = 73 * $i;
        }

        // Return the new file string.
        return $fileString;
    }


    /**
    * Va-Upload.inc.php :: Read()
    *
    * This function will read the upload.cfg file.
    *
    * @return string
    */
    function Read() {
        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/UPLOAD.CFG';

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Return the file string.
        return $fileString;
    }


    /**
    * Va-Upload.inc.php :: Write()
    *
    * This function will write a new upload.cfg file based on the file string
    * passed to it. If the file string is not in the correct format, then your
    * upload.cfg file will become corrupt and the BBS could stop functioning!
    *
    * @param string $fileString
    * @return mixed
    */
    function Write($fileString) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/UPLOAD.CFG';
        $status = WriteCfgFile($file, $fileString);

        // Return the result status.
        return $status;
    }
}
?>