<?php
/*
 * ----------------------------------------------------------------------------
 * Va-Userfile.inc.php                                               06/17/2009
 * This script will read the userfile.dat file for the specified user and store
 * its information into an array.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2009 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');


class datUserfile {
    /**
    * Va-Userfile.inc.php :: Compress()
    *
    * This function will compress the userfile to its smallest state without
    * disrupting user numbers (i.e. removes old accounts from the end).
    *
    * @return boolean
    */
    function Compress() {
        $classUserfileIdx = new idxUserfile;

        // Read the userfile.
        $fileString = $this->Read();

        // Determine the length of the file string.
        $length = strlen($fileString);

        // Get the number of users.
        $usercount = floor($length / 1048);

        // Set the file offset.
        $offset = 1048;

        // Set the starting position (last acct).
        $position = $length - 1048;

        // Pointer
        $found = FALSE;

        for ($i = $usercount; $i > 1; $i--) {
            $seclevel = GetInteger($fileString, 374 + $position);
            if ($seclevel > 0) {
                $found = TRUE;
                break;
            }
            $position = $position - $offset;
        }

        if ($i < $usercount) {
            $length = $i * 1048;
            $fileString = substr($fileString, 0, $length);
            $status = $this->Write($fileString);
        } else {
            $status = TRUE;
        }

        $classUserfileIdx->Create();

        // Return the status of the operation.
        return $status;
    }


    /**
    * Va-Userfile.inc.php :: Get()
    *
    * This function will retrieve the information stored in the userfile.dat file
    * for the specified user number. This file contains all of the user's data.
    *
    * @param integer $userNo
    * @return array
    */
    function Get($userNo,
                 $fileString = '') {
        if (!is_int($userNo)) {
            $userNo = (integer) $userNo;
        }
        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }

        $classVadvUsers = new vadvUsers;

        // Create a default blank user.
        $default['userno'] = $userNo;
        $default['handle'] = 'Guest';
        $default['realname'] = 'Guest User';
        $default['address'] = '';
        $default['city'] = '';
        $default['state'] = '';
        $default['zipcode'] = '';
        $default['phone1'] = '';
        $default['phone2'] = '';
        $default['password'] = '';
        $default['mailfwdflag'] = 0;
        $default['mailfwdhandle'] = '';
        $default['mailfwduserno'] = 0;
        $default['mailfwdnode'] = 0;
        $default['mailfwdnetno'] = 0;
        $default['lngdistcost'] = 0;
        $default['lngdistacc'] = 0;
        $default['sortfilelst'] = 0;
        $default['pagebreak'] = 24;
        $default['bdaymonth'] = 0;
        $default['bdayday'] = 0;
        $default['bdayyear'] = 0;
        $default['age'] = $GLOBALS['CONFIG']['userage'];
        $default['accflags'] = $GLOBALS['CONFIG']['useraccflags'];
        $default['flags'] = '';
        $default['securitylevel'] = $GLOBALS['CONFIG']['usersl'];
        $default['laston'] = date("ymd");
        $default['maxtime'] = 60;
        $default['timecounter'] = 0;
        $default['totalcalls'] = 0;
        $default['totaltime'] = 0;
        $default['videomode'] = 1;
        $default['ratioexempt'] = 0;
        $default['verifybday'] = 0;
        $default['textcolor'] = chr(7);
        $default['basecolor'] = chr(0);
        $default['promptcolor'] = chr(2);
        $default['inputcolor'] = chr(5);
        $default['boxcolor'] = chr(5);
        $default['mnuselcolor'] = chr(1);
        $default['headercolor'] = chr(2);
        $default['extra1'] = '';
        $default['extra2'] = '';
        $default['extra3'] = '';
        $default['extra4'] = '';
        $default['extra5'] = '';
        $default['extra6'] = '';
        $default['extra7'] = '';
        $default['extra8'] = '';
        $default['macro1'] = '';
        $default['macro2'] = '';
        $default['uplfiles'] = 0;
        $default['uplkb'] = 0;
        $default['dlfiles'] = 0;
        $default['dlkb'] = 0;
        $default['totalposts'] = 0;
        $default['totalemail'] = 0;
        $default['postsread'] = 0;
        $default['timebank'] = 0;
        $default['credits'] = 0;
        $default['comment'] = '';
        $default['protocol'] = 'Z';
        $default['testid1'] = '';
        $default['testid2'] = '';
        $default['offsym'] = '';
        $default['expdate'] = '';
        $default['reserved'] = '';
        $default['lastcalltime'] = lTime();
        $default['liststyle'] = 2;
        $default['extra'] = '';

        // If the userno is less than zero or greater than the last account, return the default information.
        if (($userNo <= 0) ||
            ($userNo > $classVadvUsers->Count())) {
            $default['userno'] = 0;
            return $default;
        }

        if ($fileString == '') {
            $fileString = $this->Read();
            if ($fileString === FALSE) {
                trigger_error(VA_Error(3));
            }
        }

        $user = array();
        $offset = 1048 * ($userNo - 1);
        $user['userno'] = $userNo;
        $user['handle'] = GetString($fileString, 1 + $offset, 40);
        $user['realname'] = GetString($fileString, 41 + $offset, 40);
        $user['address'] = GetString($fileString, 81 + $offset, 40);
        $user['city'] = GetString($fileString, 121 + $offset, 32);
        $user['state'] = GetString($fileString, 153 + $offset, 3);
        $user['zipcode'] = GetString($fileString, 156 + $offset, 10);
        $user['phone1'] = GetString($fileString, 166 + $offset, 12);
        $user['phone2'] = GetString($fileString, 178 + $offset, 12);
        // User password is encrypted for security purposes.
        $user['password'] = hash($GLOBALS['CONFIG']['encryption'], strtoupper(GetString($fileString, 190 + $offset, 8)));
        $user['mailfwdflag'] = GetInteger($fileString, 198 + $offset);
        $user['mailfwdhandle'] = GetString($fileString, 200 + $offset, 40);
        $user['mailfwduserno'] = GetLongInt($fileString, 240 + $offset);
        $user['mailfwdnode'] = GetString($fileString, 244 + $offset, 80);
        $user['mailfwdnetno'] = GetInteger($fileString, 324 + $offset);
        $user['lngdistcost'] = GetSingle($fileString, 326 + $offset, 2);
        $user['lngdistacc'] = GetSingle($fileString, 330 + $offset, 2);
        $user['sortfilelst'] = GetInteger($fileString, 334 + $offset);
        $user['pagebreak'] = GetInteger($fileString, 336 + $offset);
        $user['bdaymonth'] = GetInteger($fileString, 338 + $offset);
        $user['bdayday'] = GetInteger($fileString, 340 + $offset);
        $user['bdayyear'] = GetInteger($fileString, 342 + $offset);
        $user['age'] = CalcAge($user['bdaymonth'], $user['bdayday'], $user['bdayyear']);
        $user['flags'] = str_replace(' ', '', GetString($fileString, 344 + $offset, 26));
        $user['accflags'] = str_replace(' ', '', GetAccFlags(GetLongInt($fileString, 370 + $offset)));
        $user['securitylevel'] = GetInteger($fileString, 374 + $offset);
        $user['laston'] = GetString($fileString, 376 + $offset, 6);
        $user['maxtime'] = GetInteger($fileString, 382 + $offset);
        $user['timecounter'] = GetInteger($fileString, 384 + $offset);
        $user['totalcalls'] = GetLongInt($fileString, 386 + $offset);
        $user['totaltime'] = GetLongInt($fileString, 390 + $offset);
        $user['videomode'] = GetInteger($fileString, 394 + $offset);
        $user['ratioexempt'] = GetInteger($fileString, 396 + $offset);
        $user['verifybday'] = GetInteger($fileString, 398 + $offset);
        $user['textcolor'] = GetChar($fileString, 400 + $offset);
        $user['basecolor'] = GetChar($fileString, 401 + $offset);
        $user['promptcolor'] = GetChar($fileString, 402 + $offset, 1);
        $user['inputcolor'] = GetChar($fileString, 403 + $offset, 1);
        $user['boxcolor'] = GetChar($fileString, 404 + $offset, 1);
        $user['mnuselcolor'] = GetChar($fileString, 405 + $offset, 1);
        $user['headercolor'] = GetChar($fileString, 406 + $offset, 1);
        $user['extra1'] = GetString($fileString, 407 + $offset, 12);
        $user['extra2'] = GetString($fileString, 419 + $offset, 12);
        $user['extra3'] = GetString($fileString, 431 + $offset, 12);
        $user['extra4'] = GetString($fileString, 443 + $offset, 12);
        $user['extra5'] = GetString($fileString, 455 + $offset, 12);
        $user['extra6'] = GetString($fileString, 467 + $offset, 12);
        $user['extra7'] = GetString($fileString, 479 + $offset, 12);
        $user['extra8'] = GetString($fileString, 491 + $offset, 12);
        $user['macro1'] = GetString($fileString, 503 + $offset, 160);
        $user['macro2'] = GetString($fileString, 663 + $offset, 160);
        $user['uplfiles'] = GetLongInt($fileString, 823 + $offset);
        $user['uplkb'] = GetLongInt($fileString, 827 + $offset);
        $user['dlfiles'] = GetLongInt($fileString, 831 + $offset);
        $user['dlkb'] = GetLongInt($fileString, 835 + $offset);
        $user['totalposts'] = GetLongInt($fileString, 839 + $offset);
        $user['totalemail'] = GetLongInt($fileString, 843 + $offset);
        $user['postsread'] = GetLongInt($fileString, 847 + $offset);
        $user['timebank'] = GetInteger($fileString, 851 + $offset);
        $user['credits'] = GetLongInt($fileString, 853 + $offset);
        $user['comment'] = GetString($fileString, 857 + $offset, 16);
        $user['protocol'] = GetString($fileString, 873 + $offset, 1);
        $user['testid1'] = GetString($fileString, 874 + $offset, 3);
        $user['testid2'] = GetString($fileString, 877 + $offset, 4);
        $user['offsym'] = GetString($fileString, 881 + $offset, 22);
        $user['expdate'] = GetString($fileString, 903 + $offset, 6);
        $user['reserved'] = GetString($fileString, 909 + $offset, 6);
        $user['lastcalltime'] = GetLongInt($fileString, 915 + $offset);
        $user['liststyle'] = GetInteger($fileString, 919 + $offset);
        $user['extra'] = GetString($fileString, 921 + $offset, 128);

        // If a deleted account, return the default information (unless you are the sysop).
        if (($user['securitylevel'] == 0) &&
            ($GLOBALS['USER']['securitylevel'] < 255)) {
            return $default;
        }

        // Return the user information.
        return $user;
    }


    /**
    * Va-Userfile.inc.php :: Put()
    *
    * This function will write updated information to the specified user's account.
    *
    * @param string $fileString
    * @param integer $userNo
    * @param array $configVariable
    * @return string
    */
    function Put($fileString,
                 $userNo,
                 $configVariable) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }
        if (!is_int($userNo)) {
            $userNo = (integer) $userNo;
        }
        if (!is_array($configVariable)) {
            trigger_error(VA_Error(1));
        }

        $total = intval(strlen($fileString) / 1048);
        if ($userNo > $total) {
            $fileString = $fileString . str_pad('', 1048, chr(0));
        }

        $offset = 1048 * ($userNo - 1);
        if (isset($configVariable['handle'])) {
            $fileString = PutString($fileString, $configVariable['handle'], 1 + $offset, 40);
        }
        if (isset($configVariable['realname'])) {
            $fileString = PutString($fileString, $configVariable['realname'], 41 + $offset, 40);
        }
        if (isset($configVariable['address'])) {
            $fileString = PutString($fileString, $configVariable['address'], 81 + $offset, 40);
        }
        if (isset($configVariable['city'])) {
            $fileString = PutString($fileString, $configVariable['city'], 121 + $offset, 32);
        }
        if (isset($configVariable['state'])) {
            $fileString = PutString($fileString, $configVariable['state'], 153 + $offset, 3);
        }
        if (isset($configVariable['zipcode'])) {
            $fileString = PutString($fileString, $configVariable['zipcode'], 156 + $offset, 10);
        }
        if (isset($configVariable['phone1'])) {
            $fileString = PutString($fileString, $configVariable['phone1'], 166 + $offset, 12);
        }
        if (isset($configVariable['phone2'])) {
            $fileString = PutString($fileString, $configVariable['phone2'], 178 + $offset, 12);
        }
        // To change the passwords, they must be in plain text and less than 8 chars.
        // If it is encrypted with md5, then it has a length of 32 characters and will not write it.
        if ((isset($configVariable['password'])) &&
            (strlen($configVariable['password']) <= 8)) {
            $fileString = PutString($fileString, $configVariable['password'], 190 + $offset, 8);
        }
        if (isset($configVariable['mailfwdflag'])) {
            $fileString = PutInteger($fileString, $configVariable['mailfwdflag'], 198 + $offset);
        }
        if (isset($configVariable['mailfwdhandle'])) {
            $fileString = PutString($fileString, $configVariable['mailfwdhandle'], 200 + $offset, 40);
        }
        if (isset($configVariable['mailfwduserno'])) {
            $fileString = PutLongInt($fileString, $configVariable['mailfwduserno'], 240 + $offset);
        }
        if (isset($configVariable['mailfwdnode'])) {
            $fileString = PutString($fileString, $configVariable['mailfwdnode'], 244 + $offset, 80);
        }
        if (isset($configVariable['mailfwdnetno'])) {
            $fileString = PutInteger($fileString, $configVariable['mailfwdnetno'], 324 + $offset);
        }
        //if (isset($configVariable['lngdistcost'])) {
        //    $fileString = PutSingle($fileString, $configVariable['lngdistcost'], 326 + $offset);
        //}
        //if (isset($configVariable['lngdistacc'])) {
        //    $fileString = PutSingle($fileString, $configVariable['lngdistacc'], 330 + $offset);
        //}
        if (isset($configVariable['sortfilelst'])) {
            $fileString = PutInteger($fileString, $configVariable['sortfilelst'], 334 + $offset);
        }
        if (isset($configVariable['pagebreak'])) {
            $fileString = PutInteger($fileString, $configVariable['pagebreak'], 336 + $offset);
        }
        if (isset($configVariable['bdaymonth'])) {
            $fileString = PutInteger($fileString, $configVariable['bdaymonth'], 338 + $offset);
        }
        if (isset($configVariable['bdayday'])) {
            $fileString = PutInteger($fileString, $configVariable['bdayday'], 340 + $offset);
        }
        if (isset($configVariable['bdayyear'])) {
            $fileString = PutInteger($fileString, $configVariable['bdayyear'], 342 + $offset);
        }
        if (isset($configVariable['flags'])) {
            $fileString = PutString($fileString, MakeFlags($configVariable['flags']), 344 + $offset, 26);
        }
        if (isset($configVariable['accflags'])) {
            $fileString = PutLongInt($fileString, MakeAccFlags($configVariable['accflags']), 370 + $offset);
        }
        if (isset($configVariable['securitylevel'])) {
            $fileString = PutInteger($fileString, $configVariable['securitylevel'], 374 + $offset);
        }
        if (isset($configVariable['laston'])) {
            $fileString = PutString($fileString, $configVariable['laston'], 376 + $offset, 6);
        }
        if (isset($configVariable['maxtime'])) {
            $fileString = PutInteger($fileString, $configVariable['maxtime'], 382 + $offset);
        }
        if (isset($configVariable['timecounter'])) {
            $fileString = PutInteger($fileString, $configVariable['timecounter'], 384 + $offset);
        }
        if (isset($configVariable['totalcalls'])) {
            $fileString = PutLongInt($fileString, $configVariable['totalcalls'], 386 + $offset);
        }
        if (isset($configVariable['totaltime'])) {
            $fileString = PutLongInt($fileString, $configVariable['totaltime'], 390 + $offset);
        }
        if (isset($configVariable['videomode'])) {
            $fileString = PutInteger($fileString, $configVariable['videomode'], 394 + $offset);
        }
        if (isset($configVariable['ratioexempt'])) {
            $fileString = PutInteger($fileString, $configVariable['ratioexempt'], 396 + $offset);
        }
        if (isset($configVariable['verifybday'])) {
            $fileString = PutInteger($fileString, $configVariable['verifybday'], 398 + $offset);
        }
        if (isset($configVariable['textcolor'])) {
            $fileString = PutChar($fileString, $configVariable['textcolor'], 400 + $offset);
        }
        if (isset($configVariable['basecolor'])) {
            $fileString = PutChar($fileString, $configVariable['basecolor'], 401 + $offset);
        }
        if (isset($configVariable['promptcolor'])) {
            $fileString = PutChar($fileString, $configVariable['promptcolor'], 402 + $offset, 1);
        }
        if (isset($configVariable['inputcolor'])) {
            $fileString = PutChar($fileString, $configVariable['inputcolor'], 403 + $offset, 1);
        }
        if (isset($configVariable['boxcolor'])) {
            $fileString = PutChar($fileString, $configVariable['boxcolor'], 404 + $offset, 1);
        }
        if (isset($configVariable['mnuselcolor'])) {
            $fileString = PutChar($fileString, $configVariable['mnuselcolor'], 405 + $offset, 1);
        }
        if (isset($configVariable['headercolor'])) {
            $fileString = PutChar($fileString, $configVariable['headercolor'], 406 + $offset, 1);
        }
        if (isset($configVariable['extra1'])) {
            $fileString = PutString($fileString, $configVariable['extra1'], 407 + $offset, 12);
        }
        if (isset($configVariable['extra2'])) {
            $fileString = PutString($fileString, $configVariable['extra2'], 419 + $offset, 12);
        }
        if (isset($configVariable['extra3'])) {
            $fileString = PutString($fileString, $configVariable['extra3'], 431 + $offset, 12);
        }
        if (isset($configVariable['extra4'])) {
            $fileString = PutString($fileString, $configVariable['extra4'], 443 + $offset, 12);
        }
        if (isset($configVariable['extra5'])) {
            $fileString = PutString($fileString, $configVariable['extra5'], 455 + $offset, 12);
        }
        if (isset($configVariable['extra6'])) {
            $fileString = PutString($fileString, $configVariable['extra6'], 467 + $offset, 12);
        }
        if (isset($configVariable['extra7'])) {
            $fileString = PutString($fileString, $configVariable['extra7'], 479 + $offset, 12);
        }
        if (isset($configVariable['extra8'])) {
            $fileString = PutString($fileString, $configVariable['extra8'], 491 + $offset, 12);
        }
        if (isset($configVariable['macro1'])) {
            $fileString = PutString($fileString, $configVariable['macro1'], 503 + $offset, 160);
        }
        if (isset($configVariable['macro2'])) {
            $fileString = PutString($fileString, $configVariable['macro2'], 663 + $offset, 160);
        }
        if (isset($configVariable['uplfiles'])) {
            $fileString = PutLongInt($fileString, $configVariable['uplfiles'], 823 + $offset);
        }
        if (isset($configVariable['uplkb'])) {
            $fileString = PutLongInt($fileString, $configVariable['uplkb'], 827 + $offset);
        }
        if (isset($configVariable['dlfiles'])) {
            $fileString = PutLongInt($fileString, $configVariable['dlfiles'], 831 + $offset);
        }
        if (isset($configVariable['dlkb'])) {
            $fileString = PutLongInt($fileString, $configVariable['dlkb'], 835 + $offset);
        }
        if (isset($configVariable['totalposts'])) {
            $fileString = PutLongInt($fileString, $configVariable['totalposts'], 839 + $offset);
        }
        if (isset($configVariable['totalemail'])) {
            $fileString = PutLongInt($fileString, $configVariable['totalemail'], 843 + $offset);
        }
        if (isset($configVariable['postsread'])) {
            $fileString = PutLongInt($fileString, $configVariable['postsread'], 847 + $offset);
        }
        if (isset($configVariable['timebank'])) {
            $fileString = PutInteger($fileString, $configVariable['timebank'], 851 + $offset);
        }
        if (isset($configVariable['credits'])) {
            $fileString = PutLongInt($fileString, $configVariable['credits'], 853 + $offset);
        }
        if (isset($configVariable['comment'])) {
            $fileString = PutString($fileString, $configVariable['comment'], 857 + $offset, 16);
        }
        if (isset($configVariable['protocol'])) {
            $fileString = PutString($fileString, $configVariable['protocol'], 873 + $offset, 1);
        }
        if (isset($configVariable['testid1'])) {
            $fileString = PutString($fileString, $configVariable['testid1'], 874 + $offset, 3);
        }
        if (isset($configVariable['testid2'])) {
            $fileString = PutString($fileString, $configVariable['testid2'], 877 + $offset, 4);
        }
        if (isset($configVariable['offsym'])) {
            $fileString = PutString($fileString, $configVariable['offsym'], 881 + $offset, 22);
        }
        if (isset($configVariable['expdate'])) {
            $fileString = PutString($fileString, $configVariable['expdate'], 903 + $offset, 6);
        }
        if (isset($configVariable['reserved'])) {
            $fileString = PutString($fileString, $configVariable['reserved'], 909 + $offset, 6);
        }
        if (isset($configVariable['lastcalltime'])) {
            $fileString = PutLongInt($fileString, $configVariable['lastcalltime'], 915 + $offset);
        }
        if (isset($configVariable['liststyle'])) {
            $fileString = PutInteger($fileString, $configVariable['liststyle'], 919 + $offset);
        }
        if (isset($configVariable['extra'])) {
            $fileString = PutString($fileString, $configVariable['extra'], 921 + $offset, 128);
        }

        // Return the new file string.
        return $fileString;
    }


    /**
    * Va-Userfile.inc.php :: Read()
    *
    * This function will read the userfile.dat file.
    *
    * @return string
    */
    function Read() {
        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['data'] . '/USERFILE.DAT';

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Return the file string.
        return $fileString;
    }


    /**
    * Va-Userfile.inc.php :: Write()
    *
    * This function will write a new userfile.dat file based on the file string
    * passed to it. If the file string is not in the correct format, then your
    * userfile.dat file will become corrupt and the BBS could stop functioning!
    *
    * @param string $fileString
    * @return mixed
    */
    function Write($fileString) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['data'] . '/USERFILE.DAT';
        $status = WriteCfgFile($file, $fileString);

        // Return the result status.
        return $status;
    }
}


class idxUserfile {
    /**
    * Va-Userfile.inc.php :: Create()
    *
    * This function will create a new userfile.idx.
    *
    * @return mixed
    */
    function Create() {
        $classUserfileDat = new datUserfile;
        $classVadvUsers = new vadvUsers;

        $file = $GLOBALS['VA_SYSPATHS']['data'] . '/USERFILE.IDX';

        // If the IDX file exists, then remove it.
        if (file_exists($file)) {
            unlink($file);
        }

        $fileString = '';

        for ($i = 1; $i <= $classVadvUsers->Count(); $i++) {
            $user = $classUserfileDat->Get($i);

            // If deleted user, change handle.
            if ($user['securitylevel'] == 0) {
                $user['handle'] = '!-*<DELETED>*-!';
            }
            $fileString = $this->Put($fileString, $i, $user);
        }

        $status = $this->Write($fileString);

        // Return the result status.
        return $status;
    }


    /**
    * Va-Userfile.inc.php :: Get()
    *
    * This function will get the requested entry information from userfile.idx.
    *
    * @param integer $userNo
    * @return array
    */
    function Get($userNo, $fileString = '') {
        if (!is_int($userNo)) {
            $userNo = (integer) $userNo;
        }

        if ($userNo <= 0) {
            return 0;
        }

        if ($fileString == '') {
            $fileString = $this->Read();
            if ($fileString === FALSE) {
                trigger_error(VA_Error(3));
            }
        }

        $user = array();
        $offset = 82 * ($userNo - 1);
        $user['realname'] = GetString($fileString, 1 + $offset, 40);
        $user['handle'] = GetString($fileString, 41 + $offset, 40);
        $user['userno'] = GetInteger($fileString, 81 + $offset);

        // Return the user information.
        return $user;
    }


    /**
    * Va-Userfile.inc.php :: Put()
    *
    * This function will put new information into the userfile.idx file string.
    *
    * @param string $fileString
    * @param integer $userNo
    * @param array $configVariable
    * @return string
    */
    function Put($fileString,
                 $userNo,
                 $configVariable) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }
        if (!is_int($userNo)) {
            $userNo = (integer) $userNo;
        }

        if (!is_array($configVariable)) {
            trigger_error(VA_Error(1));
        }

        $total = intval(strlen($fileString) / 82);
        if ($userNo > $total) {
            $fileString = $fileString . str_pad('', 82, chr(0));
        }

        $offset = 82 * ($userNo - 1);
        if (isset($configVariable['realname'])) {
            $fileString = PutString($fileString, $configVariable['realname'], 1 + $offset, 40);
        }
        if (isset($configVariable['handle'])) {
            $fileString = PutString($fileString, $configVariable['handle'], 41 + $offset, 40);
        }
        if (isset($configVariable['userno'])) {
            $fileString = PutInteger($fileString, $configVariable['userno'], 81 + $offset);
        }

        // Return the new file string.
        return $fileString;
    }


    /**
    * Va-Userfile.inc.php :: Read()
    *
    * This function reads userfile.idx.
    *
    * @return string
    */
    function Read() {
        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['data'] . '/USERFILE.IDX';

        if (!file_exists($file)) {
            $this->Create();
        }

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Return the file string.
        return $fileString;
    }


    /**
    * Va-Userfile.inc.php :: Write()
    *
    * This function writes the new file string into userfile.idx.
    *
    * @param string $fileString
    * @return mixed
    */
    function Write($fileString) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['data'] . '/USERFILE.IDX';
        $status = WriteCfgFile($file, $fileString);

        // Return the result status.
        return $status;
    }
}

class vadvUsers {
    /**
    * Va-Userfile.inc.php :: AddUserEmailInfo()
    *
    * This function will add a user's email routing info into handles.uip. It will
    * attempt to keep the file in numerical order. Multiple entries that exist for
    * $userNo will be removed. This function is mainly used for creating new user
    * entries and not to modify existing ones.
    *
    * @param integer $userNo
    * @param array $configVariable
    * @return mixed
    */
    function AddUserEmailInfo($userNo,
                              $configVariable) {
        if (!is_int($userNo)) {
            $userNo = (integer) $userNo;
        }
        if (!is_array($configVariable)) {
            trigger_error(VA_Error(1));
        }

        $classNetworks = new vadvNetworks;

        $vnetinfo = $classNetworks->GetVnetInfo();

        $filename = $GLOBALS['VA_SYSPATHS']['main'] . '/HANDLES.UIP';

        if (file_exists($filename)) {
            // Read the file.
            $file = file($filename);
        } else {
            $file[0] = 'system-daemon,SYSTEM-DAEMON,' . $vnetinfo['systemid'] . ',1';
        }

        // Search for an existing entry.
        $found = 0;
        for ($i = 0; $i < count($file); $i++) {
            if (trim($file[$i]) != '') {
                $line = explode(',', $file[$i]);
                if (($line[1] == $userNo) &&
                    (trim($line[1]) != '')) {
                    if ($found == 0) {
                        $file[$i] = $configVariable['handle'] . ',' . $configVariable['userno'] . ',' .
                                    $configVariable['netnode'] . ',' . $configVariable['netno'];
                        $found = 1;
                    } else {
                        for ($j = $i; $j < count($file) - 1; $j++) {
                            // Move all elements down one.
                            $file[$j] = $file[$j + 1];
                        }

                        // Remove the last entry.
                        unset($file[$j]);

                        // Remain at the same entry.
                        $i--;
                    }
                }
            }
        }

        if ($found == 0) {
            $file[count($file)] = $configVariable['handle'] . ',' . $configVariable['userno'] . ',' .
                                  $configVariable['netnode'] . ',' . $configVariable['netno'];
        }

        // Open the file.
        $fp = fopen($filename, 'w');

        // Lock the file.
        flock($fp, 2);

        for ($i = 0; $i < count($file); $i++) {
            // Write to the file.
            $status = fwrite($fp, trim($file[$i]) . "\r\n");
        }

        // Unlock the file.
        flock($fp, 3);

        // Close the file.
        fclose($fp);

        return $status;
    }


    /**
    * Va-Userfile.inc.php :: Count()
    *
    * This function simply returns the number of user accounts in the dat file.
    *
    * @return integer
    */
    function Count() {
        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['data'] . '/USERFILE.DAT';
        if (!file_exists($file)) {
            return 0;
        }

        $userCount = intval(filesize($file) / 1048);

        return $userCount;
    }


    /**
    * Va-Userfile.inc.php :: Delete()
    *
    * This function will take care of all the chores involved with deleting a
    * user from the system.
    *
    * @param integer $userno
    */
    function Delete($userno) {
        if (!is_int($userno)) {
            $userno = (integer) $userno;
        }

        $classDatabases = new vadvDatabases;
        $classUserfileDat = new datUserfile;
        $classUserfileIdx = new idxUserfile;
        $classVoteDat = new datVote;

        // If not a sysop, then do nothing.
        if ($GLOBALS['USER']['securitylevel'] < 255) {
            return;
        }

        // Will not delete user one.
        if ($userno == 1) {
            return;
        }

        // Read the userfile.dat.
        $datString = $classUserfileDat->Read();

        // Read the userfile.idx.
        $idxString = $classUserfileIdx->Read();

        // Get the user information.
        $user = $classUserfileDat->Get($userno, $datString);

        // Set flags to none.
        $user['flags'] = '';

        // Set access flags to none.
        $user['accflags'] = '';

        // Set the sl to zero.
        $user['securitylevel'] = 0;

        // Set the maxtime to one.
        $user['maxtime'] = 1;

        // Get the user information.
        $useridx = $classUserfileIdx->Get($userno, $idxString);

        // Set the handle to deleted status.
        $useridx['handle'] = '!-*<DELETED>*-!';

        // Insert the new information.
        $datString = $classUserfileDat->Put($datString, $userno, $user);
        $idxString = $classUserfileIdx->Put($idxString, $userno, $useridx);

        // Write the new userfile.dat.
        $classUserfileDat->Write($datString);

        // Write the new userfile.idx.
        $classUserfileIdx->Write($idxString);

        // Delete any mail addressed to this account.
        $db = $classDatabases->SetEmailInfo($userno);
        $msgcnt = $db['msgcnt']['msgno'];
        $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $db['filename'] . '.DAT';
        $fileString = ReadCfgFile($file);
        $pos = 0;
        for ($i = 1; (($i <= $db['msgno']) && ($msgcnt > 0)); $i++) {
            $db = $classDatabases->SetEmailInfo($userno);
            $msgInfo = $classDatabases->GetMessageInfo($db, $i, $fileString);
            if ($msgInfo['touserno'] == $userno) {
                $classDatabases->DeleteMessage($db, $i - $pos);
                $pos++;
            }
        }

        // Delete any sent email from this account.
        $db = $classDatabases->SetSentEmailInfo($userno);
        $msgcnt = $db['msgcnt']['msgno'];
        $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $db['filename'] . '.DAT';
        $fileString = ReadCfgFile($file);
        $pos = 0;
        for ($i = 1; (($i <= $db['msgno']) && ($msgcnt > 0)); $i++) {
            $db = $classDatabases->SetSentEmailInfo($userno);
            $msgInfo = $classDatabases->GetMessageInfo($db, $i, $fileString);
            if ($msgInfo['fromuserno'] == $userno) {
                $classDatabases->DeleteMessage($db, $i - $pos);
                $pos++;
            }
        }

        // Delete any oneliners addressed to this account.
        $db = $classDatabases->SetOnelinerInfo($userno);
        $msgcnt = $db['msgcnt']['msgno'];
        $file = $GLOBALS['VA_SYSPATHS']['db'] . '/' . $db['filename'] . '.DAT';
        $fileString = ReadCfgFile($file);
        $pos = 0;
        for ($i = 1; (($i <= $db['msgno']) && ($msgcnt > 0)); $i++) {
            $db = $classDatabases->SetOnelinerInfo($userno);
            $msgInfo = $classDatabases->GetMessageInfo($db, $i, $fileString);
            if ($msgInfo['touserno'] == $userno) {
                $classDatabases->DeleteMessage($db, $i - $pos);
                $pos++;
            }
        }

        // Remove votes from any voting booths.
        $file = $GLOBALS['VA_SYSPATHS']['data'] . '/VOTE';
        for ($i = 1; file_exists($file . $i . '.DAT'); $i++) {
            $classVoteDat->Put($userno, $i, 0);
        }

        // Remove any multi-mail lists.
        $file = $GLOBALS['VA_SYSPATHS']['data'] . '/MM' . $userno . '.';
        for ($i = 1; $i <= 10; $i++) {
            if (file_exists($file . $i)) {
                unlink($file);
            }
        }

        // File Cleanup
        $file = $GLOBALS['VA_SYSPATHS']['data'] . '/AB' . $userno . '.DAT';
        if (file_exists($file)) {
            unlink($file);
        }

        $file = $GLOBALS['VA_SYSPATHS']['data'] . '/IRC' . $userno . '.DAT';
        if (file_exists($file)) {
            unlink($file);
        }
    }


    /**
    * Va-Userfile.inc.php :: FindHandle()
    *
    * This function will search the userfile.dat file for the $handle and return
    * the user number.
    *
    * @param string $handle
    * @return integer
    */
    function FindHandle($handle) {
        if (!is_string($handle)) {
            $handle = (string) $handle;
        }

        $classUserfileIdx = new idxUserfile;

        // Check to see if the variable was passed, if not then return zero.
        if (trim($handle) == '') {
            return 0;
        }

        // Read the userfile.
        $fileString = $classUserfileIdx->Read();

        // Change the handle to lowercase.
        $handle = trim(strtolower($handle));

        // Set the starting position.
        $position = 41;
        $userno = 0;

        for ($i = 1; $position < strlen($fileString); $i++) {
            $string = GetString($fileString, $position, 40);

            // Change the string to lowercase..
            $string = strtolower($string);

            // Compare the handle to the string.
            if (strcmp($handle, $string) == 0) {
                $userno = $i;
                break;
            }

            // Goto the next user record.
            $position = $position + 82;
        }

        // Return the user number.
        return $userno;
    }


    /**
    * Va-Userfile.inc.php :: FindNewAcct()
    *
    * This function will find and return the first blank (deleted) user account
    * number or an account with zero total calls that is over a week old or an
    * account that is over the set idle limit.
    *
    * @return integer
    */
    function FindNewAcct() {
        $classUserfileDat = new datUserfile;

        // Read the userfile.
        $fileString = $classUserfileDat->Read();

        // Determine the length of the file string.
        $length = strlen($fileString);

        // Set the file offset.
        $offset = 1048;

        // Set the starting position.
        $position = 0;

        for ($i = 1; $position < $length; $i++) {
            $seclevel = GetInteger($fileString, 374 + $position);
            $laston = GetString($fileString, 376 + $position, 6);
            $totalcalls = GetInteger($fileString, 386 + $position);

            // Determine age of the account.
            $month = substr($laston, 2, 2);
            $day = substr($laston, 4, 2);
            $year = substr($laston, 0, 2);
            if (($year > date("y")) ||
                ($year > 70)) {
                $year = '19' . $year;
            } else {
                $year = '20' . $year;
            }
            $last = CalcDayNumber($month, $day, $year);
            $today = CalcDayNumber(date('m'), date('d'), date('Y'));
            $age = $today - $last;

            if (($seclevel == 0) ||
               (($totalcalls == 0) &&
                ($age > 7)) ||
               ((($GLOBALS['CONFIG']['purgetime'] > 0) &&
                ($age > $GLOBALS['CONFIG']['purgetime'])) &&
                (($GLOBALS['CONFIG']['purgeminlevel'] > 0) &&
                 ($seclevel >= $GLOBALS['CONFIG']['purgeminlevel']) &&
                 ($seclevel <= $GLOBALS['CONFIG']['purgemaxlevel'])))) {
               break;
           }

            // Goto the next user record.
            $position = $position + $offset;
        }

        if ($position > $length) {
            $i++;
        }

        // Return the user number.
        return $i;
    }


    /**
    * Va-Userfile.inc.php :: FindRealName()
    *
    * This function will search the userfile.dat file for the real $name and return
    * the user number.
    *
    * @param string $name
    * @return integer
    */
    function FindRealName($realname) {
        if (!is_string($realname)) {
            $realname = (string) $realname;
        }

        // Check to see if the variable was passed, if not then return zero.
        if (trim($realname) == '') {
            return 0;
        }

        $classUserfileIdx = new $idxUserfile;

        // Read the userfile.
        $fileString = $classUserfileIdx->Read();

        // Change the name to lowercase.
        $realname = trim(strtolower($realname));

        // Set the starting position.
        $position = 1;
        $userno = 0;

        for ($i = 1; $position < strlen($fileString); $i++) {
            $string = GetString($fileString, $position, 40);

            // Change the string to lowercase.
            $string = strtolower($string);

            // Compare the name to the string.
            if (strcmp($realname, $string) == 0) {
                $userno = $i;
                break;
            }

            // Goto the next user record.
            $position = $position + 82;
        }

        // Return the user number.
        return $userno;
    }


    /**
    * Va-Userfile.inc.php :: FindUser()
    *
    * This function will search the userfile for the user and return
    * the user number.
    *
    * @param string $name
    * @return integer
    */
    function FindUser($username) {
        if (!is_string($username)) {
            $username = (string) $username;
        }

        // Check to see if the variable was passed, if not then return zero.
        if (trim($username) == '') {
            return 0;
        }

        $classUserfileIdx = new idxUserfile;

        // Read the userfile.
        $fileString = $classUserfileIdx->Read();

        // Change the name to lowercase.
        $username = trim(strtolower($username));

        // Set the starting position.
        $position = 1;
        $userno = 0;

        for ($i = 1; $position < strlen($fileString); $i++) {
            $realname = GetString($fileString, $position, 40);
            $handle = GetString($fileString, $position + 40, 40);

            // Change the string to lowercase.
            $realname = strtolower($realname);
            $handle = strtolower($handle);

            // Compare the name to the string.
            if ((strcmp($username, $realname) == 0) ||
                (strcmp($username, $handle) == 0)) {
                $userno = $i;
                break;
            }

            // Goto the next user record.
            $position = $position + 82;
        }

        // Return the user number.
        return $userno;
    }


    /**
    * Va-Userfile.inc.php :: GetUserEmailInfo()
    *
    * This function will search handles.uip and get the user's email routing
    * information.
    *
    * @param integer $userNo
    * @return array
    */
    function GetUserEmailInfo($userNo) {
        if (!is_int($userNo)) {
            $userNo = (integer) $userNo;
        }

        $user['handle'] = '';
        $user['userno'] = 0;
        $user['netnode'] = '';
        $user['netno'] = 0;

        $filename = $GLOBALS['VA_SYSPATHS']['main'] . '/HANDLES.UIP';
        if (!file_exists($file)) {
            return $user;
        }

        // Read the file into an array.
        $file = file($filename);

        $found = 0;
        for ($i = 0; (($found != 1) && ($i < count($file))); $i++) {
            $line = explode(',', $file[$i]);
            if ($line[1] == $userNo) {
                $user['handle'] = $line[0];
                $user['userno'] = $line[1];
                $user['netnode'] = $line[2];
                $user['netno'] = $line[3];
                $found = 1;
            }
        }

        // Return the user info.
        return $user;
    }
}
?>