<?php
/*
 * ----------------------------------------------------------------------------
 * Va-Vote.inc.php                                                   12/31/2008
 * This script will retrieve all of the informaton related to the configured
 * voting informatoin that are found in VOTE.CFG.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2008 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');


class cfgVote {
    function Count() {
        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/VOTE.CFG';
        if (!file_exists($file)) {
            return 0;
        }

        $voteCount = intval(filesize($file) / 546);

        return $voteCount;
    }


    /**
    * Va-Vote.inc.php :: Get()
    *
    * Retrieve all of the voting information and store it into a multi-dimensional
    * array.
    *
    * Array format: $vote[1]['setting']
    *               $vote[0] = Total number of questions.
    *
    * @return array
    */
    function Get() {
        $fileString = $this->Read();
        if ($fileString === FALSE) {
            trigger_error(VA_Error(3));
        }

        // Create the array.
        $vote = array();
        $offset = 0;

        for ($i = 1; $offset < strlen($fileString); $i++) {
            $vote[$i]['question'] = GetString($fileString, 1 + $offset, 64);
            $vote[$i]['respno'] = GetInteger($fileString, 65 + $offset);
            $vote[$i]['resp1'] = GetString($fileString, 67 + $offset, 60);
            $vote[$i]['resp2'] = GetString($fileString, 127 + $offset, 60);
            $vote[$i]['resp3'] = GetString($fileString, 187 + $offset, 60);
            $vote[$i]['resp4'] = GetString($fileString, 247 + $offset, 60);
            $vote[$i]['resp5'] = GetString($fileString, 307 + $offset, 60);
            $vote[$i]['resp6'] = GetString($fileString, 367 + $offset, 60);
            $vote[$i]['resp7'] = GetString($fileString, 427 + $offset, 60);
            $vote[$i]['resp8'] = GetString($fileString, 487 + $offset, 60);
            $offset = 546 * $i;
        }

        // Put the number of voting questions in position 0.
        $vote[0] = $i - 1;

        // Return the voting information.
        return $vote;
    }


    /**
    * Va-Vote.inc.php :: Put()
    *
    * This function will change the vote.cfg to match the configuration variable
    * passed. It then returns the new file string. The configuration variable MUST
    * be in the format of $variable[1]['setting'].
    *
    * @param string $fileString
    * @param array $configVariable
    * @return string
    */
    function Put($fileString,
                 $configVariable) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }
        if (!is_array($configVariable)) {
            trigger_error(VA_Error(1));
        }

        // Get the number of elements in the config variable.
        $length = count($configVariable);
        $pointer = 0;
        $offset = 0;

        if (isset($configVariable[0])) {
            $length--;
        }

        for ($i = 1; $pointer < $length; $i++) {
            $total = intval(strlen($fileString) / 547);
            if ($i > $total) {
                $fileString = $fileString . str_pad('', 547, chr(0));
            }

            if (isset($configVariable[$i])) {
                $pointer++;
                if (isset($configVariable[$i]['question'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['question'], 1 + $offset, 64);
                }
                if (isset($configVariable[$i]['respno'])) {
                    $fileString = PutInteger($fileString, $configVariable[$i]['respno'], 65 + $offset);
                }
                if (isset($configVariable[$i]['resp1'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['resp1'], 67 + $offset, 60);
                }
                if (isset($configVariable[$i]['resp2'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['resp2'], 127 + $offset, 60);
                }
                if (isset($configVariable[$i]['resp3'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['resp3'], 187 + $offset, 60);
                }
                if (isset($configVariable[$i]['resp4'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['resp4'], 247 + $offset, 60);
                }
                if (isset($configVariable[$i]['resp5'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['resp5'], 307 + $offset, 60);
                }
                if (isset($configVariable[$i]['resp6'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['resp6'], 367 + $offset, 60);
                }
                if (isset($configVariable[$i]['resp7'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['resp7'], 427 + $offset, 60);
                }
                if (isset($configVariable[$i]['resp8'])) {
                    $fileString = PutString($fileString, $configVariable[$i]['resp8'], 487 + $offset, 60);
                }
            }
            $offset = 547 * $i;
        }

        // Return the new file string.
        return $fileString;
    }


    /**
    * Va-Vote.inc.php :: Read()
    *
    * This function will read the vote.cfg file.
    *
    * @return string
    */
    function Read() {
        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/VOTE.CFG';

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Return the file string.
        return $fileString;
    }


    /**
    * Va-Vote.inc.php :: Write()
    *
    * This function will write a new vote.cfg file based on the file string
    * passed to it. If the file string is not in the correct format, then your
    * vote.cfg file will become corrupt and the BBS could stop functioning!
    *
    * @param string $fileString
    * @return mixed
    */
    function Write($fileString) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['main'] . '/VOTE.CFG';
        $status = WriteCfgFile($file, $fileString);

        // Return the result status.
        return $status;
    }
}


class datVote {
    /**
    * Va-Vote.inc.php :: Get()
    *
    * This function retrieves the current user's response to the question. It is
    * in the form of an integer, which corresponds to the number of the answer.
    *
    * @param integer $userno
    * @param integer $question
    * @return integer
    */
    function Get($userno,
                 $question) {
        if (!is_int($userno)) {
            $userno = (integer) $userno;
        }
        if (!is_int($question)) {
            $question = (integer) $question;
        }

        // If a guest then there is no answer.
        if ($userno == 0) {
            return 0;
        }

        if ($question < 1) {
            return FALSE;
        }

        $classVoteCfg = new cfgVote;

        // If the question is higher than the number of questions, return zero.
        if ($question > $classVoteCfg->Count()) {
            return 0;
        }

        $file = $GLOBALS['VA_SYSPATHS']['data'] . '/VOTE' . $question . '.DAT';

        // If the file is not found, then return zero.
        if (!file_exists($file)) {
            return 0;
        }

        // Determine if the user has a response first...
        if ((filesize($file) / 2) < $userno) {
            // ...if not, then return zero.
            return 0;
        }

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Determine the user's position in the file.
        $pos = $userno * 2 - 1;

        // Get the user's vote.
        $answer = GetInteger($fileString, $pos);

        // Return the answer.
        return $answer;
    }


    /**
    * Va-Vote.inc.php :: Put()
    *
    * This function will save the user's response to the proper voteX.dat file.
    *
    * @param integer $userno
    * @param integer $question
    * @param integer $answer
    * @return mixed
    */
    function Put($userno,
                 $question,
                 $answer) {
        if (!is_int($userno)) {
            $userno = (integer) $userno;
        }
        if (!is_int($question)) {
            $question = (integer) $question;
        }
        if (!is_int($answer)) {
            $answer = (integer) $answer;
        }

        // If a guest, then do not go further.
        if ($userno == 0) {
            return FALSE;
        }

        if ($question < 0) {
            return FALSE;
        }

        if ($answer < 0) {
            $answer = 0;
        }

        $classVoteCfg = new cfgVote;

        $voteInf = $classVoteCfg->Get();

        if ($question > $voteInf[0]) {
            return FALSE;
        }

        // If the passed answer integer is higher than the highest possible, then
        // it will return FALSE and not update the data file.
        if ($answer > $voteInf[$question]['respno']) {
            return FALSE;
        }

        $file = $GLOBALS['VA_SYSPATHS']['data'] . '/VOTE' . $question . '.DAT';

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Determine the user's position in the file.
        $pos = $userno * 2 - 1;
        $fileString = PutInteger($fileString, $answer, $pos);
        $status = WriteCfgFile($file, $fileString);

        // Return the status of the file write.
        return $status;
    }


    /**
    * Va-Vote.inc.php :: Stats()
    *
    * This function will calculate the total number of votes and determine the
    * number of votes for each response.
    *
    * @param integer $question
    * @return array
    */
    function Stats($question) {
        if (!is_int($question)) {
            $question = (integer) $question;
        }

        $classVoteCfg = new cfgVote;

        if ($question < 0) {
            return FALSE;
        }

        if ($question > $classVoteCfg->Count()) {
            return FALSE;
        }

        // Setup the array...
        for ($i = 0; $i <= 8; $i++) {
            $stats[$i] = 0;
        }

        $file = $GLOBALS['VA_SYSPATHS']['data'] . '/VOTE' . $question . '.DAT';

        // Read the file.
        $fileString = ReadCfgFile($file);
        $offset = 0;

        for ($i = 1; $offset < strlen($fileString); $i++) {
            // Get the response for each user.
            $resp = GetInteger($fileString, 1 + $offset);

            if ($resp > 0) {
                // Increase the total number.
                $stats[0]++;

                // Increase the response.
                $stats[$resp]++;
            }

            // Increase the offset.
            $offset = 2 * $i;
        }

        // Return the results.
        return $stats;
    }
}
?>