<?php
/*
 * ----------------------------------------------------------------------------
 * Vadv-Php.inc.php                                                  01/06/2022
 * This script is used to setup VADV-PHP initiation. This script should always
 * be included first in any script that will use any VADV-PHP functions.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2022 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

// Current VADV-PHP version. DO NOT CHANGE.
$VERSION = '2.1.220106';

// Check PHP version.
if (version_compare(PHP_VERSION, '5.0.0', '<')) {
    die('VADV-PHP v2.0+ requires PHP v5.0 or higher.');
}

/*
 * The DEBUG variables can be altered so you can receive any error messages that
 * occur while in operation. By default, this feature is turned off.
 *
 * Note that when customizing the information below for your account, this
 * feature is known to NOT work with some mail configurations.
 */
// Turn on/off debug mailer.
// This should only be used while developing and turned off on a production site.
$DEBUG_MAIL = FALSE;

// The email address to send bug reports to.
$DEBUG_EMAIL = 'bugs@vadvphp.com';

// The SMTP server address to use to send any bug reports out of.
$DEBUG_SMTP_SERVER = 'mail.at2k.org';

// Set the default error handler.
//set_error_handler('CustomErrorHandler');

// Set errors to be reported, which is used by the error handler.
error_reporting(E_ERROR |
                E_WARNING |
                E_NOTICE |
                E_USER_ERROR |
                E_USER_WARNING |
                E_USER_NOTICE);

// Set default timezone.
if (ini_get('date.timezone') == '') {
    date_default_timezone_set(GetLocalTimezone());
}

// Begin includes...
require_once('include_dir.php');
require_once($includeDir . 'va-functions.inc.php');
require_once($includeDir . 'common.inc.php');
require_once($includeDir . 'va-syspaths.inc.php');

$classVADVPHPCfg = new cfgVADVPHP;
$classSysPathsCfg = new cfgSysPaths;

// Check SSL.
if (($GLOBALS['CONFIG']['forcessl']) &&
    (!IsHttps())) {
    RedirectUser('', 0, 1);
}

require_once($includeDir . 'at2k.inc.php');
//require_once($includeDir . 'forms.inc.php');
require_once($includeDir . 'va-arcs.inc.php');
require_once($includeDir . 'va-channels.inc.php');
require_once($includeDir . 'va-channelx.inc.php');
require_once($includeDir . 'va-colors.inc.php');
require_once($includeDir . 'va-control.inc.php');
require_once($includeDir . 'va-database.inc.php');
require_once($includeDir . 'va-doors.inc.php');
require_once($includeDir . 'va-download.inc.php');
require_once($includeDir . 'vadv-php_dat.inc.php');
require_once($includeDir . 'va-events.inc.php');
require_once($includeDir . 'va-main.inc.php');
require_once($includeDir . 'va-misc.inc.php');
require_once($includeDir . 'va-multifbx.inc.php');
require_once($includeDir . 'va-networks.inc.php');
require_once($includeDir . 'va-quickval.inc.php');
require_once($includeDir . 'va-random.inc.php');
require_once($includeDir . 'va-stats.inc.php');
require_once($includeDir . 'va-textsect.inc.php');
require_once($includeDir . 'va-upload.inc.php');
require_once($includeDir . 'va-userfile.inc.php');
require_once($includeDir . 'va-vote.inc.php');

$classMainCfg = new cfgMain;
$classStatVBBSDat = new datStatVBBS;
$classVADVPHPDat = new datVADVPHP;

require_once($includeDir . 'va-loginauth.inc.php');

// Added for theme support ----------------------------------
require_once($includeDir . 'vadv-php_themes.inc.php');
// ----------------------------------------------------------

// ...end includes.

// Seed random number generator.
srand((double) microtime() * 1000000);

$GLOBALS['USER_SETTINGS'] = $classVADVPHPDat->Get($GLOBALS['USER']['userno']);

if (!isset($GLOBALS['VA_ACTION'])) {
    $GLOBALS['VA_ACTION'] = 'Browsing';
}

// Update the WEBCALLER.LST file.
$GLOBALS['VA_USERS'] = UpdateWhosOnline($GLOBALS['VA_ACTION']);

// Increase the hit counter by one if a new session.
if (((!isset($_SESSION['hitcounter'])) ||
    (!$_SESSION['hitcounter'])) &&
    (!DetectSpider())) {
    $fileString = $classStatVBBSDat->Read();
    if ($fileString !== FALSE) {
        $stats = $classStatVBBSDat->Get($fileString);
        $stats['webhitstotal']++;
        $fileString = $classStatVBBSDat->Put($fileString, $stats);
        $classStatVBBSDat->Write($fileString);
        $_SESSION['hitcounter'] = TRUE;
    }
}

class cfgVADVPHP {
    function __CONSTRUCT() {
        if ((!isset($GLOBALS['CONFIG'])) ||
            (count($GLOBALS['CONFIG']) == 0)) {
            $GLOBALS['CONFIG'] = $this->Get();
        }
    }


    /**
    * Va-Syspaths.inc.php :: Get()
    *
    * This function will read the VADV-PHP configuration file (vadv-php_config.php)
    * and place its variables into an array.
    *
    * @return array
    */
    function Get() {
        // Set the default values.
        $config = array();
        $config['banned'] = '';                 // Banned user accounts.
        $config['bbsdir'] = 'c:/va';            // BBS directory.
        $config['cookielife'] = 7;              // Cookie life span.
        $config['defaulttheme'] = 1;            // Default theme ID.
        $config['encryption'] = 'sha256';       // Encryption method.
        $config['fileencryption'] = 'md5';      // File hash method.
        $config['filehost'] = '';               // Network file directory.
        $config['fileupload'] = 0;              // File uploads.
        $config['fileuploadsl'] = 255;          // File upload security level.
        $config['flashterm'] = 0;               // Flashterm support.
        $config['forcessl'] = 0;                // Force SSL.
        $config['guestacct'] = 0;               // Guest account number.
        $config['hostname'] = '';               // Network BBS directory.
        $config['newfilescan'] = 1;             // Newfile days old.
        $config['newslist'] = TRUE;             // NewsList support.
        $config['newuseroneliner'] = FALSE;     // Send SysOp Oneliner on new user registration
        $config['newuserreq'] = 141;            // Required new user information.
        $config['postitnt'] = TRUE;             // Post-It! Network support.
        $config['purgetime'] = 0;               // Purge time.
        $config['purgeminlevel'] = 0;           // Purge minimum security level.
        $config['purgemaxlevel'] = 0;           // Purge maximum security level.
        $config['refresh'] = 0;                 // Refresh time.
        $config['requiresecurelogin'] = 0;      // Require secure login using js.
        $config['rssfeed'] = TRUE;              // RSS feed support.
        $config['rssfeedlimit'] = 50;           // RSS feed max number of messages.
        $config['showcustomdbs'] = FALSE;       // Show custom DB's to SysOp.
        $config['sysnews'] = TRUE;              // System News support.
        $config['syspass'] = TRUE;              // System password for Vconfig.
        $config['telnet'] = '';                 // Telnet address.
        $config['telnetport'] = 23;             // Telnet port.
        $config['themetracker'] = FALSE;        // Themetracker
        $config['useraccflags'] = '';           // Guest access flags.
        $config['userage'] = 0;                 // Guest age.
        $config['usersl'] = 15;                 // Guest security level.
        $config['webmaster'] = 'webmaster@';    // Webmaster email address.

        global $rootDir;
        global $includeDir;

        // Check to see if the config file exists.
        if (!file_exists($includeDir . 'vadv-php_config.php')) {
            // If missing, copy a fresh one from the files folder.
            if (file_exists($rootDir . 'files/vadv-php_config.php')) {
                copy($rootDir . 'files/vadv-php_config.php', $includeDir . 'vadv-php_config.php');
            } else {
                die('Configuration file (vadv-php_config.php) was not found.');
            }
        }

        include($includeDir . 'vadv-php_config.php');

        // Tidy it up a bit...
        $config['banned'] = trim($config['banned']);
        $config['bbsdir'] = trim($config['bbsdir']);
        $config['cookielife'] = (integer) $config['cookielife'];
        $config['defaulttheme'] = (integer) $config['defaulttheme'];
        $config['encryption'] = trim(strtolower($config['encryption']));
        $config['fileencryption'] = trim(strtolower($config['fileencryption']));
        $config['filehost'] = trim($config['filehost']);
        $config['fileupload'] = (boolean) $config['fileupload'];
        $config['fileuploadsl'] = (integer) $config['fileuploadsl'];
        $config['flashterm'] = (boolean) $config['flashterm'];
        $config['forcessl'] = (boolean) $config['forcessl'];
        $config['guestacct'] = (integer) $config['guestacct'];
        $config['hostname'] = trim($config['hostname']);
        $config['newfilescan'] = (integer) $config['newfilescan'];
        $config['newslist'] = (boolean) $config['newslist'];
        $config['newuseroneliner'] = (boolean) $config['newuseroneliner'];
        $config['newuserreq'] = (integer) $config['newuserreq'];
        $config['postitnt'] = (boolean) $config['postitnt'];
        $config['purgetime'] = (integer) $config['purgetime'];
        $config['purgeminlevel'] = (integer) $config['purgeminlevel'];
        $config['purgemaxlevel'] = (integer) $config['purgemaxlevel'];
        $config['refresh'] = (integer) $config['refresh'];
        $config['requiresecurelogin'] = (boolean) $config['requiresecurelogin'];
        $config['rssfeed'] = (boolean) $config['rssfeed'];
        $config['rssfeedlimit'] = (integer) $config['rssfeedlimit'];
        $config['showcustomdbs'] = (boolean) $config['showcustomdbs'];
        $config['sysnews'] = (boolean) $config['sysnews'];
        $config['syspass'] = (boolean) $config['syspass'];
        $config['telnet'] = trim($config['telnet']);
        $config['telnetport'] = (integer) $config['telnetport'];
        $config['themetracker'] = (boolean) $config['themetracker'];
        $config['useraccflags'] = trim(strtoupper(str_replace(' ', '', $config['useraccflags'])));
        $config['userage'] = (integer) $config['userage'];
        $config['usersl'] = (integer) $config['usersl'];
        $config['webmaster'] = trim($config['webmaster']);

        // Check default theme.
        if ($config['defaulttheme'] < 1) {
            $config['defaulttheme'] = 1;
        }

        // If the guest account is used, then ban that account.
        if ($config['guestacct'] > 0) {
            $config['banned'] = $config['guestacct'] . ',' . $config['banned'];
        }

        // If the purge max security level is zero, then set it to 254.
        if ($config['purgemaxlevel'] == 0) {
            $config['purgemaxlevel'] = 254;
        }

        return $config;
    }


    /**
    * Va-SysPaths.inc.php :: Write()
    *
    * This function will write a new vadv-php_config.php file based on the values in
    * the configuration variable.
    *
    * @param array $configVariable
    * @return mixed
    */
    function Write($configVariable) {
        if (!is_array($configVariable)) {
            trigger_error(VA_Error(1));
        }

        global $includeDir;

        $filename = $includeDir . 'vadv-php_config.php';
        if (!file_exists($filename)) {
            return FALSE;
        }

        // Read the file into an array.
        $file = file($filename);

        $newFile = '';
        for ($i = 0; $i < count($file) - 1; $i++) {
            $line = $file[$i];
            $comment = strpos($line, '//');
            if ($comment !== FALSE) {
                $line = trim(substr($line, 0, $comment - 1));
            }
            $line = trim(strtolower($line));
            if (($line != '') ||
                ($line != '?'. '>')) {
                $pos = strpos($line, '$config[');
                if ($pos !== FALSE) {
                    $line = trim(substr($line, $pos + 9));
                    $pos = strpos($line, ']');
                    $line = trim(substr($line, 0, $pos - 1));
                    reset($configVariable);
                    while (list($key, $value) = each($configVariable)) {
                        if (strcmp(trim(strtolower($key)), trim($line)) == 0) {
                            $file[$i] = '$' . "config['" . $key . "'] = '" . trim($value) . "'" . ';' . "\r\n";
                            unset($configVariable[$key]);
                        }
                    }
                }
            }
            $newFile = $newFile . $file[$i];
        }

        // Any left over config settings, append to the end of the file.
        reset($configVariable);
        while (list($key, $value) = each($configVariable)) {
            $newFile = $newFile . '$' . "config['" . $key . "'] = '" . trim($value) . "'" . ';' . "\r\n";
        }

        $newFile = $newFile . '?' . '>';

        // Write the new config file.
        $status = WriteCfgFile($filename, $newFile);

        // Return the status.
        return $status;
    }
}


/**
* GetLocalTimezone()
*
* This function will return the php equivalent of the timezone configured in
* the OS. This code was originally found in the php manual.
*
* @return string
*/
function GetLocalTimezone() {
    $iTime = time();
    $arr = localtime($iTime);
    $arr[5] += 1900;
    $arr[4]++;
    $iTztime = gmmktime($arr[2], $arr[1], $arr[0], $arr[4], $arr[3], $arr[5], $arr[8]);
    $offset = doubleval(($iTztime-$iTime)/(60*60));
    $zonelist = array (
        'Kwajalein' => -12.00,
        'Pacific/Midway' => -11.00,
        'Pacific/Honolulu' => -10.00,
        'America/Anchorage' => -9.00,
        'America/Los_Angeles' => -8.00,
        'America/Denver' => -7.00,
        'America/Chicago' => -6.00,
        'America/New_York' => -5.00,
        'America/Caracas' => -4.30,
        'America/Halifax' => -4.00,
        'America/St_Johns' => -3.30,
        'America/Argentina/Buenos_Aires' => -3.00,
        'America/Sao_Paulo' => -3.00,
        'Atlantic/South_Georgia' => -2.00,
        'Atlantic/Azores' => -1.00,
        'Europe/Dublin' => 0,
        'Europe/Belgrade' => 1.00,
        'Europe/Minsk' => 2.00,
        'Asia/Kuwait' => 3.00,
        'Asia/Tehran' => 3.30,
        'Asia/Muscat' => 4.00,
        'Asia/Yekaterinburg' => 5.00,
        'Asia/Kolkata' => 5.30,
        'Asia/Katmandu' => 5.45,
        'Asia/Dhaka' => 6.00,
        'Asia/Rangoon' => 6.30,
        'Asia/Krasnoyarsk' => 7.00,
        'Asia/Brunei' => 8.00,
        'Asia/Seoul' => 9.00,
        'Australia/Darwin' => 9.30,
        'Australia/Canberra' => 10.00,
        'Asia/Magadan' => 11.00,
        'Pacific/Fiji' => 12.00,
        'Pacific/Tongatapu' => 13.00
    );

    $index = array_keys($zonelist, $offset);
    if (sizeof($index) != 1) {
        return FALSE;
    }

    return $index[0];
 }


/**
* GetVADVTimezone()
*
* This function will return the php equivalent of the timezone configured in
* VConfig.
*
* @return string
*/
function GetVADVTimezone() {
    switch (intval($GLOBALS['VA_MAIN']['timezone'])) {
        case -1200:
            $timezone = 'Kwajalein';
            break;
        case -1100:
            $timezone = 'Pacific/Midway';
            break;
        case -1000:
            $timezone = 'Pacific/Honolulu';
            break;
        case -900:
            $timezone = 'America/Anchorage';
            break;
        case -800:
            $timezone = 'America/Los_Angeles';
            break;
        case -700:
            $timezone = 'America/Denver';
            break;
        case -600:
            $timezone = 'America/Chicago';
            break;
        case -500:
            $timezone = 'America/New_York';
            break;
        case -430:
            $timezone = 'Caracas';
            break;
        case -400:
            $timezone = 'Halifax';
            break;
        case -330:
            $timezone = 'America/Argentina/Buenos_Aires';
            break;
        case -300:
            $timezone = 'America/Sao_Paulo';
            break;
        case -200:
            $timezone = 'Atlantic/South_Georgia';
            break;
        case -100:
            $timezone = 'Atlantic/Azores';
            break;
        case 0:
            $timezone = 'Europe/Dublin';
            break;
        case 100:
            $timezone = 'Europe/Belgrade';
            break;
        case 200:
            $timezone = 'Europe/Minsk';
            break;
        case 300:
            $timezone = 'Asia/Kuwait';
            break;
        case 330:
            $timezone = 'Asia/Tehran';
            break;
        case 400:
            $timezone = 'Asia/Muscat';
            break;
        case 500:
            $timezone = 'Asia/Yekaterinburg';
            break;
        case 530:
            $timezone = 'Asia/Kolkata';
            break;
        case 545:
            $timezone = 'Asia/Katmandu';
            break;
        case 600:
            $timezone = 'Asia/Dhaka';
            break;
        case 630:
            $timezone = 'Asia/Rangoon';
            break;
        case 700:
            $timezone = 'Asia/Krasnoyarsk';
            break;
        case 800:
            $timezone = 'Asia/Brunei';
            break;
        case 900:
            $timezone = 'Asia/Seoul';
            break;
        case 930:
            $timezone = 'Australia/Darwin';
            break;
        case 1000:
            $timezone = 'Australia/Canberra';
            break;
        case 1100:
            $timezone = 'Asia/Magadan';
            break;
        case 1200:
            $timezone = 'Pacific/Fiji';
            break;
        case 1300:
            $timezone = 'Pacific/Tongatapu';
            break;
        default:
            $timezone = 'America/Chicago';
            break;
    }
print $timezone;
    return $timezone;
}
?>