<?php
/*
 * ----------------------------------------------------------------------------
 * VADV-PHP_Dat.inc.php                                              12/22/2009
 * This script will read the vadv-php.dat file for the specified user and store
 * its information into an array. This data file is not created by VADV, but is
 * a VADV-PHP settings file that stores various user preferences.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2008 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');


class datVADVPHP {
    /**
    * VADV-PHP_Dat.inc.php :: Get()
    *
    * This function will get the requested entry information from vadv-php.dat.
    *
    * @param integer $userNo
    * @return array
    */
    function Get($userNo) {
        if (!is_int($userNo)) {
            $userNo = (integer) $userNo;
        }

        // Defaults
        $default['hidedeleted'] = 0;
        $default['hidesentdeleted'] = 0;
        $default['timezone'] = (integer) date("O");
        $default['htmlencode'] = 0;
        $default['tagline'] = 0;
        $default['newmessagescan'] = 0;    // Reverse: 0 = true, 1 = false
        $default['password'] = '';
        $default['theme'] = $GLOBALS['CONFIG']['defaulttheme'];

        if (($userNo <= 0) ||
            ($userNo == $GLOBALS['CONFIG']['guestacct'])) {
            return $default;
        }

        $fileString = $this->Read();

        $user = array();
        $offset = 100 * ($userNo - 1);
        $user['hidedeleted'] = GetInteger($fileString, 1 + $offset);
        $user['hidesentdeleted'] = GetInteger($fileString, 3 + $offset);
        $user['timezone'] = GetInteger($fileString, 5 + $offset);
        $user['htmlencode'] = GetInteger($fileString, 7 + $offset);
        $user['tagline'] = GetInteger($fileString, 9 + $offset);
        $user['newmessagescan'] = GetInteger($fileString, 11 + $offset);
        $user['password'] = GetString($fileString, 13 + $offset, 64);
        $user['theme'] = GetInteger($fileString, 77 + $offset);

        // If any of the settings are not present, then set them to the defaults.
        if ($user['hidedeleted'] === '' ||
            $user['hidedeleted'] === 0) {
            $user['hidedeleted'] = $default['hidedeleted'];
        }
        if ($user['hidesentdeleted'] === '' ||
            $user['hidesentdeleted'] === 0) {
            $user['hidesentdeleted'] = $default['hidesentdeleted'];
        }
        if (($user['timezone'] === '') ||
            ($user['timezone'] == 0)) {
            $user['timezone'] = (integer) date("O");
        } elseif ($user['timezone'] == 1) {
            $user['timezone'] = 0;
        }
        if ($user['htmlencode'] === '' ||
            $user['htmlencode'] === 0) {
            $user['htmlencode'] = $default['htmlencode'];
        }
        if ($user['tagline'] === '' ||
            $user['tagline'] === 0) {
            $user['tagline'] = $default['tagline'];
        }
        if ($user['newmessagescan'] === '' ||
            $user['newmessagescan'] === 0) {
            $user['newmessagescan'] = $default['newmessagescan'];
        }
        if ($user['password'] === '') {
            $user['password'] = $default['password'];
        }
        if ($user['theme'] === '' ||
            $user['theme'] === 0) {
            $user['theme'] = $default['theme'];
        }

        // Until we can add the sent db to vcleandb...
        $user['hidesentdeleted'] = 0;

        // Return the user information.
        return $user;
    }


    /**
    * VADV-PHP_Dat.inc.php :: Put()
    *
    * This function will put new information into the vadv-php.dat file string.
    *
    * @param string $fileString
    * @param integer $userNo
    * @param array $configVariable
    * @return string
    */
    function Put($fileString,
                 $userNo,
                 $configVariable) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }
        if (!is_int($userNo)) {
            $userNo = (integer) $userNo;
        }
        if (!is_array($configVariable)) {
            trigger_error(VA_Error(1));
        }

        $total = intval(strlen($fileString) / 100);
        if ($userNo > $total) {
            $fileString = $fileString . str_pad('', 100, chr(0));
        }

        $offset = 100 * ($userNo - 1);
        if (isset($configVariable['hidedeleted'])) {
            $fileString = PutInteger($fileString, $configVariable['hidedeleted'], 1 + $offset);
        }
        if (isset($configVariable['hidesentdeleted'])) {
            $fileString = PutInteger($fileString, $configVariable['hidesentdeleted'], 3 + $offset);
        }
        if (isset($configVariable['timezone'])) {
            $fileString = PutInteger($fileString, $configVariable['timezone'], 5 + $offset);
        }
        if (isset($configVariable['htmlencode'])) {
            $fileString = PutInteger($fileString, $configVariable['htmlencode'], 7 + $offset);
        }
        if (isset($configVariable['tagline'])) {
            $fileString = PutInteger($fileString, $configVariable['tagline'], 9 + $offset);
        }
        if (isset($configVariable['newmessagescan'])) {
            $fileString = PutInteger($fileString, $configVariable['newmessagescan'], 11 + $offset);
        }
        if (isset($configVariable['password'])) {
            $fileString = PutString($fileString, $configVariable['password'], 13 + $offset, 64);
        }
        if (isset($configVariable['theme'])) {
            $fileString = PutInteger($fileString, $configVariable['theme'], 77 + $offset);
        }

        // Return the new file string.
        return $fileString;
    }


    /**
    * VADV-PHP_Dat.inc.php :: Read()
    *
    * This function will read the vadv-php.dat file.
    *
    * @return string
    */
    function Read() {
        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['data'] . '/VADV-PHP.DAT';

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Return the file string.
        return $fileString;
    }


    /**
    * VADV-PHP_Dat.inc.php :: Write()
    *
    * This function writes the new file string into vadv-php.dat.
    *
    * @param string $fileString
    * @return mixed
    */
    function Write($fileString) {
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }

        // Set the filename.
        $file = $GLOBALS['VA_SYSPATHS']['data'] . '/VADV-PHP.DAT';
        $status = WriteCfgFile($file, $fileString);

        // Return the result status.
        return $status;
    }
}


?>