<?php
/*
 * ----------------------------------------------------------------------------
 * VADV-PHP_Themes_Dat.inc.php                                       09/02/2011
 * This script will read the vadv-php_themes.dat file for installed themes and
 * store its information into an array. This data file is not created by VADV,
 * but is a VADV-PHP settings file that stores the website theme information.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2011 Steve Winn. All Rights Reserved.
 * Theme integration by Darrell Fox, Sept-Dec, 2009
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');
require_once($includeDir . 'vadv-php_themefunc.inc.php');

// Set default session variable.
if (!isset($_SESSION['theme'])) {
    $_SESSION['theme'] = '';
}

class datVADVPHPThemes {
    /**
    * VADV-PHP_Themes_Dat.inc.php :: Get()
    *
    * This function will get the requested entry information from vadv-php_themes.dat.
    *
    * @param integer $themeID
    * @return array
    */
    function Get($themeID) {
        if (!is_int($themeID)) {
            $themeID = (integer) $themeID;
        }

        // Defaults
        $default['themeName'] = '';
        $default['themeVer'] = '0.0';
        $default['themeDate'] = '0';
        $default['themeAuthor'] = '';
        $default['themeEmail'] = '';
        $default['themeSite'] = '';
        $default['themeDisabled'] = 1;
        $default['themeSecLevel'] = 0;

        if ($themeID <= 0 ||
            $themeID > $this->count()) {
            $default['themeID'] = 0;
            return $default;
        }

        $fileString = $this->Read();

        $theme = array();
        $offset = 200 * ($themeID - 1);
        $theme['themeName'] = GetString($fileString, 1 + $offset, 16);
        $theme['themeVer'] = GetString($fileString, 17 + $offset, 10);
        $theme['themeDate'] = GetLongInt($fileString, 27 + $offset);
        $theme['themeAuthor'] = GetString($fileString, 31 + $offset, 40);
        $theme['themeEmail'] = GetString($fileString, 71 + $offset, 60);
        $theme['themeSite'] = GetString($fileString, 131 + $offset, 60);
        $theme['themeDisabled'] = GetInteger($fileString, 191 + $offset);
        $theme['themeSecLevel'] = GetInteger($fileString, 193 + $offset);


        // If any of the settings are not present, then set them to the defaults.
        if ($theme['themeName'] === '' ||
            $theme['themeName'] === 0) {
            $theme['themeName'] = $default['themeName'];
        }
        if ($theme['themeVer'] === '' ||
            $theme['themeVer'] === 0) {
            $theme['themeVer'] = $default['themeVer'];
        }
        if ($theme['themeDate'] === '' ||
            $theme['themeDate'] === 0) {
            $theme['themeDate'] = $default['themeVer'];
        }
        if ($theme['themeAuthor'] === '' ||
            $theme['themeAuthor'] === 0) {
            $theme['themeAuthor'] = $default['themeAuthor'];
        }
        if ($theme['themeEmail'] === '' ||
            $theme['themeEmail'] === 0) {
            $theme['themeEmail'] = $default['themeEmail'];
        }
        if ($theme['themeSite'] === '' ||
            $theme['themeSite'] === 0) {
            $theme['themeSite'] = $default['themeSite'];
        }
        if ($theme['themeDisabled'] === '') {
            $theme['themeDisabled'] = $default['themeDisabled'];
        }
        if ($theme['themeSecLevel'] === '') {
            $theme['themeSecLevel'] = $default['themeSecLevel'];
        }

        // Return the theme information.
        return $theme;
    }


    /**
    * VADV-PHP_Themes_Dat.inc.php :: Put()
    *
    * This function will put new information into the vadv-php_themes.dat file string.
    *
    * @param string $fileString
    * @param integer $themeID
    * @param array $configVariable
    * @return string
    */
    function Put($fileString,
                 $themeID,
                 $configVariable) {
        if ($fileString === FALSE) {
            return FALSE;
        }
        if (!is_int($themeID)) {
            $themeID = (integer) $themeID;
        }
        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }
        if (!is_array($configVariable)) {
            // Why doesn't this exit the functions (it's in every class's Put() function also, but doesn't exit,
            // just returns an error message and continues with execution on an invalid variable that's not an array...
            // TODO: end functions and properly show error messages... ;) -D-
            trigger_error(VA_Error(1));
        }

        if ($themeID > $this->count()) {
            $fileString = $fileString . str_pad('', 200, chr(0));
        }

        $offset = 200 * ($themeID - 1);
        if (isset($configVariable['themeName'])) {
            $fileString = PutString($fileString, $configVariable['themeName'], 1 + $offset, 16);
        }
        if (isset($configVariable['themeVer'])) {
            $fileString = PutString($fileString, $configVariable['themeVer'], 17 + $offset, 10);
        }
         if (isset($configVariable['themeDate'])) {
            $fileString = PutLongInt($fileString, $configVariable['themeDate'], 27 + $offset, 1);
        }
        if (isset($configVariable['themeAuthor'])) {
            $fileString = PutString($fileString, $configVariable['themeAuthor'], 31 + $offset, 40);
        }
        if (isset($configVariable['themeEmail'])) {
            $fileString = PutString($fileString, $configVariable['themeEmail'], 71 + $offset, 60);
        }
        if (isset($configVariable['themeSite'])) {
            $fileString = PutString($fileString, $configVariable['themeSite'], 131 + $offset, 60);
        }
        if (isset($configVariable['themeDisabled']) && $themeID != $GLOBALS['CONFIG']['defaulttheme']) {
            $fileString = PutInteger($fileString, $configVariable['themeDisabled'], 191 + $offset);
        }
        if (isset($configVariable['themeSecLevel'])) {
            $fileString = PutInteger($fileString, $configVariable['themeSecLevel'], 193 + $offset);
        }

        // Return the new file string.
        return $fileString;
    }


    /**
    * VADV-PHP_Themes_Dat.inc.php :: Read()
    *
    * This function will read the vadv-php_themes.dat file.
    *
    * @return string
    */
    function Read() {
        global $rootDir;
        // Set the filename.
        $file = $rootDir . 'themes/vadv-php_themes.dat';

        // Read the file.
        $fileString = ReadCfgFile($file);

        // Return the file string.
        return $fileString;
    }


    /**
    * VADV-PHP_Themes_Dat.inc.php :: Write()
    *
    * This function writes the new file string into vadv-php_themes.dat.
    *
    * @param string $fileString
    * @return mixed
    */
    function Write($fileString) {
        global $rootDir;
        if ($fileString === FALSE) {
            return FALSE;
        }

        if (!is_string($fileString)) {
            $fileString = (string) $fileString;
        }

        // Set the filename.
        $file = $rootDir . 'themes/vadv-php_themes.dat';
        $status = WriteCfgFile($file, $fileString);

        // Return the result status.
        return $status;
    }


    /**
    * VADV-PHP_Themes_Dat.inc.php :: Count()
    *
    * This function simply returns the number of installed themes in the dat file.
    *
    * @return integer
    */
    function Count() {
        global $rootDir;
        // Set the filename.
        $file = $rootDir . 'themes/vadv-php_themes.dat';
        if (!file_exists($file)) {
            return 0;
        }

        $themeCount = intval(filesize($file) / 200);

        return $themeCount;
    }
}


?>