<?php
/*
 * ----------------------------------------------------------------------------
 * recreate_theme_dat.php                                            05/05/2013
 * This script will recreate your original /themes/vadv-php_themes.dat file
 * in case of corruption or deletion.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2013 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 *
 ******************************************************************************
 * NOTE: The VirtualClassic theme **MUST** exist for this to work!
 *
 * It must be located at: $rootDir/themes/VirtualClassic
 * It must contain a valid vadvtheme.cfg in it's root
 *
 * If not, get a new one or a copy from somone and unzip it properly to work
 ******************************************************************************
 * ----------------------------------------------------------------------------
 */

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');


// First, check to see if they're logged in, if not, allow them to login
$loginerror = '';
$enc_script = '';
$status = false;
$themeError = '';

if ((isset($_SESSION['auth'])) && (!$_SESSION['auth'])) {

    if ((isset($_SESSION['error'])) && ($_SESSION['error'])) {

        if (isset($_SESSION['login'])) {
            // Destroy the session information.
            setcookie('vadvphp', '', time() - 86400, '/', '', 0);
            $_SESSION = array('');
            session_destroy();
            session_start();
        }

        $loginerror = 'Invalid username or password!' . "\n";
    }

    // User is NOT logged in...
    $status = false;

    // Get login id for this session...
    $loginId = '';
    for ($i = 1; $i <= $GLOBALS['VA_USERS'][0]; $i++) {
        if ($GLOBALS['VA_USERS'][$i]['session'] == session_id()) {
            $loginId = $GLOBALS['VA_USERS'][$i]['id'];
            break;
        }
    }

    // Get encryption method for JavaScript login
    switch ($GLOBALS['CONFIG']['encryption']) {
        case 'md5':
            $enc_script = '<script language="JavaScript" type="text/javascript" src="' . $includeDir . 'javascript/md5.js"></script>' . "\n";
            break;
        case 'sha1':
            $enc_script = '<script language="JavaScript" type="text/javascript" src="' . $includeDir . 'javascript/sha1.js"></script>' . "\n";
            break;
        case 'sha256':
            $enc_script = '<script language="JavaScript" type="text/javascript" src="' . $includeDir . 'javascript/sha256.js"></script>' . "\n";
            break;
        default:
            $enc_script = '';
            break;
    }
}else{
    $status = true;
}



//=====================================================================================================
// Output header
if (!isset($_POST['submit'])) {
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" dir="ltr" lang="en" xml:lang="en">
<head>

<title><?php if ($pagetitle != '') { print '[' . $pagetitle . '] - '; } print HtmlSpace($GLOBALS['VA_MAIN']['bbsname']); ?>'s VADV-PHP Home</title>

<meta http-equiv="content-type" content="text/html; charset=UTF-8" />
<meta http-equiv="imagetoolbar" content="no" />
<meta name="distribution" content="global" />
<meta name="copyright" content="Copyright &#169; <?php print date('Y'); ?>, Steve Winn" />
<meta name="description" content="" />
<meta name="keywords" content="" />
<?php
if (defined('USE_REFRESH') && $GLOBALS['CONFIG']['refresh'] > 0) {
    print '<meta http-equiv="refresh" content="' . $GLOBALS['CONFIG']['refresh'] . '" />' . "\n";
}
?>

<style type="text/css">
* { font-size: 11px; }
html { height: 101%;margin-bottom: 1px; }
body { font-family: Verdana, Helvetica, Arial, sans-serif; font-size: 11px; color: #000000; background-color: #FFFFFF; }
hr { height:2px; color: #000080; border-width:0; background-color:#000080; }
#mainframe { width: 100%; }
#main { width: 750px; padding: 10px 0 5px 0; background-color: #FFFFFF; }
#head { width: 730px; margin: 0 0 0 0; height: 60px; padding: 0 0 0 0; background-color: #000080; }
#page { width: 730px; padding: 6px 0 10px 0; }
.hdrTitle { color: #FFFFFF; font-size: 19px; font-weight: bold; font-style: italic; }
.boldBody { color: #000000; font-weight:bold; }
.infoBarBody { color: #FFFFFF; background-color: #0066CC; }
.boldtopicsmall { color: #FFFFFF; background-color: #000080; font-weight: bold; }
.notify { color: red; font-weight: bold; }
.notifyTitle { font-size: 13px; color: #FFFF00; font-weight: bold; }
.yellowTitle { font-size: 19px; color: #FFFF00; font-weight:bold; font-style:italic; }
.copyright { color: #000000; font-size: 9px; font-weight: normal; }
.footerVersion { font-family: monospace; color: #808080; font-weight: bold; }
a:active { color: #FF0000; }
a:link { color: #0000D0; }
a:visited { color: #0000D0; }
a:hover { color: #FF0000; }
form { padding: 0; margin: 0; }
input.text { background-color: #A0C0F0; font-weight: bold; }
input.submit, button.submit {
    width: 98px; height: 30px; border: none; background-color: transparent; background-image: url('buttonback.png'); color: #FFFFFF; font-size: 12px; font-weight: bold; padding-bottom: 5px;
}
input.checkbox { border: none; vertical-align: middle; background-color: transparent; padding: 0; }
</style>


</head>
<body>
<div id="mainframe" align="center">
<div id="main">

<div id="head" align="center">
<table cellpadding="0" cellspacing="0" style="padding: 5px 5px 5px 5px;" width="100%" height="50">
    <tr>
        <td class="hdrTitle" align="left" width="550"><?php print HtmlSpace($GLOBALS['VA_MAIN']['bbsname']); ?>'s VADV-PHP Home</td>
        <td align="right" width="180"><a href="http://www.vadvphp.com">
            <img border="0" src="vadv-php_blue.gif" alt="VADV-PHP" title="VADV-PHP" width="50" height="50" align="right" /></a>
        </td>
    </tr>
</table>
</div> <!-- head -->

<div id="page">
<?php
}
//=====================================================================================================


if ($status) {
    // They're logged in, now check their security level and kick 'em to the curb if they're not high enough
    if ($GLOBALS['USER']['securitylevel'] < 255) {
        $themeError = '<br /><h4>You do not have access to this page. Press the BACK button on your browser.</h4>';
    }


    $classThemeCreate = new datVADVPHPThemes;


    $themeInfo = getThemeCfg('VirtualClassic');

    if ($themeInfo == false) {
        $themeError = "<br /><b>Please copy or unZip the VirtualClassic theme package to &quot;YourWebRoot/themes/&quot;<br /><br />";
        $themeError .= "When properly done, your theme should reside at:<br /><br />&quot;YourWebRoot/themes/VirtualClassic/&quot;<br /><br />";
        $themeError .= "and contain a valid &quot;vadvtheme.cfg&quot;<br /><br />";
    }

    if (isset($_POST['submit'])) {
        $themeInfo['themeDisabled'] = 0;
        $themeInfo['themeSecLevel'] = 0;

        $fileString = '';
        $fileString = $classThemeCreate->Put($fileString, 1, $themeInfo);
        $classThemeCreate->Write($fileString);

        // Change our Global Config to make sure our default theme is this one
        $newConfig = $GLOBALS['CONFIG'];
        $newConfig['defaulttheme'] = 1;
        $classVADVPHPCfg->Write($newConfig);

        // Output the VirtualClassic header for our success page
        $themepath = $rootDir . 'themes/VirtualClassic';
        include($themepath . '/templates/header.php');

        $errortype = 'ReCreate Theme Dat File';
        $message = '<br /><b style="color: green;">New VADV-PHP_THEMES.DAT File Created!</b><br /><br />';
        $message .= 'You now have one theme installed: VirtualClassic (Default OEM Theme)<br /><br />';
        $message .= 'You will now be redirected to your main index page...';
        $delay = 7;
        $redirectURL = $rootDir . 'index.php';

        // Output success page
        include($themepath . '/templates/errmsg.php');

        // Output footer
        include($themepath . '/templates/footer.php');
        exit;
    }


}
//=====================================================================================================
// Output page
print $enc_script;
if (!$status) { ?>
<script language="JavaScript" type="text/javascript">
<!--
    var submitted = 0;
    function validate(form) {
        if (submitted == 0) {
            if ((form.username.value.length == 0) ||
                (form.password.value.length == 0)) {
                alert("Please enter a username and password.");
                form.username.focus();
                return false;
            } else if (form.username.value.length > 40) {
                alert("The username cannot be longer than 40 characters.");
                form.username.focus();
                return false;
            } else if (form.password.value.length > 32) {
                alert("The password cannot be longer than 32 characters.");
                form.password.focus();
                return false;
            } else {
                form.enc.value = form.password.value.toUpperCase();
<?php
    switch ($GLOBALS['CONFIG']['encryption']) {
        case 'md5':
            print 'form.enc.value = hex_md5(form.enc.value);' . "\n";
            print 'form.enc.value = "|enc|" + hex_hmac_md5(form.id.value, form.enc.value);' . "\n";
            break;
        case 'sha1':
            print 'form.enc.value = hex_sha1(form.enc.value);' . "\n";
            print 'form.enc.value = "|enc|" + hex_hmac_sha1(form.id.value, form.enc.value);' . "\n";
            break;
        case 'sha256':
            //print 'form.enc.value = SHA256_hash(form.enc.value);' . "\n";
            //print 'form.enc.value = "|enc|" + HMAC_SHA256_MAC(form.id.value, form.enc.value);' . "\n";
            print 'form.enc.value = hex_sha256(form.enc.value);' . "\n";
            print 'form.enc.value = "|enc|" + hex_hmac_sha256(form.id.value, form.enc.value);' . "\n";
            break;
        default:
            break;
    }
?>
                form.password.value = "";
                form.id.value = "";
                submitted = 1;
                return true;
            }
        } else {
            return false;
        }
    }
//-->
</script>

<table cellspacing="0"cellpadding="2" width="100%" style="border: solid 1px #666666;">
    <form name="login" method="POST" action="<?php print $rootDir; ?>login.php" onsubmit="return validate(login)">
    <input type="hidden" name="referer" value="<?php print GetCurrentURL(0); ?>" />
    <tr>
        <td class="infoBarBody" align="left" valign="top" colspan="2">
            <span class="yellowTitle">&nbsp;Login</span>
        </td>
    </tr>
    <script language="JavaScript" type="text/javascript">
    <!--
        document.write('<input type="hidden" name="id" value="<?php print $loginId; ?>" />');
        document.write('<input type="hidden" name="enc" value="" />');
    -->
    </script>
    <?php if (isset($_GET['url'])) { ?>
        <input type="hidden" name="url" value="<?php print $_GET['url']; ?>" />
    <?php } ?>
    <tr class="boldtopicsmall">
        <td colspan="2" width="100%">&nbsp;</td>
    </tr>
    <noscript>
    <tr>
        <td align="center" colspan="2" style="color: red;font-weight: bold;">
            <br />
            <?php if ($GLOBALS['CONFIG']['requiresecurelogin']) { ?>
            Your browser has JavaScript disabled. It is required to login.
            <?php } else { ?>
            Enable JavaScript for a more secure login!
            <?php } ?>
        </td>
    </tr>
    </noscript>

    <?php if ($loginerror != '') { ?>
    <tr>
        <td class="notify" align="center" colspan="2"><?php print $loginerror; ?></td>
    </tr>
    <?php } ?>
    <tr>
        <td class="boldBody" align="center" colspan="2">
            <br />
            <?php if (($GLOBALS['USER']['userno'] == 0) || ($GLOBALS['USER']['userno'] == $GLOBALS['CONFIG']['guestacct'])) { ?>
            Login here using your account information!<br />
            <?php } ?>
        </td>
    </tr>
    <tr>
        <td align="right" width="50%">Username:</td>
        <td align="left"><input type="text" class="text" name="username" size="11" maxlength="40" /></td>
    </tr>
    <tr>
        <td align="right" width="50%">Password:</td>
        <td align="left"><input type="password" class="text" name="password" size="11" maxlength="32" /></td>
    </tr>
    <tr>
        <td align="center" colspan="2" width="100%">
            <?php if ($GLOBALS['CONFIG']['cookielife'] > 0) { ?>
            <input type="checkbox" class="checkbox" name="remember" value="1" /> Remember Me<br />
            <span style="font-size: 9px;">(expires in
                <?php
                    if ($GLOBALS['CONFIG']['cookielife'] == 1) {
                        print $GLOBALS['CONFIG']['cookielife'] . ' day';
                    } else {
                        print $GLOBALS['CONFIG']['cookielife'] . ' days';
                    } ?>
                )</span>
            <?php } ?>
        </td>
    </tr>
    <tr>
        <td align="center" colspan="2" width="100%">
            <br /><input type="submit" class="submit" name="submit" value="Login" /><br /><br />
        </td>
    </tr>
    <tr>
        <td class="infoBarBody" colspan="2" width="100%">&nbsp;</td>
    </tr>
    </form>
</table>

<script language="JavaScript" type="text/javascript">
<!--
document.login.username.focus();
//-->
</script>


<?php } else if ($themeError != '') { ?>
<table cellspacing="0" cellpadding="4" width="100%" style="border: solid 1px #666666;">
    <tr>
        <td class="infoBarBody" align="left" style="padding: 5px 5px 5px 5px;">
            <span class="notifyTitle">VirtualClassic Theme Does Not Exist</span>
        </td>
    </tr>
    <tr class="boldtopicsmall">
        <td>&nbsp;</td>
    </tr>
    <tr>
        <td class="boldBody" align="center"><?php print $themeError; ?></td>
    </tr>
    <tr>
        <td class="infoBarBody" height="20" width="100%">&nbsp;</td>
    </tr>
</table>

<?php } else if ($themeError == '') { ?>
<table cellspacing="0" cellpadding="4" width="100%" style="border: solid 1px #666666;">
    <tr>
        <td class="infoBarBody" align="left" style="padding: 5px 5px 5px 5px;">
            <span class="notifyTitle">Re-Create vadv-php_themes.dat File</span>
        </td>
    </tr>
    <tr class="boldtopicsmall">
        <td>&nbsp;</td>
    </tr>
    <tr>
        <form method="post" action="<?php print $rootDir; ?>themes/recreate_theme_dat.php">
        <td class="boldBody" align="center" style="padding-left: 75px;padding-right: 75px;">
            <br />
            <span style="color: red; ">Are you sure you want to run this script?</span><br /><br />
            This cannot be undone!
            <br /><br />
            This will overwrite your existing vadv-php_themes.dat file with fresh information from the vadvtheme.cfg located in the /themes/VirtualClassic directory.<br /><br />
            <input type="submit" class="submit" name="submit" value="RECREATE" />
        </td>
        </form>
    </tr>
    <tr>
        <td class="infoBarBody" height="20" width="100%">&nbsp;</td>
    </tr>
</table>
<?php }
//=====================================================================================================

//=====================================================================================================
// Output footer
$totalTime = round(ProgramTime() - $startTime, 4); ?>
<hr />
<table cellpadding="0" cellspacing="0" width="100%">
    <tr>
        <td width="100%" align="center" valign="top">
            <a href="http://www.vadvphp.com">
            <img border="0" src="<?php print $rootDir; ?>images/poweredby_vadv-php.png" alt="VADV-PHP" title="VADV-PHP" width="100" height="25" align="middle" />
            </a><br />
            Execution Time: <?php print $totalTime; ?> seconds<br /><br />
            <span class="copyright">
            VADV-PHP Copyright &#169; 2002-<?php print date('Y'); ?> Steve Winn, Aspect Technologies. All Rights Reserved.<br /> Virtual Advanced Copyright &#169; 1995-1997 Roland De Graaf.<br />
            </span>
            <span class="footerVersion">v<?php print $VERSION; ?></span>
            </td>
        </tr>
</table>
</div> <!-- page -->

</div> <!-- main -->
</div> <!-- mainframe -->

</body>
</html>
<?php
//=====================================================================================================
?>