<?php
/*
 * ----------------------------------------------------------------------------
 * Upload.php                                                        12/22/2009
 * This script will allow a web user to upload a file from their computer to
 * the BBS. Depending on the BBS configuration, the file is either placed in
 * a holding directory or is immediately added to the current database.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2009 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

// Useful if a user hits the back button.
session_cache_limiter('nocache');

$GLOBALS['VA_ACTION'] = 'Uploading a File';

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');

$classDatabases = new vadvDatabases;

// Turn off buffering.
ob_end_clean();

if ((!isset($_POST['referer'])) && (!isset($_GET['referer']))) {
    die('<br /><h4>Invalid message parameters. Please press the BACK button on your browser.</h4>');
}
if (isset($_GET['referer'])) {
    $_POST['referer'] = urldecode($_GET['referer']);
}

// Check the passed parameters. This script will look directly at the database
// to determine the filename instead of passing the filename and path directly.
// Set the passed variables.
// Topic.
if (isset($_POST['t'])) {
    $t = htmlspecialchars($_POST['t']);
} else {
    die('<br /><h4>Invalid message parameters. Please press the BACK button on your browser.</h4>');
}

// Database.
if (isset($_POST['d'])) {
    $d = (integer) $_POST['d'];
} else {
    die('<br /><h4>Invalid message parameters. Please press the BACK button on your browser.</h4>');
}

// Check to see if this page was referred by another page, and check to see where
// that page was at.
if ((!isset($_SERVER['HTTP_REFERER'])) ||
    (strpos($_SERVER['HTTP_REFERER'], $_SERVER['HTTP_HOST']) === FALSE)) {
    die('<br /><h4>You are not allowed to access this script from an outside source.<br />Press the BACK button on your browser.</h4>');
}

// Check user security levels, secure upload, and if uploads are even enabled.
if ((($GLOBALS['USER']['securitylevel'] < 255) &&
     ($GLOBALS['CONFIG']['fileupload'] != 1)) ||
    (trim(ini_get('file_uploads')) == '')) {
    die('<br /><h4>This server does not allow uploads. Please press the BACK button on your browser.</h4>');
} else if (!is_dir(ini_get('upload_tmp_dir'))) {
    die('<br /><h4>This server supports uploads, but there is a configuration error. Please press the BACK button on your browser.</h4>');
} else if (($GLOBALS['USER']['securitylevel'] < 255) &&
           ((($GLOBALS['USER']['userno'] == 0) ||
             ($GLOBALS['USER']['userno'] == $GLOBALS['CONFIG']['guestacct']) ||
             ($GLOBALS['USER']['securitylevel'] < $GLOBALS['CONFIG']['fileuploadsl'])) ||
            (($GLOBALS['VA_MAIN']['secuploads'] == 0) &&
             ($GLOBALS['USER']['securitylevel'] < $db['minwritesl'])))) {
    die('<br /><h4>You do not have access to upload files. Please press the BACK button on your browser.</h4>');
}

// Retrieve database information.
$db = $classDatabases->GetDbInfo($t, $d);

// Check to see if the file was already uploaded, if so then process it.
if ((isset($_POST['uploadflag'])) && ($_POST['uploadflag'] == 1)) {
    if (isset($_POST['desc'])) {
        $description = trim($_POST['desc']);
    } else {
        $description = 'NO DESCRIPTION';
    }

    if (isset($_POST['longdesc'])) {
        $longDescription = trim($_POST['longdesc']);
    } else {
        $longDescription = '';
    }

    $filename = '';
    if ((isset($_FILES['file'])) && ($_FILES['file']['size'] > 0)) {
        if ($_FILES['file']['error'] == 'UPLOAD_ERR_OK') {
            if (is_uploaded_file($_FILES['file']['tmp_name'])) {
                // If secure upload, put file in sysop dir and add to upload database.
                if ($GLOBALS['VA_MAIN']['secuploads'] == 1 && $GLOBALS['USER']['securitylevel'] < 255) {
                    // Get database information.
                    $dbInfo = $classDatabases->SetUploadInfo();
                } else {
                    // If not secure upload, put file in actual dir and database.
                    // Get database information.
                    $dbInfo = $db;
                }
                if ($GLOBALS['CONFIG']['filehost'] == '') {
                    $fileDirectory = $dbInfo['filepath'];
                } else {
                    $fileDirectory = ConvertDir($GLOBALS['CONFIG']['filehost'], $dbInfo['filepath']);
                }

                $filename = $_FILES['file']['name'];
                $shortFilename = ConvertLongFilename($filename, $fileDirectory);

                $filePath = $fileDirectory . '/' . $filename;
                $filePath = str_replace('/', "\\", $filePath);
                $shortFilePath = $fileDirectory . '/' . $shortFilename;
                $shortFilePath = str_replace('/', "\\", $shortFilePath);
                if (!is_dir($fileDirectory)) {
                    mkdir($fileDirectory);
                }
                move_uploaded_file($_FILES['file']['tmp_name'], $filePath);

                // Add any information to the message itself.
                if ($filename != $_FILES['file']['name']) {
                    $longDescription = rtrim($longDescription) . "\r\n\r\n\r\n--------------";
                    $longDescription = $longDescription . "\r\nOriginal filename: " . $_FILES['file']['name'] . "\r\nThe filename was changed for compatibility reasons.";
                }

                $log = 'File Upload     "' . $GLOBALS['USER']['handle'] . '" --> "' . $filename . '" - ' . $description;

                // Log the email.
                WriteLog('VADV-PHP.LOG', $log);

                // Add new record to the database.
                $classDatabases->WriteFile($dbInfo, $GLOBALS['USER']['userno'], $description, $longDescription, $shortFilePath);

                // Redirect the user.
                RedirectUser($_POST['referer']);
            }
        } else {
            switch ($_FILES['file']['error']) {
                case 'UPLOAD_ERR_FORM_SIZE':
                    unlink($_FILES['file']['tmp_name']);
                    die('<br /><h4>Attached file is too big (>' . ini_get('upload_max_filesize') . '). Please press the BACK button on your browser.</h4>');
                    break;
                case 'UPLOAD_ERR_PARTIAL':
                    unlink($_FILES['file']['tmp_name']);
                    die('<br /><h4>Attached file upload was interrupted. Please press the BACK button on your browser.</h4>');
                    break;
                case 'UPLOAD_ERR_NO_FILE':
                    die('<br /><h4>Invalid attachment. Please press the BACK button on your browser.</h4>');
                    break;
                default:
                    die('<br /><h4>An error with the file attachment occurred. Please press the BACK button on your browser.</h4>');
                    break;
            }
        }
    }
}


// Include javascript for the upload form validation
$hdr_include .= '<script language="JavaScript" type="text/javascript">' . "\n";
$hdr_include .= '<!--' . "\n";
$hdr_include .= '    var submitted = 0;' . "\n";
$hdr_include .= '    function validate(form) {' . "\n";
$hdr_include .= '        if (submitted == 0) {' . "\n";
$hdr_include .= '            if ((form.file.value.length == 0) ||' . "\n";
$hdr_include .= '                (form.desc.value.length == 0)) {' . "\n";
$hdr_include .= '                alert("Please enter a valid filename and description.");' . "\n";
$hdr_include .= '                form.file.focus();' . "\n";
$hdr_include .= '                return false;' . "\n";
$hdr_include .= '            }' . "\n";
$hdr_include .= '            submitted = 1;' . "\n";
$hdr_include .= '            form.submit.disabled = true;' . "\n";
$hdr_include .= '            form.cancel.disabled = true;' . "\n";
$hdr_include .= '            return true;' . "\n";
$hdr_include .= '        }' . "\n";
$hdr_include .= '    }' . "\n\n";

$hdr_include .= '    function HelpWindow(hpath, bmark) {' . "\n";
$hdr_include .= '        var url = hpath + "#" + bmark;' . "\n";
$hdr_include .= '        var newwindow = window.open(url, "helpwindow", "toolbar=no, location=no, directories=no, status=no, menubar=no, scrollbars=yes, resizable=no, width=350, height=450");' . "\n";
$hdr_include .= '        newwindow.focus();' . "\n";
$hdr_include .= '    }' . "\n";
$hdr_include .= '//-->' . "\n";
$hdr_include .= '</script>' . "\n";




// Output header
$pagetitle = 'File Upload';
include($themepath . '/templates/header.php');



$maxsize = ini_get('upload_max_filesize');
$megpos = strpos(strtoupper($maxsize), 'M');
if ($megpos !== FALSE) {
    $maxsize = (integer) $maxsize;
    $maxsize = 1024 * 1024 * $maxsize;
}



// Output page
include($themepath . '/templates/fileupload.php');

// Output footer
include($themepath . '/templates/footer.php');



?>