<?php
/*
 * ----------------------------------------------------------------------------
 * SystemLogin.php                                                   05/05/2013
 * This script will allow a user logged in as the SysOp to login with the
 * system password for added protection when using the VConfig scripts.
 * ----------------------------------------------------------------------------
 * Copyright (C) 2002-2013 Steve Winn. All Rights Reserved.
 *
 * For licensing information, view LICENSE.TXT.
 * ----------------------------------------------------------------------------
 */

$GLOBALS['VA_ACTION'] = 'SysOp Function';

require_once('include_dir.php');
require_once($includeDir . 'vadv-php.inc.php');

if (($GLOBALS['USER']['userno'] == 0) ||
    ($GLOBALS['USER']['userno'] == $GLOBALS['CONFIG']['guestacct']) ||
    ($GLOBALS['USER']['securitylevel'] == 0)) {
    RedirectUser($rootDir . 'index.php');
}
if ($GLOBALS['USER']['securitylevel'] < 255) {
    $log = 'VConfig Error!  "' . $GLOBALS['USER']['handle'] . '" - Attempted Unauthorized VConfig Entry';
    WriteLog('VADV-PHP.LOG', $log);
    RedirectUser($rootDir . 'index.php');
}


// Load Who's Online list
$classControlDat = new datControl;
$who = $classControlDat->Get();

$maxChannel = 10;    // Display only first 10 channels.
if ($GLOBALS['VA_MAIN']['maxchannelno'] < $maxChannel) {
    $maxChannel = $GLOBALS['VA_MAIN']['maxchannelno'];
}
if ($who['nodetotal'] < $maxChannel) {
    $maxChannel = $who['nodetotal'];
}

// Get number of guests...
$totalGuests = 0;
for ($i = 1; $i <= $GLOBALS['VA_USERS'][0]; $i++) {
    if (($GLOBALS['VA_USERS'][$i]['userno'] == 0) || ($GLOBALS['VA_USERS'][$i]['userno'] == $GLOBALS['CONFIG']['guestacct'])) {
        $totalGuests++;
    }
}
$totalUsers = $GLOBALS['VA_USERS'][0] - $totalGuests;



// Process the form if already submitted...
if (((isset($_POST['password'])) &&
    (trim($_POST['password']) != '')) ||
    ((isset($_POST['enc'])) &&
     (trim($_POST['enc']) != ''))) {
    if (!isset($_POST['referer'])) {
        die('<br /><h4>An error occurred. Please press the BACK button on your browser.</h4>');
    }

    // Encryption authentication.
    if ((isset($_POST['enc'])) &&
        (trim($_POST['enc']) != '')) {
        // Remove identifier.
        $encryption = substr(trim($_POST['enc']), 5);
        $loginId = '';
        // Get user id.
        for ($i = 1; $i <= $GLOBALS['VA_USERS'][0]; $i++) {
            if ($GLOBALS['VA_USERS'][$i]['session'] == session_id()) {
                $loginId = $GLOBALS['VA_USERS'][$i]['id'];
                break;
            }
        }
        // Reset the id.
        UpdateWhosOnline('Logging In...', 900, 1);
        if (hash_hmac($GLOBALS['CONFIG']['encryption'], $GLOBALS['VA_MAIN']['systempw'], $loginId) === $encryption) {
            // Set password as correct.
            $password = $GLOBALS['VA_MAIN']['systempw'];
        } else {
            $password = '';
        }
    } else {
        $password = strtoupper(trim(htmlspecialchars($_POST['password'])));
        if (strlen($password) > 16) {
            $password = substr($password, 0, 16);
        }
        $password = hash($GLOBALS['CONFIG']['encryption'], $password);
    }

    if ($password === $GLOBALS['VA_MAIN']['systempw']) {
        $_SESSION['systempw'] = $password;
        if (isset($_POST['url'])) {
            RedirectUser($_POST['url']);
        } else {
            RedirectUser($rootDir . 'index.php');
        }
    } else {
        $_SESSION = array('');
        session_destroy();
        RedirectUser($rootDir . 'index.php');
    }
}

// Get login id for this session...
$loginId = '';
for ($i = 1; $i <= $GLOBALS['VA_USERS'][0]; $i++) {
    if ($GLOBALS['VA_USERS'][$i]['session'] == session_id()) {
        $loginId = $GLOBALS['VA_USERS'][$i]['id'];
        break;
    }
}


switch ($GLOBALS['CONFIG']['encryption']) {
    case 'md5':
        $hdr_include .= '<script language="JavaScript" type="text/javascript" src="' . $includeDir . 'javascript/md5.js"></script>' . "\n";
        break;
    case 'sha1':
        $hdr_include .= '<script language="JavaScript" type="text/javascript" src="' . $includeDir . 'javascript/sha1.js"></script>' . "\n";
        break;
    case 'sha256':
        $hdr_include .= '<script language="JavaScript" type="text/javascript" src="' . $includeDir . 'javascript/sha256.js"></script>' . "\n\n";
        break;
    default:
        break;
}

$hdr_include .= '<script language="JavaScript" type="text/javascript">' . "\n";
$hdr_include .= '<!--' . "\n";
$hdr_include .= '    var submitted = 0;' . "\n";
$hdr_include .= '    function validate(form) {' . "\n";
$hdr_include .= '        if (submitted == 0) {' . "\n";
$hdr_include .= '            form.enc.value = form.password.value.toUpperCase();' . "\n";

switch ($GLOBALS['CONFIG']['encryption']) {
    case 'md5':
        $hdr_include .= 'form.enc.value = hex_md5(form.enc.value);' . "\n";
        $hdr_include .= 'form.enc.value = "|enc|" + hex_hmac_md5(form.id.value, form.enc.value);' . "\n";
        break;
    case 'sha1':
        $hdr_include .= 'form.enc.value = hex_sha1(form.enc.value);' . "\n";
        $hdr_include .= 'form.enc.value = "|enc|" + hex_hmac_sha1(form.id.value, form.enc.value);' . "\n";
        break;
    case 'sha256':
        //$hdr_include .= 'form.enc.value = SHA256_hash(form.enc.value);' . "\n";
        //$hdr_include .= 'form.enc.value = "|enc|" + HMAC_SHA256_MAC(form.id.value, form.enc.value);' . "\n";
        $hdr_include .= 'form.enc.value = hex_sha256(form.enc.value);' . "\n";
        $hdr_include .= 'form.enc.value = "|enc|" + hex_hmac_sha256(form.id.value, form.enc.value);' . "\n";
        break;
    default:
        break;
}

$hdr_include .= '            form.password.value = "";' . "\n";
$hdr_include .= '            form.id.value = "";' . "\n";
$hdr_include .= '            submitted = 1;' . "\n";
$hdr_include .= '            return true;' . "\n";
$hdr_include .= '        } else {' . "\n";
$hdr_include .= '            return false;' . "\n";
$hdr_include .= '        }' . "\n";
$hdr_include .= '    }' . "\n";
$hdr_include .= '//-->' . "\n";
$hdr_include .= '</script>' . "\n\n";

// Output header
include($themepath . '/templates/admin/admin_header.php');





// Output page
include($themepath . '/templates/admin/admin_syslogin.php');

// Output footer
include($themepath . '/templates/admin/admin_footer.php');

?>